#<profile4xi profile = "12" />
# =================================================================================================
#<program4xi program = "hpu2image4xi.py">
#<abstract4xi abstract = "HPU 2D image management"/>
#<refer4xi reference = "image2d4xi.py"/>
#<verify4xi verification = "10" target = "hpu2image4xi5test.py">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting; [Rel 7.21]"/>
# ================================================================================================= 

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'HPU2Image4Xi',                        # ACK 5.31  p76 
            
            'copy2image4hpu',                      # ACK 5.3.3 p88
            'copy2imagetobuffer4hpu',              # ACK 5.3.4 p90
            'copy2buffertoimage4hpu',              # ACK 5.3.4 p92
            'create2image4hpu',                    # ACK 5.31  p76            

            'get2imageinfo4hpu',                   # ACK 5.3.6 p97 nb: modernized so that you can also access only selected elements, if you want           
            'get2supportedimageformats4hpu',       # ACK 5.3.2 p82 nb: a separate function to get format whereas using get2imageinfo4hpu you can access any element by key such as 'FORMAT' 
                              
            'manoeuvre2copyimage4hpu',             # ACK modernized 
            'manoeuvre2copyimagetobuffer4hpu',     # ACK modernized        
            'manoeuvre2copybuffertoimage4hpu',     # ACK modernized
            'manoeuvre2mapimage4hpu',              # ACK modernized                                           
            'manoeuvre2readimage4hpu',             # ACK modernized
            'manoeuvre2writeimage4hpu',            # ACK modernized            
            'map2image4hpu',                       # ACK 5.3.5 p94 
            
            'read2image4hpu',                      # ACK 5.3.3 p84  
            'retain2image4hpu',                    # ACK 5.4.1 p99
            'release2image4hpu',                   # ACK 5.4.1 p99

            'set2imagecallback4hpu',               # ACK modernized
            'set2image4hpu',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    

            'trace2image4hpu',                     # ACK modernized 
                                    
            'wait2image4hpu',                      # ACK modernized nb: checks Queue object and thereby causes delay doing something useful
            'write2image4hpu'                      # ACK 5.3.3 p84                 
          ] 
#"
#/>


# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os
from platform import processor
import sys
import threading                
import _thread   
from decimal import *
#"
#/>


# -------------------------------------------------------------------------------------------------
# Scope4Xi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
import version4xi

# -------------------------------------------------------------------------------------------------
# Buffer module                           
# -------------------------------------------------------------------------------------------------
from hpu2buffer4xi import HPU2Buffer4Xi
from hpu2buffer4xi import create2buffer4hpu
from hpu2buffer4xi import create2subbuffer4hpu
from hpu2buffer4xi import get2bufferinfo4hpu
from hpu2buffer4xi import release2buffer4hpu
from hpu2buffer4xi import retain2buffer4hpu
from hpu2buffer4xi import set2buffer4hpu
from hpu2buffer4xi import trace2buffer4hpu

# -------------------------------------------------------------------------------------------------
# Command queue module                           
# -------------------------------------------------------------------------------------------------
from hpu2commandqueue4xi import HPU2CommandQueue4Xi
from hpu2commandqueue4xi import create2commandqueue4hpu 
from hpu2commandqueue4xi import flush2commandqueue4hpu
from hpu2commandqueue4xi import finish2commandqueue4hpu
from hpu2commandqueue4xi import get2commandqueueinfo4hpu
from hpu2commandqueue4xi import release2commandqueue4hpu
from hpu2commandqueue4xi import retain2commandqueue4hpu
from hpu2commandqueue4xi import set2commandqueue4hpu
from hpu2commandqueue4xi import trace2commandqueue4hpu

# -------------------------------------------------------------------------------------------------
# Context module                           
# -------------------------------------------------------------------------------------------------
from hpu2context4xi import HPU2Context4Xi 
from hpu2context4xi import create2context4hpu
from hpu2context4xi import get2context4hpu
from hpu2context4xi import release2context4hpu
from hpu2context4xi import retain2context4hpu
from hpu2context4xi import set2context4hpu


# -------------------------------------------------------------------------------------------------
# Device module                        
# -------------------------------------------------------------------------------------------------
from hpu2device4xi import HPU4Xi
from hpu2device4xi import init2hpu4xi 
from hpu2device4xi import set2hpu4xi
from hpu2device4xi import get2hpu4xi

# -------------------------------------------------------------------------------------------------
# Event module                           
# -------------------------------------------------------------------------------------------------
from hpu2event4xi import HPU2Event4Xi
from hpu2event4xi import create2event4hpu
from hpu2event4xi import get2eventinfo4hpu
from hpu2event4xi import release2event4hpu
from hpu2event4xi import retain2event4hpu
from hpu2event4xi import set2eventcallback4hpu 
from hpu2event4xi import set2event4hpu
from hpu2event4xi import trace2event4hpu
from hpu2event4xi import wait2event4hpu


# -------------------------------------------------------------------------------------------------
# Platform module                           
# -------------------------------------------------------------------------------------------------
from platformids4xi  import get2platformids4xi
from platforminfo4xi import PlatformInfo4Xi                                                                        
from platforminfo4xi import init2platforminfo4xi
from platforminfo4xi import set2platforminfo4xi
from platforminfo4xi import get2platforminfo4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from arena4xi import Arena4Xi
from cpu_corecount4xi import cpu_corecount4xi
from dummyprocess4xi import DummyProcess4Xi
from print4xi import print4xi
from pipe4xi import Pipe4Xi
from queue4xi import Queue4Xi  
from time4xi import time4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi


#"
#/>

# -------------------------------------------------------------------------------------------------
# Global constants                            
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global hpu2image4xi                                                     # 2D-IMAGES FOR HPU

global hpu2trace4xi                                                     # TRACE

global hpu2buffer4xi                                                    # BUFFER FOR HPU

global control2readimage4hpu                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH HPU  
global state2read4hpu                                                   # STATE FOR COMMAND AND CONTROL
global max2readstate4hpu                                                # MAX FOR STATE
global ack2readinfo4hpu                                                 # ACK FOR IMAGE INFO
global message2readget4hpu                                              # QUEUE GET
global message2readput4hpu                                              # QUEUE PUT
global message2readreceive4hpu                                          # PIPE RECEIVE
global message2readsend4hpu                                             # PIPE SEND

global control2writeimage4hpu                                           # COMMAND AND CONTROL TO WRITE 2D IMAGES WITH HPU 
global state2write4hpu                                                  # STATE FOR COMMAND AND CONTROL
global max2writestate4hpu                                               # MAX FOR STATE
global ack2writeinfo4hpu                                                # ACK FOR IMAGE INFO
global message2writeget4hpu                                             # QUEUE GET
global message2writeput4hpu                                             # QUEUE PUT
global message2writereceive4hpu                                         # PIPE RECEIVE
global message2writesend4hpu                                            # PIPE SEND

global control2copyimage4hpu                                            # COMMAND AND CONTROL TO COPY 2D IMAGES WITH HPU 
global state2copyimage4hpu                                              # STATE FOR COMMAND AND CONTROL
global max2copyimagestate4hpu                                           # MAX FOR STATE
global ack2copyimageinfo4hpu                                            # ACK FOR IMAGE INFO
global message2copyimageget4hpu                                         # QUEUE GET
global message2copyimageput4hpu                                         # QUEUE PUT
global message2copyimagereceive4hpu                                     # PIPE RECEIVE
global message2copyimagesend4hpu                                        # PIPE SEND

global control2copyimagetobuffer4hpu                                    # COMMAND AND CONTROL TO COPY 2D IMAGES TO BUFFER WITH HPU 
global state2copyimagetobuffer4hpu                                      # STATE FOR COMMAND AND CONTROL
global max2copyimagetobufferstate4hpu                                   # MAX FOR STATE
global ack2copyimagetobufferinfo4hpu                                    # ACK FOR IMAGE INFO
global message2copyimagetobufferget4hpu                                 # QUEUE GET
global message2copyimagetobufferput4hpu                                 # QUEUE PUT
global message2copyimagetobufferreceive4hpu                             # PIPE RECEIVE
global message2copyimagetobuffersend4hpu                                # PIPE SEND

global control2copybuffertoimage4hpu                                    # COMMAND AND CONTROL TO COPY BUFFER TO 2D IMAGE WITH HPU 
global state2copybuffertoimage4hpu                                      # STATE FOR COMMAND AND CONTROL
global max2copybuffertoimagestate4hpu                                   # MAX FOR STATE
global ack2copybuffertoimageinfo4hpu                                    # ACK FOR IMAGE INFO
global message2copybuffertoimageget4hpu                                 # QUEUE GET
global message2copybuffertoimageput4hpu                                 # QUEUE PUT
global message2copybuffertoimagereceive4hpu                             # PIPE RECEIVE
global message2copybuffertoimagesend4hpu                                # PIPE SEND                      

global control2mapimage4hpu                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH HPU  
global state2map4hpu                                                   # STATE FOR COMMAND AND CONTROL
global max2mapstate4hpu                                                # MAX FOR STATE
global ack2mapinfo4hpu                                                 # ACK FOR IMAGE INFO
global message2mapget4hpu                                              # QUEUE GET
global message2mapput4hpu                                              # QUEUE PUT
global message2mapreceive4hpu                                          # PIPE RECEIVE
global message2mapsend4hpu                                             # PIPE SEND
            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "

#"
#"/>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "trace2image4hpu" usage = "
#
#   print('=' *33)          
#   print('(Step 15A/25) OF HPU2IMAGE4XI5TEST - TRYING TO TRACE HPU: ', time4xi())      
#   print('=' *33)      
#   trace2hpu = None
#   try:
#       trace2hpu = trace2image4hpu() 
#       print('(Step 15B/25) OF HPU2IMAGE4XI5TEST - TRACE2HPU: ', trace2hpu)  
#       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4hpu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4hpu', 'METHOD': 'trace2image4hpu'}
#   except:
#       print('(Step 15Z1/25) OF HPU2IMAGE4XI5TEST *** FAILED TO TRACE HPU: ', trace2hpu)       
#">

# -------------------------------------------------------------------------------------------------   
def trace2image4hpu(task4trace2hpu=None):  

    '''
    creates callback to hpu2image4xi
    '''
    
    global hpu2trace4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "trace2image4hpu" 
    method2print = "trace2image4hpu"

    message2print = "{(Step 1A/33) started" + "; trace2image4hpu-started: " + str(trace2image4hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # <function trace2image4hpu at 0x0000000003AF0348> 
  
             
    message2print = "{(Step 1B/33) passed" + "; task4trace2hpu-to-trace2image4hpu: " + str(task4trace2hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              # 
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    hpu2trace4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
      
    if task4trace2hpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY HPUIZED CALLBACK !
        # -----------------------------------------------------------------------------------------    
        task4trace2hpu = {}                                                                                            
            
        task4trace2hpu['CLASS']   = "trace2image4hpu"                                                               # HPUIZE ZPU TRACE!
        task4trace2hpu['METHOD']  = "trace2image4hpu" 
        task4trace2hpu['MESSAGE'] = "{(Step 2Z0/33) failed" + "; trace2image4hpu-failed " + str(hpu2trace4xi)  + "; time: " + time4xi() + "}"               
  
        
        message2print = "{(Step 2A/33) set" + "; task4trace2hpu-set-by-trace2image4hpu to: " + str(task4trace2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4hpu-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4hpu',
        #  'METHOD': 'trace2image4hpu'}
        #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}
 
    
        code2ack = code2ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # PARAMETER GIVEN BY HPUIZED CALLBACK !
    # -----------------------------------------------------------------------------------------          
    message2print = "{(Step 3A/33) try" + "; task4trace2hpu-try-by-trace2image4hpu: " + str(task4trace2hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)    
    #  { trace2image4hpu : trace2image4hpu ; (Step 3A/33) try; task4trace2hpu-try-by-trace2image4hpu: 
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4hpu-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'trace2image4hpu',
    #  'METHOD': 'trace2image4hpu'}
    #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}

    try:
        hpu2trace4xi ['CLASS']  = task4trace2hpu['CLASS']                                                      # HPUIZE HPU TRACE!
        hpu2trace4xi ['METHOD']  = task4trace2hpu['METHOD']
        hpu2trace4xi ['MESSAGE'] = task4trace2hpu['MESSAGE']
        
        message2print = "{(Step 3B/33) set" + "; hpu2trace4xi-set-by-trace2image4hpu to: " + str(hpu2trace4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4hpu : trace2image4hpu ; (Step 3B/33) set; hpu2trace4xi-set-by-trace2image4hpu to: 
        #                     {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4hpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}',
        #                      'CLASS': 'trace2image4hpu',
        #                      'METHOD': 'trace2image4hpu'}
        #                     ; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3B-'         
    except:
        message2print = "{(Step 3Z1/33) failed" + "; hpu-trace-failed-by-trace2image4hpu: " + str(task4trace2hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z1-' 
        
    try:
        # =================================
        # SIMULATE HPU TRACE
        # =================================
        class2print   = task4trace2hpu['CLASS']                                                                # HPUIZE HPU TRACE!
        method2print  = task4trace2hpu['METHOD']
        
        message2print = task4trace2hpu['MESSAGE']
        print4xi(class2print, method2print, message2print)                                                     # HPUIZED TRACE FOR OPENCL CALLBACK!
        # { trace2image4hpu : trace2image4hpu ; (Step 2Z0/33) failed; trace2image4hpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}
        
        code2ack = code2ack + '3C-'         
    except:
        class2print = "trace2image4hpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4hpu"   
        message2print = "{(Step 3Z2/33) failed" + "; simulation-of-hpu-trace-failed-by-trace2image4hpu: " + str(task4trace2hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z2-' 
        
    try:
        # =================================
        # NORMAL TRACE
        # =================================                     
        class2print = "trace2image4hpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4hpu"                
        message2print = "{(Step 3D/33) performed" + "; hpu-trace-performed-by-trace2image4hpu: " + str(task4trace2hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4hpu : trace2image4hpu ; (Step 3D/33) performed; hpu-trace-performed-by-trace2image4hpu: 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4hpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
        # 'CLASS': 'trace2image4hpu', 'METHOD': 'trace2image4hpu'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3D-' 
    except:
        message2print = "{(Step 3Z3/33) failed" + "; hpu-trace-failed-by-trace2image4hpu: " + str(task4trace2hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z3-' 

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; hpu2trace4xi-returning-by-trace2image4hpu: " + str(hpu2trace4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4hpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
    #  'CLASS': 'trace2image4hpu', 
    #  'METHOD': 'trace2image4hpu'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

    
    return hpu2trace4xi
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2imagecallback4hpu" usage = "
#
#   print('(Step 14A/700) OF HPU2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE CALLBACK OF HPU DEVICE ', time4xi())  
#   try:                               
#       response4setcallback2image = set2imagecallback4hpu()                                                                                                 
#       print('(Step 14B/700) OF HPU2IMAGE4XI5TEST - SET 2D IMAGE CALLBACK OF HPU DEVICE TO: ', response4setcallback2image)   
#       # <function trace2image4hpu at 0x0000000004E2CAC8>
#   except:
#       print('(Step 14Z1/700) OF HPU2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE CALLBACK OF HPU DEVICE ', time4xi())  
# 
#">

# ------------------------------------------------------------------------------------------------- 
 
def set2imagecallback4hpu(task4set2imagecallback=None):

    '''
    registers callback function to hpu2image4xi
    '''
    global hpu2image4xi
    
    class2print = "set2imagecallback4hpu" 
    method2print = "set2imagecallback4hpu"
    
    message2print = "{(Step 1A/70) started" + "; set2imagecallback4hpu-started: " + str(set2imagecallback4hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function set2imagecallback4hpu at 0x000000000390B748>
    
    try:
    
        hpu2image4xi ['CALLBACK'] = trace2image4hpu
        
        message2print = "{(Step 1B/70) set callback2hpu" + "; imagecallback4hpu-set-by-set2imagecallback4hpu: " + str(hpu2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4hpu at 0x0000000003A9E348> 
        
        message2print = "{(Step 1C/70) returning" + "; callback-of-hpu2image4xi-returning-by-set2imagecallback4hpu: " + str(hpu2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4hpu at 0x0000000003864A48> 
        
        return hpu2image4xi ['CALLBACK']      
        
    except:
        message2print = "{(Step 1Z1/70) failed" + "; callback2image-failed-by-set2imagecallback4hpu: " + str(callback2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         

        message2print = "{(Step 1Z2/70) returning" + "; trace2image4hpu-returning-by-set2imagecallback4hpu: " + str(trace2image4hpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)      
            
        return trace2image4hpu
            
#</function4xi> 




# -------------------------------------------------------------------------------------------------
#<function4xi function = "create2image4hpu" usage = "
#   #1     
#   print('=' *33)          
#   print('(Step 2A/700) OF HPU2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE4HPU ', time4xi())      
#   print('=' *33)        
#   # 
#   try:
#       response4create2Dimage = create2image4hpu()                                                                 
#       print('(Step 2B/700) OF HPU2IMAGE4XI5TEST - CREATED 2D IMAGE4HPU ', time4xi())
#       keys2response = response4create2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 2Z1/700) OF HPU2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE4HPU *** ', time4xi())
#   #2
#   print('(Step 123A/700) OF HPU2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE for HPU ', time4xi()) 
#   # 
#   try:
#       xi2hpu = HPU2Image4Xi()                            
#       response4create2Dimage = xi2hpu.create2image()                                  
#       keys2response = response4create2Dimage.keys()
#       print('(Step 122B/700) OF HPU2IMAGE4XI5TEST - CREATED 2D IMAGE ', time4xi())  
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 123Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE for HPU ', time4xi())    
#">

# -------------------------------------------------------------------------------------------------   
def create2image4hpu(task4imagecreate2hpu=None):

    '''
    creates 2D image for HPU
    '''
    
    global hpu2image4xi                                                                                              
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "create2image4hpu" 
    method2print = "create2image4hpu"

    message2print = "{(Step 1A/70) started" + "; create2image4hpu-started: " + str(create2image4hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function create2image4hpu at 0x00000000038D97C8> 
  
             
    message2print = "{(Step 1B/70) passed" + "; task4imagecreate2hpu-to-create2image4hpu: " + str(task4imagecreate2hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    hpu2image4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4imagecreate2hpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO HPU! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecreate2hpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4imagecreate2hpu-created-by-create2image4hpu to: " + str(task4imagecreate2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            function2callback4hpu = trace2image4hpu                                                                 # CALLBACK FUNCTION FOR HPU
            
            message2print = "{(Step 1D/70) initialized" + "; function2callback4hpu-created-by-create2image4hpu to: " + str(function2callback4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4hpu at 0x00000000038D9348>
            
            code2ack  = code2ack + '1D-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "create2image4hpu"                                                          # MODULE OF HPUIZED OPENCL MODULE
            args2callback['METHOD']  = "create2image4hpu"                                                          # FUNCTION OF HPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create2image4hpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR HPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args2callback-created-by-create2image4hpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4hpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4hpu',
            #  'METHOD': 'create2image4hpu'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

            code2ack  = code2ack + '1E-'
            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - EXTRA                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecreate2hpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO HPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecreate2hpu['FUNCTION2CALLBACK'] = function2callback4hpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4imagecreate2hpu-created-by-create2image4hpu to: " + str(task4imagecreate2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4hpu at 0x00000000038D9348>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4hpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4hpu',
            #  'METHOD': 'create2image4hpu'}}; time: 00:54:15 09/06/11 Pacific Daylight Time}
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - EXTRA                      
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-create2image4hpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-create2image4hpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 00:54:15 09/06/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-create2image4hpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecreate2hpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            context2hpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2hpu-trying-by-create2image4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2hpu = get2context4hpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2hpu-got-by-create2image4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4hpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2context4hpu',
                #  'METHOD': 'create2context4hpu'},
                #  'FUNCTION2CALLBACK': <function trace2context4hpu at 0x00000000037D0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0, 'PROPERTIES': None}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2hpu-failed-by-create2image4hpu: " + str(context2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecreate2hpu ['CONTEXT'] = context2hpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - EXTRA                            
            # ---------------------------------------------------------------------------------------------    
            device2hpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2hpu-trying-by-create2image4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2hpu = get2hpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2hpu-got-by-create2image4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318,
                #  'ID2PLATFORM': '0000000001509568',
                #  'PROFILING2QUEUE': 'ACK',
                #  'EXECUTE2KERNELS4OPENCL': 'ACK',
                #  'NATIVE2WIDTHLONG4VECTOR': 2,
                #  'MAX2UNITS4COMPUTE': 4,
                #  'SIZE2MEMORY4GLOBAL': 3221225472,
                #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
                #  'MAX2SIZE4WORKGROUP': 1024,
                #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
                #  'SIZE2CACHE': 65536,
                #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
                #  'TYPE4CACHE': 'Read/Write',
                #  'PREFERRED2WIDTHINT4VECTOR': 4,
                #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
                #  'NATIVE2WIDTHINT4VECTOR': 4,
                #  'SIZE2BUFFER4CONSTANT': 65536,
                #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
                #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
                #  'MAX2NUMBER4CONSTANTARGS': 8,
                #  'VERSION2DRIVER': '2.0',
                #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics 
              
                #  'AVAILABLE2COMPILER': 'ACK',
                #  'MAX2DIMENSIONS4WORKITEMS': 3,
                #  'SUPPORT2CORRECTION4ERROR': 0,
                #  'BITS2ADDRESS': 64,
                #  'AVAILABLE': 'ACK',
                #  'SIZE2MEMORY4LOCAL': 32768,
                #  'VENDOR': 'AuthenticAMD',
                #  'SUPPORT2IMAGE': 'NACK',
                #  'MAX2SIZE4KERNELARGUMENT': 4096,
                #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-
                #  'OUT-OF-ORDER2QUEUE': 'NACK',
                #  'ALIGNMENT2ADDRESS4BASE': 1024,
                #  'PROFILE': 'FULL_PROFILE',
                #  'TYPE2MEMORY4LOCAL': 'Global',
                #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
                #  'PREFERRED2WIDTHLONG4VECTOR': 2,
                #  'MAX2FREQUENCY4CLOCK': 3210,
                #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
                #  'ID': 4033,
                #  'MAX2WORKITEM2': 1024,
                #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
                #  'ROUND2ZERO': 'ACK',
                #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
                #  'MAX2WORKITEM1': 1024,
                #  'MAX2WORKITEM0': 1024,
                #  'SIZE2LINE4CACHE': 64,
                #  'MAX2ALLOCATION4MEMORY': 1073741824,
                #  'NATIVE2WIDTHCHAR4VECTOR': 16,
                #  'QUIET2NANS': 'ACK',
                #  'NATIVE2WIDTHSHORT4VECTOR': 8,
                #  'ROUND2NEAREST4EVEN': 'ACK',
                #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
                #  'NACK2RETURN': ' ',
                #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
                #  'ENDIANESS2DEVICE': 'Little',
                #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
                #  'ROUND2+VE3INFINITY': 'ACK',
                #  'TYPE': 'CL_DEVICE_TYPE_HPU',
                #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2hpu-failed-by-create2image4hpu: " + str(device2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
                
            task4imagecreate2hpu ['DEVICE'] = device2hpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) ERROR CODE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            task4imagecreate2hpu ['CODE2ERROR'] = 0             
            code2ack  = code2ack + '5B-'              

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - EXTRA                          
            # ---------------------------------------------------------------------------------------------         
            queue2hpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2hpu-trying-by-create2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2hpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2hpu-got-by-create2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003991B70> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2hpu-failed-by-create2image4hpu: " + str(queue2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagecreate2hpu ['QUEUE'] = queue2hpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - EXTRA                              
            # ---------------------------------------------------------------------------------------------         
            pipe2hpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2hpu-trying-by-create2image4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2hpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2hpu-got-by-create2image4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2hpu-failed-by-create2image4hpu: " + str(pipe2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecreate2hpu ['PIPE'] = pipe2hpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2hpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2hpu-trying-by-create2image4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2hpu = get2commandqueueinfo4hpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2hpu-got-by-create2image4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4hpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2commandqueue4hpu',
                #  'METHOD': 'create2commandqueue4hpu'},
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4hpu at 0x000000000372C848>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
                #  'INFO2PRIVATE': None,
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A7A6D8>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0 ... 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4hpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4hpu', 
                #  'METHOD': 'create2context4hpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4hpu at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2hpu-failed-by-create2image4hpu: " + str(commandqueueinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagecreate2hpu ['COMMANDQUEUE'] = commandqueueinfo2hpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2hpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2hpu-trying-by-create2image4hpu: " + str(bufferinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2hpu = get2bufferinfo4hpu()                                                                   # SYSTEM CREATES BUFFER FOR HPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2hpu-got-by-create2image4hpu: " + str(bufferinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4hpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2buffer4hpu',
                #                    'METHOD': 'create2buffer4hpu'},
                #  'FUNCTION2CALLBACK': <function trace2buffer4hpu at 0x0000000003B2B348>,
                #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003ADD2F0>,
                #  'FLAGS': 0,
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4hpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4hpu', 
                #  'METHOD': 'create2context4hpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4hpu at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2hpu-failed-by-create2image4hpu: " + str(bufferinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecreate2hpu ['BUFFER'] = bufferinfo2hpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            eventinfo2hpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2hpu-trying-by-create2image4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2hpu = get2eventinfo4hpu()                                                                     # SYSTEM CREATES EVENT IF NECESSAY   
                message2print = "{(Step 10B/70) got" + "; eventinfo2hpu-got-by-create2image4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4hpu-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2event4hpu',
                #                    'METHOD': 'create2event4hpu'},
                #  'FUNCTION2CALLBACK': <function trace2event4hpu at 0x0000000003747D48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A02898>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4hpu-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}', 
                #                                                      'CLASS': 'create2context4hpu', 'METHOD': 'create2context4hpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4hpu at 0x00000000036F0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0,
                #  'PROPERTIES': None},
                #  'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
                
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2hpu-failed-by-create2image4hpu: " + str(eventinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecreate2hpu ['EVENT'] = eventinfo2hpu             
            
            # ---------------------------------------------------------------------------------------------
            # (11) FLAGS - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            flags2hpu = 0
           
            message2print = "{(Step 11A/70) trying" + "; flags2hpu-trying-by-create2image4hpu: " + str(flags2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2hpu = 0                                                              
                message2print = "{(Step 11B/70) set" + "; flags2hpu-set-by-create2image4hpu to: " + str(flags2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; flags2hpu-failed-by-create2image4hpu: " + str(flags2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecreate2hpu ['FLAGS'] = flags2hpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) WIDTH - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            width2hpu = 0

            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI ETC...
            width2layer = 10                                                                       # SCALED TO 10 EACH LAYER            
            
            message2print = "{(Step 14A/70) trying" + "; width2hpu-trying-by-create2image4hpu: " + str(width2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                width2hpu = int(width2layer * (quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra))
                width2hpu = 2 * width2hpu                                                          # POS and NEG X  2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 14B/70) set" + "; width2hpu-set-by-create2image4hpu to: " + str(width2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; width2hpu-failed-by-create2image4hpu: " + str(width2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecreate2hpu ['WIDTH'] = width2hpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) HEIGHT - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            height2hpu = 0
            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI
            
            quantity2total = quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra
            height2layer = 10                                                                      # SCALED TO 10 
                       
            message2print = "{(Step 15A/70) trying" + "; height2hpu-trying-by-create2image4hpu: " + str(height2hpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
                        
            try:
                height2hpu = height2layer * quantity2total
                
                height2hpu = 2 * height2hpu                                                        # POS and NEG Y   2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 15B2/70) set" + "; height2hpu-set-by-create2image4hpu to: " + str(height2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; height2hpu-failed-by-create2image4hpu: " + str(height2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '15Z-'
             
                        
            task4imagecreate2hpu ['HEIGHT'] = height2hpu 
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2hpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2hpu-trying-by-create2image4hpu: " + str(rowpitch2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2hpu = int(width2hpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2hpu-set-by-create2image4hpu to: " + str(rowpitch2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                             
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2hpu-failed-by-create2image4hpu: " + str(rowpitch2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagecreate2hpu ['ROWPITCH'] = rowpitch2hpu     
            
            # ---------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED                           
            # ---------------------------------------------------------------------------------------------             
            pointer2hpu = None
            size2hpu = int(width2hpu * height2hpu)                                                                  # CALCULATED MEMORY
            if size2hpu == 0:
                size2hpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2hpu-trying-by-create2image4hpu: " + str(pointer2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2hpu = Arena4Xi(size2hpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B1/70) got" + "; arena2hpu-got-by-create2image4hpu: " + str(arena2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041C0D68>
                
                size2arena = arena2hpu.size                                                                         # 
                              
                message2print = "{(Step 17B2/70) got" + "; size2arena-got-by-create2buffer4hpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2hpu.buffer
                
                message2print = "{(Step 17B3/70) got" + "; buffer2arena-got-by-create2buffer4hpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x0000000003AAF298> 
                
                code2ack  = code2ack + '17B-' 
                
                if size2arena < size2hpu:
                    size2hpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17C/70) set" + "; size2hpu-set-by-create2buffer4hpu to: " + str(size2hpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17C-'
                else:
                    pass
                    
                pointer2hpu = buffer2arena
                message2print = "{(Step 17D/70) set" + "; pointer2hpu-set-by-create2buffer4hpu to: " + str(pointer2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x0000000003AAF298>
                
                code2ack  = code2ack + '17D-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2hpu-failed-by-create2image4hpu: " + str(pointer2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagecreate2hpu ['POINTER'] = pointer2hpu                            
            task4imagecreate2hpu ['SIZE']    = size2hpu                                                             # EXTRA
            
            
            # ---------------------------------------------------------------------------------------------
            # (18) FORMAT - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            format2hpu = {}
           
            message2print = "{(Step 18A/70) trying" + "; format2hpu-trying-by-create2image4hpu: " + str(format2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                format2hpu['ORDER'] = 'CL_RGBA'                                                              
                message2print = "{(Step 18B/70) set" + "; format2hpu-set-by-create2image4hpu to: " + str(format2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'ORDER': 'CL_RGBA'}
                 
                code2ack  = code2ack + '18B-'
                
                format2hpu['TYPE'] = 'CL_UNORM_INT8'                                                              
                message2print = "{(Step 18C/70) set" + "; format2hpu-set-by-create2image4hpu to: " + str(format2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
                
                code2ack  = code2ack + '18C-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; format2hpu-failed-by-create2image4hpu: " + str(format2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
        
            task4imagecreate2hpu ['FORMAT'] = format2hpu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) DEPTH - MUST BE 0                               
            # ---------------------------------------------------------------------------------------------             
            depth2hpu = 0
            message2print = "{(Step 19A/70) set" + "; depth2hpu-set-by-create2image4hpu to: " + str(depth2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
            code2ack  = code2ack + '19A-' 
            task4imagecreate2hpu ['DEPTH'] = depth2hpu
                                                                                                                                       
        else:
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4imagecreate2hpu-yielded-by-create2image4hpu: " + str(task4imagecreate2hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-' 

    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4imagecreate2hpu.keys()        
    message2print = "{(Step 19A/70) using" + "; keys2task-using-by-create2image4hpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['BUFFER',
    #  'CODE2ERROR',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'FLAGS',
    #  'FORMAT',
    #  'FUNCTION2CALLBACK',
    #  'HEIGHT',
    #  'PIPE',
    #  'PLATFORM',
    #  'POINTER',
    #  'QUEUE',
    #  'ROWPITCH',
    #  'SIZE',
    #  'WIDTH']; time: 00:53:55 10/05/11 Pacific Daylight Time}
  
    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - CREATE 2D IMAGE FOR HPU! 
    # ---------------------------------------------------------------------------------------------    
    hpu2image4xi = {}                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; hpu2image4xi-initialized-by-create2image4hpu to: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {} 
    
    message2print = "{(Step 21A0/70) try" + "; data2callback-try-by-create2image4hpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA 
        # -----------------------------------------------------------------------------------------               
        hpu2image4xi['DATA2CALLBACK'] = task4imagecreate2hpu['DATA2CALLBACK']
        
        message2print = "{(Step 21A1/70) initialized" + "; data2callback-created-by-create2image4hpu to: " + str(hpu2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4hpu-failed ; time: 05:56:13 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4hpu',
        #  'METHOD': 'create2image4hpu'}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-hpu2image4xi-yielded-by-create2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-create2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        hpu2image4xi['FUNCTION2CALLBACK'] = task4imagecreate2hpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) initialized" + "; function2callback-created-by-create2image4hpu to: " + str(hpu2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # <function trace2image4hpu at 0x0000000003B04348>  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-hpu2image4xi-yielded-by-create2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform-try-by-create2image4hpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        hpu2image4xi['PLATFORM'] = task4imagecreate2hpu['PLATFORM']
        
        message2print = "{(Step 22C1/70) initialized" + "; platform-created-by-create2image4hpu to: " + str(hpu2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 05:56:13 09/06/11 Pacific Daylight Time}
  
        code2ack  = code2ack + '22C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-hpu2image4xi-yielded-by-create2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2hpu-try-by-create2image4hpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        hpu2image4xi['CONTEXT'] = task4imagecreate2hpu['CONTEXT']
        
        message2print = "{(Step 23B/70) initialized" + "; context-created-by-create2image4hpu to: " + str(hpu2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4hpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2context4hpu',
        #  'METHOD': 'create2context4hpu'},
        #  'FUNCTION2CALLBACK': <function trace2context4hpu at 0x00000000036B0148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
        #  'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-hpu2image4xi-yielded-by-create2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-create2image4hpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        hpu2image4xi['DEVICE'] = task4imagecreate2hpu['DEVICE']
        
        message2print = "{(Step 24B/70) initialized" + "; device-created-by-create2image4hpu to: " + str(hpu2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 ...
        
        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-hpu2image4xi-yielded-by-create2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-create2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        hpu2image4xi['CODE2ERROR'] = task4imagecreate2hpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) initialized" + "; code2error-created-by-create2image4hpu to: " + str(hpu2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-hpu2image4xi-yielded-by-create2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-create2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        hpu2image4xi['QUEUE'] = task4imagecreate2hpu['QUEUE']
        
        message2print = "{(Step 26B/70) initialized" + "; queue-created-by-create2image4hpu to: " + str(hpu2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003ED99E8>  
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-hpu2image4xi-yielded-by-create2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-create2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        hpu2image4xi['PIPE'] = task4imagecreate2hpu['PIPE']
        
        message2print = "{(Step 27B/70) initialized" + "; pipe-created-by-create2image4hpu to: " + str(hpu2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 376>, <read-write PipeConnection, handle 380>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-hpu2image4xi-yielded-by-create2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-create2image4hpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        hpu2image4xi['COMMANDQUEUE']  = task4imagecreate2hpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) initialized" + "; commandqueue-created-by-create2image4hpu to: " + str(hpu2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4hpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2commandqueue4hpu',
        #  'METHOD': 'create2commandqueue4hpu'},
        #  'FUNCTION2CALLBACK': <function trace2commandqueue4hpu at 0x00000000037BC848>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
        #  'INFO2PRIVATE': None,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000432B3C8>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-create2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
              
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-create2image4hpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        hpu2image4xi['BUFFER']  = task4imagecreate2hpu['BUFFER']  
 
        message2print = "{(Step 29B/70) initialized" + "; bufferinfo-created-by-create2image4hpu to: " + str(hpu2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4hpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2buffer4hpu',
        #  'METHOD': 'create2buffer4hpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4hpu at 0x0000000003AFC348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003AAF2F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4hpu-failed ...
        
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-create2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-create2image4hpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        hpu2image4xi['EVENT']  = task4imagecreate2hpu['EVENT']  
 
        message2print = "{(Step 30B/70) initialized" + "; eventinfo-created-by-create2image4hpu to: " + str(hpu2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4hpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}',
        #                     'CLASS': 'create2event4hpu',
        #                     'METHOD': 'create2event4hpu'},
        #  'FUNCTION2CALLBACK': <function trace2event4hpu at 0x0000000003707D48>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000039C3860>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-create2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-create2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        hpu2image4xi['FLAGS']  = task4imagecreate2hpu['FLAGS']  
 
        message2print = "{(Step 31B/70) initialized" + "; flags-created-by-create2image4hpu to: " + str(hpu2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-create2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-create2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        hpu2image4xi['WIDTH']  = task4imagecreate2hpu['WIDTH']  
 
        message2print = "{(Step 34H/70) initialized" + "; width-created-by-create2image4hpu to: " + str(hpu2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-create2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-create2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        hpu2image4xi['HEIGHT']  = task4imagecreate2hpu['HEIGHT']  
 
        message2print = "{(Step 35H/70) initialized" + "; height-created-by-create2image4hpu to: " + str(hpu2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-create2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-create2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        hpu2image4xi['ROWPITCH']  = task4imagecreate2hpu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) initialized" + "; rowpitch-created-by-create2image4hpu to: " + str(hpu2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-create2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-create2image4hpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        hpu2image4xi['POINTER']  = task4imagecreate2hpu['POINTER']  
 
        message2print = "{(Step 37B/70) initialized" + "; pointer-created-by-create2image4hpu to: " + str(hpu2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <mmap.mmap object at 0x0000000003C050E0>
         
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-create2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-create2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        hpu2image4xi['FORMAT']  = task4imagecreate2hpu['FORMAT']  
 
        message2print = "{(Step 38B/70) initialized" + "; format-created-by-create2image4hpu to: " + str(hpu2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-create2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '38Z1-'

    message2print = "{(Step 39A/70) try" + "; depth-try-by-create2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (39) DEPTH - MUST BE 0                              
        # ---------------------------------------------------------------------------------------------             
                        
        hpu2image4xi['DEPTH']  = task4imagecreate2hpu['DEPTH']  
 
        message2print = "{(Step 39B/70) initialized" + "; depth-created-by-create2image4hpu to: " + str(hpu2image4xi['DEPTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '39B-'
    except: 
        message2print = "{(Step 39Z1/70) yielded" + "; depth-yielded-by-create2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '39Z1-'
                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    hpu2image4xi ['ACK2RETURN']  = code2ack
    hpu2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; hpu2image4xi-returning-by-create2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    # 'DEPTH'
    # 'WIDTH'
    # 'HEIGHT'
    return hpu2image4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2image4hpu" usage = "
#    
#   print('(Step 4B/33) OF HPU2IMAGE4XI5TEST - TRYING TO GET PLATFORMIDS4XI', time4xi())
#   platform2xi = {}
#   try:
#       platform2xi = get2platformids4xi()
#       print('(Step 4B/33) of CONTEXT4HPU5TEST - GOT PLATFORMIDS4XI', platform2xi) 
#   except:
#       print('(Step 4Z/33) of CONTEXT4HPU5TEST *** FAILED TO GET PLATFORMIDS4XI', time4xi())     
#   
#   print('(Step 2A/33) OF DEVICE4HPU5TEST - TRYING TO INITIALIZE DEVICE4HPU ', time4xi())   
#   print('=' *33)        
#   try:
#       device2hpu = init2hpu4xi()                                                                                
#       print('(Step 2B/33) OF DEVICE4HPU5TEST - INITIALIZED DEVICE4HPU TO: ', device2hpu)        
#   except:
#       print('(Step 2Z/33) OF DEVICE4HPU5TEST *** FAILED TO INITIALIZE DEVICE4HPU', time4xi()) 
#   print('(Step 3A/33) OF HPU2IMAGE4XI5TEST - TRYING TO CREATE CONTEXT4HPU ', time4xi())    
#   print('=' *33)        
#   try:
#       context2hpu = create2context4hpu()                                                                               
#       print('(Step 3B/33) OF HPU2IMAGE4XI5TEST - CREATED CONTEXT4HPU: ', context2hpu)                                  
#   except:
#       print('(Step 3Z/33) OF HPU2IMAGE4XI5TEST *** FAILED TO CREATE CONTEXT4HPU', time4xi()) 
#          
#   task4set2image = {}
#   task4set2image  ['CONTEXT'] = context2hpu  
# 
#   ... other data for set operation ...
#    
#        
#   response4set2Dimage = {} 
#   print('(Step 11A/700) OF HPU2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE4HPU ', time4xi())   
#   try:                               
#       response4set2Dimage = set2image4hpu(task4set2image)                                                                                                         
#                                                                                     
#       print('(Step 11B/700) OF HPU2IMAGE4XI5TEST - SET 2D IMAGE4HPU ', time4xi())
#       keys2response = response4set2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4set2Dimage[key2response])
#           # 
#   except:
#       print('(Step 11Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE4HPU *** ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def set2image4hpu(task4imageset2hpu=None):

    '''
    sets 2D images for HPU
    '''
    global hpu2image4xi  
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set2image4hpu"
    method2print = "set2image4hpu"

    message2print = "{(Step 1A/33) started" + "; set2image4hpu-started: " + str(set2image4hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set2image4hpu at 0x0000000003A62848> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageset2hpu-to-set2image4hpu: " + str(task4imageset2hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE2ERROR': 0,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003D4A710>,
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4hpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4hpu',
    #              'METHOD': 'create2context4hpu'},
    #              'FUNCTION2CALLBACK': <function trace2context4hpu at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #  'COMMANDQUEUE': {'CODE2ERROR': 0,
    #              'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4hpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2commandqueue4hpu',
    #              'METHOD': 'create2commandqueue4hpu'},
    #  'FUNCTION2CALLBACK': <function trace2commandqueue4hpu at 0x00000000037BC848>,
    #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
    #  'INFO2PRIVATE': None,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003EDA780>,
    #  'NACK2RETURN': ' ',
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4hpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4hpu',
    #              'METHOD': 'create2context4hpu'},
    #              'FUNCTION2CALLBACK': <function trace2context4hpu at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'],
    #                           'ack2return': ' 2C-2E-2F-2G-',
    #                           'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #              'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318,
    #                          'ID2PLATFORM': '0000000001509568',
    #  'PROPERTIES': None,
    #  'PIPE': (<read-write PipeConnection, handle 552>, <read-write PipeConnection, handle 556>)},
    #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK',
    #              'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 
    #              'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 
    #              'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 
            
    code2ack  = ' '
    code2nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if hpu2image4xi is None:
            hpu2image4xi = create2image4hpu()                                                                              # INITIALIZE IMAGE FOR HPU 
            message2print = "{(Step 1C/70) set" + "; hpu2image4xi-set-by-set2image4hpu to: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4hpu-skipped-by-set2image4hpu: " + str(create2image4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function create2image4hpu at 0x0000000003A627C8>;                                  
    except:
        try:
            hpu2image4xi = create2image4hpu()
            message2print = "{(Step 1E/70) set" + "; hpu2image4xi-set-by-set2image4hpu to: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            hpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; hpu2image4xi-yielded-by-set2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET            
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data2callback-try-by-set2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        hpu2image4xi['DATA2CALLBACK'] = task4imageset2hpu['DATA2CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data2callback-set-by-set2image4hpu to: " + str(hpu2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-hpu2image4xi-yielded-by-set2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-set2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        hpu2image4xi['FUNCTION2CALLBACK'] = task4imageset2hpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function2callback-set-by-set2image4hpu to: " + str(hpu2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-hpu2image4xi-yielded-by-set2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        hpu2image4xi['PLATFORM'] = task4imageset2hpu['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set2image4hpu to: " + str(hpu2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # yielded  
        code2ack  = code2ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-hpu2image4xi-yielded-by-set2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2hpu-try-by-set2image4hpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        hpu2image4xi['CONTEXT'] = task4imageset2hpu['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set2image4hpu to: " + str(hpu2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4hpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #                    'CLASS': 'create2context4hpu',
        #                    'METHOD': 'create2context4hpu'},
        #  'FUNCTION2CALLBACK': <function trace2context4hpu at 0x0000000003770148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 07:13:59 09/06/11 Pacific Daylight Time} 

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-hpu2image4xi-yielded-by-set2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set2image4hpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        hpu2image4xi['DEVICE'] = task4imageset2hpu['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set2image4hpu to: " + str(hpu2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0',
        #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics ...
        #  'AVAILABLE2COMPILER': 'ACK',
        #  'MAX2DIMENSIONS4WORKITEMS': 3,
        #  'SUPPORT2CORRECTION4ERROR': 0,
        #  'BITS2ADDRESS': 64,
        #  'AVAILABLE': 'ACK',
        #  'SIZE2MEMORY4LOCAL': 32768,
        #  'VENDOR': 'AuthenticAMD',
        #  'SUPPORT2IMAGE': 'NACK',
        #  'MAX2SIZE4KERNELARGUMENT': 4096,
        #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-...
        #  'OUT-OF-ORDER2QUEUE': 'NACK',
        #  'ALIGNMENT2ADDRESS4BASE': 1024,
        #  'PROFILE': 'FULL_PROFILE',
        #  'TYPE2MEMORY4LOCAL': 'Global',
        #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
        #  'PREFERRED2WIDTHLONG4VECTOR': 2,
        #  'MAX2FREQUENCY4CLOCK': 3210,
        #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
        #  'ID': 4033,
        #  'MAX2WORKITEM2': 1024,
        #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
        #  'ROUND2ZERO': 'ACK',
        #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
        #  'MAX2WORKITEM1': 1024,
        #  'MAX2WORKITEM0': 1024,
        #  'SIZE2LINE4CACHE': 64,
        #  'MAX2ALLOCATION4MEMORY': 1073741824,
        #  'NATIVE2WIDTHCHAR4VECTOR': 16,
        #  'QUIET2NANS': 'ACK',
        #  'NATIVE2WIDTHSHORT4VECTOR': 8,
        #  'ROUND2NEAREST4EVEN': 'ACK',
        #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
        #  'NACK2RETURN': ' ',
        #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
        #  'ENDIANESS2DEVICE': 'Little',
        #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
        #  'ROUND2+VE3INFINITY': 'ACK',
        #  'TYPE': 'CL_DEVICE_TYPE_HPU',
        #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 07:13:59 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-hpu2image4xi-yielded-by-set2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-set2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        hpu2image4xi['CODE2ERROR'] = task4imageset2hpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code2error-set-by-set2image4hpu to: " + str(hpu2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-hpu2image4xi-yielded-by-set2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        hpu2image4xi['QUEUE'] = task4imageset2hpu['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set2image4hpu to: " + str(hpu2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003D4A710> 
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-hpu2image4xi-yielded-by-set2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        hpu2image4xi['PIPE'] = task4imageset2hpu['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set2image4hpu to: " + str(hpu2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-hpu2image4xi-yielded-by-set2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set2image4hpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        hpu2image4xi['COMMANDQUEUE']  = task4imageset2hpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set2image4hpu to: " + str(hpu2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set2image4hpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        hpu2image4xi['BUFFER']  = task4imageset2hpu['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set2image4hpu to: " + str(hpu2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set2image4hpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        hpu2image4xi['EVENT']  = task4imageset2hpu['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set2image4hpu to: " + str(hpu2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-set2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        hpu2image4xi['FLAGS']  = task4imageset2hpu['FLAGS']  
 
        message2print = "{(Step 31B/70) set" + "; flags-set-by-set2image4hpu to: " + str(hpu2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-set2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-set2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        hpu2image4xi['WIDTH']  = task4imageset2hpu['WIDTH']  
 
        message2print = "{(Step 34H/70) set" + "; width-set-by-set2image4hpu to: " + str(hpu2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-set2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-set2image4hpu: None"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        hpu2image4xi['HEIGHT']  = task4imageset2hpu['HEIGHT']  
 
        message2print = "{(Step 35H/70) set" + "; height-set-by-set2image4hpu to: " + str(hpu2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-set2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-set2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        hpu2image4xi['ROWPITCH']  = task4imageset2hpu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) set" + "; rowpitch-set-by-set2image4hpu to: " + str(hpu2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-set2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-set2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        hpu2image4xi['POINTER']  = task4imageset2hpu['POINTER']  
 
        message2print = "{(Step 37B/70) set" + "; pointer-set-by-set2image4hpu to: " + str(hpu2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-set2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-set2image4hpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        hpu2image4xi['FORMAT']  = task4imageset2hpu['FORMAT']  
 
        message2print = "{(Step 38B/70) set" + "; format-set-by-set2image4hpu to: " + str(hpu2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-set2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '38Z1-'
                        
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    hpu2image4xi ['ACK2RETURN']  = code2ack
    hpu2image4xi ['NACK2RETURN'] = code2nack 
    
    message2print = "{(Step 70/70) returning" + "; hpu2image4xi-returning-by-set2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-', 'ROWPITCH': 800, 'DATA2CALLBACK': ...
    
    
    return hpu2image4xi

           
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2image4hpu" usage = "
#                                                                                                              
#   # -----------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4HPU                      
#   # -----------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF HPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4HPU ', time4xi())    
#   response4copy2image = {} 
#   task4copy2image = None
#   try:                               
#       response4copy2image = copy2image4hpu(task4copy2image)                                                                                                          
#       print('(Step 16B/700) OF HPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGE4HPU : ', response4copy2image)
#       queue2copy4hpu      = response4copy2image['QUEUE']
#       channel2copypublish4hpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copyconsume4hpu = response4copy2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4HPU ', time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4HPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF HPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4HPU ', time4xi())     
#   try:                               
#       message2copyimagesend4hpu = "<copy4xi>copy2copy4hpu</copy4xi>"
#       channel2copypublish4hpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4hpu.send(message2copyimagesend4hpu)
#       print("(Step 17B/700) OF HPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4HPU: ", channel2copypublish4hpu, " MESSAGE2SEND4HPU: " , message2copyimagesend4hpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF HPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4HPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4HPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi()) 
#   try:        
#       response2manoeuvre4hpu = manoeuvre2copyimage4hpu(queue2copy4hpu, channel2copyconsume4hpu, channel2copypublish4hpu)
#       print("(Step 18D/700) OF HPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", manoeuvre2copyimage4hpu)
#   except:
#       print("(Step 18Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # -----------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4HPU ', time4xi())     
#   try:
#       channel2copyconsume4hpu = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4hpu = channel2copyconsume4hpu.recv()
#       print("(Step 19B/700) OF HPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4HPU: ", channel2copyconsume4hpu, " CONTROL & COMMAND: ", message2copyimagereceive4hpu)                                                                                                  
#       # <roger4xi>roger2copy4hpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4HPU ', time4xi())         
#   # -----------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4HPU TO RECEIVE                
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF HPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4HPU ', time4xi())     
#   try:                               
#       message2copyimagesend4hpu = "<copy4xi>copy2copy4hpu</copy4xi>"
#       channel2copypublish4hpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4hpu.send(message2copyimagesend4hpu)
#       print("(Step 20B/700) OF HPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4HPU: ", channel2copypublish4hpu, " MESSAGE2SEND4HPU: " , message2copyimagesend4hpu)                                                                                                
#       # <copy4xi>copy2copy4hpu</copy4xi> 
#   except:
#        print('(Step 20Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4HPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4HPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi()) 
#   try:        
#       response2manoeuvre4hpu = manoeuvre2copyimage4hpu(queue2copy4hpu, channel2copyconsume4hpu, channel2copypublish4hpu)
#       print("(Step 21B/700) OF HPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", manoeuvre2copyimage4hpu)
#   except:
#       print("(Step 21Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4HPU ', time4xi())     
#   try:
#       channel2copyconsume4hpu = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4hpu = channel2copyconsume4hpu.recv()
#       print("(Step 22B/700) OF HPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4HPU: ", channel2copyconsume4hpu, " CONTROL & COMMAND: ", message2copyimagereceive4hpu)                                                                                                  
#       # <copy4xi>copy2copy4hpu</copy4xi> 
#   except:
#       print('(Step 22Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4HPU ', time4xi())          
#   # ----------------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # ----------------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copy4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4HPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi()) 
#   try:        
#       response2manoeuvre4hpu = manoeuvre2copyimage4hpu(queue2copy4hpu, channel2copyconsume4hpu, channel2copypublish4hpu)
#       print("(Step 25B/700) OF HPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", manoeuvre2copyimage4hpu)
#   except:
#       print("(Step 25Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4hpu = channel2copyconsume4hpu.recv()
#       print("(Step 26D/700) OF HPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4HPU: ", channel2copyconsume4hpu, " CONTROL & COMMAND: ", message2copyimagereceive4hpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4hpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -----------------------------------------------------------------------------------------------
#   (27) LET MANOEUVRE2IMAGE4HPU GO TO STATE 0               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi()) 
#   try:        
#       response2manoeuvre4hpu = manoeuvre2copyimage4hpu(queue2copy4hpu, channel2copyconsume4hpu, channel2copypublish4hpu)
#       print("(Step 27B/700) OF HPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4HPU ", response2manoeuvre4hpu)
#   except:
#       print("(Step 27Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2image4hpu(task4imagecopy2hpu=None):

    '''
    copies 2D images for HPU
    '''
    global hpu2image4xi                                                                                             # 2D-IMAGES FOR HPU  
    global control2copyimage4hpu                                                                                    # COMMAND AND CONTROL FOR HPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2image4hpu"
    method2print = "copy2image4hpu"

    message2print = "{(Step 0A/33) started" + "; copy2image4hpu-started: " + str(copy2image4hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                               
    # <function copy2image4hpu at 0x00000000038538C8>
    
    message2print = "{(Step 0B/33) passed" + "; task4imagecopy2hpu-to-copy2image4hpu: " + str(task4imagecopy2hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE HPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if hpu2image4xi is None:
            hpu2image4xi = create2image4hpu()                                                                       # INITIALIZE IMAGE FOR HPU 
            message2print = "{(Step 0C/70) created" + "; hpu2image4xi-created-by-copy2image4hpu to: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        else:
            message2print = "{(Step 0D/70) skipped" + "; create2image4hpu-skipped-by-copy2image4hpu: " + str(copy2image4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function copy2image4hpu at 0x00000000038538C8>                                 
    except:
        try:
            hpu2image4xi = create2image4hpu()
            message2print = "{(Step 0E/70) set" + "; hpu2image4xi-set-by-copy2image4hpu to: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            hpu2image4xi = {}
            message2print = "{(Step 0Z/70) yielded" + "; hpu2image4xi-yielded-by-copy2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '0Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagecopy2hpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecopy2hpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagecopy2hpu-created-by-copy2image4hpu to: " + str(task4imagecopy2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {} 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4hpu = trace2image4hpu                                                                 # CALLBACK FUNCTION FOR HPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4hpu-created-by-copy2image4hpu to: " + str(function2callback4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4hpu at 0x000000000384E9C8> 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4hpu"                                                            # MODULE OF HPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4hpu"                                                            # FUNCTION OF HPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4hpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR HPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2image4hpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4hpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4hpu',
            #  'METHOD': 'copy2image4hpu'}; time: 07:01:44 10/04/11 Pacific Daylight Time}


            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2hpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO HPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2hpu['FUNCTION2CALLBACK'] = function2callback4hpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagecopy2hpu-created-by-copy2image4hpu to: " + str(task4imagecopy2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4hpu at 0x000000000384E9C8>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4hpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4hpu',
            #  'METHOD': 'copy2image4hpu'}}; time: 07:01:44 10/04/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2image4hpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2image4hpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 07:01:44 10/04/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2image4hpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecopy2hpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2hpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2hpu-trying-by-copy2image4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2hpu = get2context4hpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2hpu-got-by-copy2image4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) ...
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2hpu-failed-by-copy2image4hpu: " + str(context2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecopy2hpu ['CONTEXT'] = context2hpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2hpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2hpu-trying-by-copy2image4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2hpu = get2hpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2hpu-got-by-copy2image4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2hpu-failed-by-copy2image4hpu: " + str(device2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagecopy2hpu ['DEVICE'] = device2hpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE SOURCE OBJECT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------         
            imagesourceinfo2hpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2hpu-trying-by-copy2image4hpu: " + str(imagesourceinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagesourceinfo2hpu = get2imageinfo4hpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imagesourceinfo2hpu-got-by-copy2image4hpu: " + str(imagesourceinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2hpu-failed-by-copy2image4hpu: " + str(imagesourceinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagecopy2hpu ['IMAGESOURCE'] = imagesourceinfo2hpu         
                   
            # ---------------------------------------------------------------------------------------------
            # (6) IMAGE DESTINATION OBJECT - REQUIRED                        
            # ---------------------------------------------------------------------------------------------                         
            imagedestinationinfo2hpu = {}
             
            message2print = "{(Step 6A/70) trying" + "; imagedestinationinfo2hpu-trying-by-copy2image4hpu: " + str(imagedestinationinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagedestinationinfo2hpu = get2imageinfo4hpu()                                                                         
                
                message2print = "{(Step 6B/70) got" + "; imagedestinationinfo2hpu-got-by-copy2image4hpu: " + str(imagedestinationinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                # note: source and destination will be the same !             
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; imagedestinationinfo2hpu-failed-by-copy2image4hpu: " + str(imagedestinationinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'          
                      
            task4imagecopy2hpu ['IMAGEDESTINATION'] = imagedestinationinfo2hpu     
            
            # ---------------------------------------------------------------------------------------------
            # (7) QUEUE - REQUIRED BY HPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2hpu = None
            
            message2print = "{(Step 7A/70) trying" + "; queue2hpu-trying-by-copy2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2hpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 7B/70) got" + "; queue2hpu-got-by-copy2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x000000000453EB38>
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; queue2hpu-failed-by-copy2image4hpu: " + str(queue2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecopy2hpu ['QUEUE'] = queue2hpu

            
            # ---------------------------------------------------------------------------------------------
            # (8) PIPE - REQUIRED BY HPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2hpu = None
           
            message2print = "{(Step 8A/70) trying" + "; pipe2hpu-trying-by-copy2image4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2hpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 8B/70) got" + "; pipe2hpu-got-by-copy2image4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 612>, <read-write PipeConnection, handle 616>)
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; pipe2hpu-failed-by-copy2image4hpu: " + str(pipe2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
                
            task4imagecopy2hpu ['PIPE'] = pipe2hpu 
            
            # ---------------------------------------------------------------------------------------------
            # (9) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2hpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2hpu-trying-by-copy2image4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2hpu = get2commandqueueinfo4hpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 9B/70) got" + "; commandqueueinfo2hpu-got-by-copy2image4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4hpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4hpu', 
                #  'METHOD': 'create2commandqueue4hpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4hpu at 0x00000000037EDA48> ...
                
                code2ack  = code2ack + '9B-'  
            except:
                message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2hpu-failed-by-copy2image4hpu: " + str(commandqueueinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecopy2hpu ['COMMANDQUEUE'] = commandqueueinfo2hpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (10) BUFFER - AUTOMATIC BY HPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2hpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; bufferinfo2hpu-trying-by-copy2image4hpu: " + str(bufferinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2hpu = get2bufferinfo4hpu()                                                               # SYSTEM CREATES BUFFER FOR HPU, IF NECESSARY
                message2print = "{(Step 10B/70) got" + "; bufferinfo2hpu-got-by-copy2image4hpu: " + str(bufferinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE' ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; bufferinfo2hpu-failed-by-copy2image4hpu: " + str(bufferinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecopy2hpu ['BUFFER'] = bufferinfo2hpu 
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2hpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventinfo2hpu-trying-by-copy2image4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2hpu = get2eventinfo4hpu()                                                                    
                message2print = "{(Step 11B/70) got" + "; eventinfo2hpu-got-by-copy2image4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4hpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
                #  'CLASS': 'create2event4hpu',
                #  'METHOD': 'create2event4hpu'},
                #  'FUNCTION2CALLBACK': <function trace2event4hpu at 0x0000000003803F48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000423B668>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventinfo2hpu-failed-by-copy2image4hpu: " + str(eventinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecopy2hpu ['EVENT'] = eventinfo2hpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2hpu = {}
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlist2hpu-trying-by-copy2image4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2hpu = {}                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlist2hpu-set-by-copy2image4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2hpu-failed-by-copy2image4hpu: " + str(eventwaitlist2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagecopy2hpu ['EVENTWAITLIST'] = eventwaitlist2hpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (13) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2hpu = 0
            
            message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2hpu-trying-by-copy2image4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2hpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2hpu = eventwaitlistquantity2hpu + 1    
                
                message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2hpu-set-by-copy2image4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0  
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2hpu-failed-by-copy2image4hpu: " + str(eventwaitlistquantity2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagecopy2hpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2hpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN FOR SOURCE - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin4source2hpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin4source2hpu-trying-by-copy2image4hpu: " + str(origin4source2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4source2hpu['X'] = 0 
                origin4source2hpu['Y'] = 0  
                origin4source2hpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin4source2hpu-set-by-copy2image4hpu to: " + str(origin4source2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin4source2hpu-failed-by-copy2image4hpu: " + str(origin4source2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecopy2hpu ['SOURCEORIGIN'] = origin4source2hpu
            
            # ---------------------------------------------------------------------------------------------
            # (15) ORIGIN FOR DESTINATION - REQUIRED                             
            # ---------------------------------------------------------------------------------------------              
            origin4destination2hpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; origin4destination2hpu-trying-by-copy2image4hpu: " + str(origin4destination2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4destination2hpu['X'] = 100 
                origin4destination2hpu['Y'] = 100  
                origin4destination2hpu['Z'] = 100                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; origin4destination2hpu-set-by-copy2image4hpu to: " + str(origin4destination2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; origin4destination2hpu-failed-by-copy2image4hpu: " + str(origin4destination2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagecopy2hpu ['DESTINATIONORIGIN'] = origin4destination2hpu
                        
            
            
            # --------------------------------------------------------------------------------------------------------
            # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
            # --------------------------------------------------------------------------------------------------------             
            region4copy2hpu = {}                                                                                                         
           
            message2print = "{(Step 16A/70) trying" + "; region4copy2hpu-trying-by-copy2image4hpu: " + str(region4copy2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                region4copy2hpu['X'] = 10 
                region4copy2hpu['Y'] = 10  
                region4copy2hpu['Z'] = 10                                                                           
                message2print = "{(Step 16B/70) set" + "; region4copy2hpu-set-by-copy2image4hpu to: " + str(region4copy2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 

                code2ack  = code2ack + '16B-'                
                 
            except:
                message2print = "{(Step 16Z/70) failed" + "; region4copy2hpu-failed-by-copy2image4hpu: " + str(region4copy2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '16Z-'
            
            task4imagecopy2hpu ['REGION'] = region4copy2hpu              
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2hpu.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-copy2image4hpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:21:58 10/05/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4hpu = trace2image4hpu                                                            # CALLBACK FUNCTION FOR HPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4hpu-created-by-copy2image4hpu to: " + str(function2callback4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                 # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4hpu"                                                        # MODULE OF HPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4hpu"                                                        # FUNCTION OF HPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4hpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR HPUIZED OPENCL TO RETURN    
            
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2image4hpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2hpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO HPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2hpu['FUNCTION2CALLBACK'] = function2callback4hpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
                
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagecopy2hpu-created-by-copy2image4hpu to: " + str(task4imagecopy2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
                 
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2image4hpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2image4hpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2image4hpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagecopy2hpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2hpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2hpu-trying-by-copy2image4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2hpu = get2context4hpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2hpu-got-by-copy2image4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2hpu-failed-by-copy2image4hpu: " + str(context2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagecopy2hpu ['CONTEXT'] = context2hpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2hpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2hpu-trying-by-copy2image4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2hpu = get2hpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2hpu-got-by-copy2image4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2hpu-failed-by-copy2image4hpu: " + str(device2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagecopy2hpu ['DEVICE'] = device2hpu
        
        if 'IMAGESOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) SOURCE IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourceimageinfo2hpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2hpu-trying-by-copy2image4hpu: " + str(sourceimageinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourceimageinfo2hpu = get2imageinfo4hpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; sourceimageinfo2hpu-got-by-copy2image4hpu: " + str(sourceimageinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2hpu-failed-by-copy2image4hpu: " + str(sourceimageinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagecopy2hpu ['IMAGESOURCE'] = sourceimageinfo2hpu         
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2hpu = {}
             
            message2print = "{(Step 26A/70) trying" + "; destinationimageinfo2hpu-trying-by-copy2image4hpu: " + str(destinationimageinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2hpu = get2imageinfo4hpu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; destinationimageinfo2hpu-got-by-copy2image4hpu: " + str(destinationimageinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; destinationimageinfo2hpu-failed-by-copy2image4hpu: " + str(destinationimageinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4imagecopy2hpu ['IMAGEDESTINATION'] = destinationimageinfo2hpu    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY HPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2hpu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2hpu-trying-by-copy2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2hpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2hpu-got-by-copy2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2hpu-failed-by-copy2image4hpu: " + str(queue2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagecopy2hpu ['QUEUE'] = queue2hpu

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY HPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2hpu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2hpu-trying-by-copy2image4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2hpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2hpu-got-by-copy2image4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2hpu-failed-by-copy2image4hpu: " + str(pipe2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4imagecopy2hpu ['PIPE'] = pipe2hpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2hpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2hpu-trying-by-copy2image4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2hpu = get2commandqueueinfo4hpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2hpu-got-by-copy2image4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2hpu-failed-by-copy2image4hpu: " + str(commandqueueinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagecopy2hpu ['COMMANDQUEUE'] = commandqueueinfo2hpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) BUFFER - AUTOMATIC BY HPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2hpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; bufferinfo2hpu-trying-by-copy2image4hpu: " + str(bufferinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2hpu = get2bufferinfo4hpu()                                                           # SYSTEM CREATES BUFFER FOR HPU, IF NECESSARY
                message2print = "{(Step 30B/70) got" + "; bufferinfo2hpu-got-by-copy2image4hpu: " + str(bufferinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; bufferinfo2hpu-failed-by-copy2image4hpu: " + str(bufferinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagecopy2hpu ['BUFFER'] = bufferinfo2hpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2hpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2hpu-trying-by-copy2image4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2hpu = get2eventinfo4hpu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2hpu-got-by-copy2image4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2hpu-failed-by-copy2image4hpu: " + str(eventinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagecopy2hpu ['EVENT'] = eventinfo2hpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2hpu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2hpu-trying-by-copy2image4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2hpu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2hpu-set-by-copy2image4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2hpu-failed-by-copy2image4hpu: " + str(eventwaitlist2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagecopy2hpu ['EVENTWAITLIST'] = eventwaitlist2hpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------
            eventwaitlistquantity2hpu = 0
            
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2hpu-trying-by-copy2image4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2hpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2hpu = eventwaitlistquantity2hpu + 1    
                
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2hpu-set-by-copy2image4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2hpu-failed-by-copy2image4hpu: " + str(eventwaitlistquantity2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
                    
            task4imagecopy2hpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2hpu       
                                             
                    
        if 'SOURCEORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) SOURCEORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            source4origin2hpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; source4origin2hpu-trying-by-copy2image4hpu: " + str(source4origin2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                source4origin2hpu['X'] = 0 
                source4origin2hpu['Y'] = 0  
                source4origin2hpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; source4origin2hpu-set-by-copy2image4hpu to: " + str(source4origin2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; source4origin2hpu-failed-by-copy2image4hpu: " + str(source4origin2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagecopy2hpu ['SOURCEORIGIN'] = source4origin2hpu
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (35) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2hpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; destination4origin2hpu-trying-by-copy2image4hpu: " + str(destination4origin2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2hpu['X'] = 100 
                destination4origin2hpu['Y'] = 100  
                destination4origin2hpu['Z'] = 100                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; destination4origin2hpu-set-by-copy2image4hpu to: " + str(destination4origin2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; destination4origin2hpu-failed-by-copy2image4hpu: " + str(destination4origin2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagecopy2hpu ['DESTINATIONSOURCEORIGIN'] = destination4origin2hpu                
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2hpu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2hpu-trying-by-copy2image4hpu: " + str(region2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2hpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2hpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2hpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2hpu-set-by-copy2image4hpu to: " + str(region2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2hpu-failed-by-copy2image4hpu: " + str(region2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4imagecopy2hpu ['REGION'] = region2hpu
        
        # ---------------------------------------------------------------------------------------------
        #  EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2hpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2image4hpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagecopy2hpu-yielded-by-copy2image4hpu: " + str(task4imagecopy2hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimage4hpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2hpu = False
    channels2hpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2hpu-trying-by-copy2image4hpu: " + str(channels2hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2hpu, publisherchannel2hpu = task4imagecopy2hpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2hpu-got-by-copy2image4hpu: " + str(consumerchannel2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 612> 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2hpu-got-by-copy2image4hpu: " + str(publisherchannel2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 616>
                              
        ack4channels2hpu = True
        
        control2copyimage4hpu['CONSUMERCHANNEL']  = consumerchannel2hpu
        control2copyimage4hpu['PUBLISHERCHANNEL'] = publisherchannel2hpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2hpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2hpu = False
    queue2hpu = None
    message2print = "{(Step 41A/70) trying" + "queue2hpu-trying-by-copy2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2hpu = task4imagecopy2hpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2hpu-got-by-copy2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004721CF8>
              
        ack4queue2hpu = True
        control2copyimage4hpu['QUEUE']  = queue2hpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2hpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4hpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4hpu-trying-by-copy2image4hpu: " + str(dummy2image4hpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4hpu = DummyProcess4Xi(target=manoeuvre2copyimage4hpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2hpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2hpu, publisherchannel2hpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4hpu-created-by-copy2image4hpu: " + str(dummy2image4hpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2image4hpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2image4hpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimage4hpu-returning-by-copy2image4hpu: " + str(control2copyimage4hpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004721CF8>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 616>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 612>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
   
    return control2copyimage4hpu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2imagetobuffer4hpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (60) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4HPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 60A/700) OF HPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4HPU ', time4xi())    
#   response4copy2imagetobuffer = {} 
#   task4copy2imagetobuffer = None
#   try:                               
#       response4copy2imagetobuffer = copy2imagetobuffer4hpu(task4copy2imagetobuffer)                                                                                                          
#       print('(Step 60B/700) OF HPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4HPU : ', response4copy2imagetobuffer)
#       queue2copyimagetobuffer4hpu      = response4copy2imagetobuffer['QUEUE']
#       channel2copyimagetobufferpublish4hpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferconsume4hpu = response4copy2imagetobuffer['CONSUMERCHANNEL']
#   except:
#       print('(Step 60Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4HPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (61) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4HPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 61A/700) OF HPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4HPU ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4hpu = "<copy4xi>copy2copyimagetobuffer4hpu</copy4xi>"
#       channel2copyimagetobufferpublish4hpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4hpu.send(message2copyimagetobuffersend4hpu)
#       print("(Step 61B/700) OF HPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4HPU: ", channel2copyimagetobufferpublish4hpu, " MESSAGE2SEND4HPU: " , message2copyimagetobuffersend4hpu)                                                                                                
#   except:
#       print('(Step 61Z1/700) OF HPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4HPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (62) LET MANOEUVRE2COPYIMAGETOBUFFER4HPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 62A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4hpu = manoeuvre2copyimagetobuffer4hpu(queue2copyimagetobuffer4hpu, channel2copyimagetobufferconsume4hpu, channel2copyimagetobufferpublish4hpu)
#       print("(Step 62B/700) OF HPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", manoeuvre2copyimagetobuffer4hpu)
#   except:
#       print("(Step 62Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (63) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 63A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4HPU ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4hpu = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4hpu = channel2copyimagetobufferconsume4hpu.recv()
#       print("(Step 63B/700) OF HPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4HPU: ", channel2copyimagetobufferconsume4hpu, " CONTROL & COMMAND: ", message2copyimagereceive4hpu)                                                                                                  
#       # 
#   except:
#       print('(Step 63Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4HPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (64) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4HPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 64A/700) OF HPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4HPU ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4hpu = "<copy4xi>copy2copyimagetobuffer4hpu</copy4xi>"
#       channel2copyimagetobufferpublish4hpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4hpu.send(message2copyimagetobuffersend4hpu)
#       print("(Step 64B/700) OF HPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4HPU: ", channel2copyimagetobufferpublish4hpu, " MESSAGE2SEND4HPU: " , message2copyimagetobuffersend4hpu)                                                                                                
#       #  
#   except:
#       print('(Step 64Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4HPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (65) LET MANOEUVRE2COPYIMAGETOBUFFER4HPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 65A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4hpu = manoeuvre2copyimagetobuffer4hpu(queue2copyimagetobuffer4hpu, channel2copyimagetobufferconsume4hpu, channel2copyimagetobufferpublish4hpu)
#       print("(Step 65B/700) OF HPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", manoeuvre2copyimagetobuffer4hpu)
#   except:
#       print("(Step 65Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (66) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 66A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4HPU ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4hpu = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4hpu = channel2copyimagetobufferconsume4hpu.recv()
#       print("(Step 66B/700) OF HPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4HPU: ", channel2copyimagetobufferconsume4hpu, " CONTROL & COMMAND: ", message2copyimagereceive4hpu)                                                                                                  
#       # 
#   except:
#       print('(Step 66Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4HPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (67) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 67A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copyimagetobuffer4dragon.put(path2dragon)             
#       print("(Step 67B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 67Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (68) LET MANOEUVRE2COPYIMAGETOBUFFER4HPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 68A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4hpu = manoeuvre2copyimagetobuffer4hpu(queue2copyimagetobuffer4hpu, channel2copyimagetobufferconsume4hpu, channel2copyimagetobufferpublish4hpu)
#       print("(Step 68B/700) OF HPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", manoeuvre2copyimagetobuffer4hpu)
#   except:
#       print("(Step 68Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (69) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 69A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4hpu = channel2copyimagetobufferconsume4hpu.recv()
#       print("(Step 69D/700) OF HPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4HPU: ", channel2copyimagetobufferconsume4hpu, " CONTROL & COMMAND: ", message2copyimagereceive4hpu)                                                                                                   
#   except:
#       print('(Step 69Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (70) LET MANOEUVRE2COPYIMAGETOBUFFER4HPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 70A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4hpu = manoeuvre2copyimagetobuffer4hpu(queue2copyimagetobuffer4hpu, channel2copyimagetobufferconsume4hpu, channel2copyimagetobufferpublish4hpu)
#       print("(Step 70B/700) OF HPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4HPU ", response2manoeuvreimagecopytobuffer4hpu)
#   except:
#       print("(Step 70Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2imagetobuffer4hpu(task4copyimagetobuffer2hpu=None):

    '''
    copies 2D images to buffer for HPU
    '''
    global hpu2image4xi                                                                                             # 2D-IMAGES FOR HPU  
    global hpu2buffer4xi                                                                                            # BUFFER FOR HPU
    
    global control2copyimagetobuffer4hpu                                                                            # COMMAND AND CONTROL FOR HPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2imagetobuffer4hpu"
    method2print = "copy2imagetobuffer4hpu"

    message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer4hpu-started: " + str(copy2imagetobuffer4hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                 
    # <function copy2imagetobuffer4hpu at 0x000000000390B948>
    
    message2print = "{(Step 1B/33) passed" + "; task4copyimagetobuffer2hpu-to-copy2imagetobuffer4hpu: " + str(task4copyimagetobuffer2hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE HPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if hpu2image4xi is None:
            hpu2image4xi = create2image4hpu()                                                                              # INITIALIZE IMAGE FOR HPU 
            message2print = "{(Step 1C/70) created" + "; hpu2image4xi-created-by-copy2imagetobuffer4hpu to: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4hpu-skipped-by-copy2imagetobuffer4hpu: " + str(copy2imagetobuffer4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4hpu-skipped-by-copy2imagetobuffer4hpu                                 
    except:
        try:
            hpu2image4xi = create2image4hpu()
            message2print = "{(Step 1E/70) set" + "; hpu2image4xi-set-by-copy2imagetobuffer4hpu to: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            hpu2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; hpu2image4xi-yielded-by-copy2imagetobuffer4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE HPU2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if hpu2buffer4xi is None:
            hpu2buffer4xi = create2buffer4hpu()                                                                     # INITIALIZE BUFFER FOR HPU 
            message2print = "{(Step 1F/70) created" + "; hpu2buffer4xi-created-by-copy2imagetobuffer4hpu to: " + str(hpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4hpu-skipped-by-copy2imagetobuffer4hpu: " + str(copy2imagetobuffer4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                  
    except:
        try:
            hpu2buffer4xi = create2buffer4hpu()
            message2print = "{(Step 1H/70) set" + "; hpu2buffer4xi-set-by-copy2imagetobuffer4hpu to: " + str(hpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0,
            #  'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4hpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}',
            #   'CLASS': 'create2buffer4hpu',
            #   'METHOD': 'create2buffer4hpu'},
            #   'FUNCTION2CALLBACK': <function trace2buffer4hpu at 0x0000000004782B48>,
            #   'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>,
            #   'FLAGS': 0,
            #   'CONTEXT': {'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': ...
            
        except:
            hpu2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; hpu2buffer4xi-yielded-by-copy2imagetobuffer4hpu: " + str(hpu2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASK          
    # -----------------------------------------------------------------------------------------          
              
    if task4copyimagetobuffer2hpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
            
        task4copyimagetobuffer2hpu = {}                                                                                                                                                                                       
            
        message2print = "{(Step 1A/70) initialized" + "; task4copyimagetobuffer2hpu-created-by-copy2imagetobuffer4hpu to: " + str(task4copyimagetobuffer2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
            
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4hpu = trace2image4hpu                                                                 # CALLBACK FUNCTION FOR HPU
            
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4hpu-created-by-copy2imagetobuffer4hpu to: " + str(function2callback4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4hpu at 0x0000000004797548> 
            
        code2ack  = code2ack + '1B1-'
           
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4hpu"                                                            # MODULE OF HPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4hpu"                                                            # FUNCTION OF HPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4hpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR HPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4hpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4hpu-failed; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4hpu', 
        #  'METHOD': 'copy2imagetobuffer4hpu'}; time: 03:40:47 10/04/11 Pacific Daylight Time}


        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2hpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO HPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2hpu['FUNCTION2CALLBACK'] = function2callback4hpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copyimagetobuffer2hpu-created-by-copy2imagetobuffer4hpu to: " + str(task4copyimagetobuffer2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4hpu at 0x0000000004797548>,
        #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4hpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4hpu', 
        #  'METHOD': 'copy2imagetobuffer4hpu'}}; time: 03:40:47 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4hpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4hpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4hpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copyimagetobuffer2hpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2hpu = {}
        message2print = "{(Step 3A/70) trying" + "; context2hpu-trying-by-copy2imagetobuffer4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2hpu = get2context4hpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2hpu-got-by-copy2imagetobuffer4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4hpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4hpu', 
            #  'METHOD': 'create2context4hpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4hpu at 0x000000000394F348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2hpu-failed-by-copy2imagetobuffer4hpu: " + str(context2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
            
        task4copyimagetobuffer2hpu ['CONTEXT'] = context2hpu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2hpu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2hpu-trying-by-copy2imagetobuffer4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2hpu = get2hpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2hpu-got-by-copy2imagetobuffer4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, 
            #  'NATIVE2WIDTHFLOAT4VECTOR': 4, 
            #  'MAX2SIZE4WORKGROUP': 1024, 
            #  'PREFERRED2WIDTHCHAR4VECTOR': 16, 
            #  'SIZE2CACHE': 65536, 
            #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 
            #  'TYPE4CACHE': 'Read/Write', 
            #  'PREFERRED2WIDTHINT4VECTOR': 4, 
            #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 
            #  'NATIVE2WIDTHINT4VECTOR': 4, 
            #  'SIZE2BUFFER4CONSTANT': 65536, 
            #  'EXECUTE2FUNCTION4NATIVE': 'ACK', 
            #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 
            #  'MAX2NUMBER4CONSTANTARGS': 8, 
            #  'VERSION2DRIVER': '2.0', 
            #  'EXTENSIONS': 'cl_amd_fp64 ...
            #  'AVAILABLE2COMPILER': 'ACK', 
            #  'MAX2DIMENSIONS4WORKITEMS': 3, 
            #  'SUPPORT2CORRECTION4ERROR': 0, 
            #  'BITS2ADDRESS': 64, 
            #  'AVAILABLE': 'ACK', 
            #  'SIZE2MEMORY4LOCAL': 32768, 
            #  'VENDOR': 'AuthenticAMD', 
            #  'SUPPORT2IMAGE': 'NACK', 
            #  'MAX2SIZE4KERNELARGUMENT': 4096, 
            #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-...
            #  'OUT-OF-ORDER2QUEUE': 'NACK', 
            #  'ALIGNMENT2ADDRESS4BASE': 1024, 
            #  'PROFILE': 'FULL_PROFILE', 
            #  'TYPE2MEMORY4LOCAL': 'Global', 
            #  'PREFERRED2WIDTHFLOAT4VECTOR': 4, 
            #  'PREFERRED2WIDTHLONG4VECTOR': 2, 
            #  'MAX2FREQUENCY4CLOCK': 3210, 
            #  'PREFERRED2WIDTHSHORT4VECTOR': 8, 
            #  'ID': 4033, 
            #  'MAX2WORKITEM2': 1024, 
            #  'NATIVE2WIDTHDOUBLE4VECTOR': 0, 
            #  'ROUND2ZERO': 'ACK', 
            #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, 
            #   AuthenticAMD', 
            #  'MAX2WORKITEM1': 1024, 
            #  'MAX2WORKITEM0': 1024, 
            #  'SIZE2LINE4CACHE': 64, 
            #  'MAX2ALLOCATION4MEMORY': 1073741824, 
            #  'NATIVE2WIDTHCHAR4VECTOR': 16, 
            #  'QUIET2NANS': 'ACK', 
            #  'NATIVE2WIDTHSHORT4VECTOR': 8, 
            #  'ROUND2NEAREST4EVEN': 'ACK', 
            #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 
            #  'NACK2RETURN': ' ', 
            #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 
            #  'ENDIANESS2DEVICE': 'Little', 
            #  'MINIMUM2ALIGNMENT4DATATYPE': 128, 
            #  'ROUND2+VE3INFINITY': 'ACK', 
            #  'TYPE': 'CL_DEVICE_TYPE_HPU', 
            #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 03:40:47 10/04/11 Pacific Daylight Time}
            
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2hpu-failed-by-copy2imagetobuffer4hpu: " + str(device2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
        
        task4copyimagetobuffer2hpu ['DEVICE'] = device2hpu
        
               
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE SOURCE OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagesourceinfo2hpu = {}
         
        message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2hpu-trying-by-copy2imagetobuffer4hpu: " + str(imagesourceinfo2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagesourceinfo2hpu = get2imageinfo4hpu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagesourceinfo2hpu-got-by-copy2imagetobuffer4hpu: " + str(imagesourceinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000042514E0>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': ...

            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2hpu-failed-by-copy2imagetobuffer4hpu: " + str(imagesourceinfo2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copyimagetobuffer2hpu ['IMAGESOURCE'] = imagesourceinfo2hpu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER DESTINATION OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        bufferdestinationinfo2hpu = {}
         
        message2print = "{(Step 6A/70) trying" + "; bufferdestinationinfo2hpu-trying-by-copy2imagetobuffer4hpu: " + str(bufferdestinationinfo2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            bufferdestinationinfo2hpu = get2bufferinfo4hpu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; bufferdestinationinfo2hpu-got-by-copy2imagetobuffer4hpu: " + str(bufferdestinationinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4hpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4hpu', 
            #  'METHOD': 'create2buffer4hpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4hpu at 0x0000000004782B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; bufferdestinationinfo2hpu-failed-by-copy2imagetobuffer4hpu: " + str(bufferdestinationinfo2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copyimagetobuffer2hpu ['BUFFERDESTINATION'] = bufferdestinationinfo2hpu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY HPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4hpu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copyimagetobuffer4hpu-trying-by-copy2imagetobuffer4hpu: " + str(queue2copyimagetobuffer4hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4hpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copyimagetobuffer4hpu-got-by-copy2imagetobuffer4hpu: " + str(queue2copyimagetobuffer4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x00000000042B2208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copyimagetobuffer4hpu-failed-by-copy2imagetobuffer4hpu: " + str(queue2copyimagetobuffer4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copyimagetobuffer2hpu ['QUEUE'] = queue2copyimagetobuffer4hpu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY HPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2hpu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2hpu-trying-by-copy2imagetobuffer4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2hpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2hpu-got-by-copy2imagetobuffer4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 660>, <read-write PipeConnection, handle 664>)
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2hpu-failed-by-copy2imagetobuffer4hpu: " + str(pipe2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copyimagetobuffer2hpu ['PIPE'] = pipe2hpu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2hpu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2hpu-trying-by-copy2imagetobuffer4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2hpu = get2commandqueueinfo4hpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2hpu-got-by-copy2imagetobuffer4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4hpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2commandqueue4hpu', 
            #  'METHOD': 'create2commandqueue4hpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2commandqueue4hpu at 0x000000000395EA48>, 
            #  'ACK2RETURN': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2hpu-failed-by-copy2imagetobuffer4hpu: " + str(commandqueueinfo2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copyimagetobuffer2hpu ['COMMANDQUEUE'] = commandqueueinfo2hpu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2hpu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2hpu-trying-by-copy2imagetobuffer4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2hpu = get2eventinfo4hpu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2hpu-got-by-copy2imagetobuffer4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4hpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2event4hpu', 
            #  'METHOD': 'create2event4hpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2event4hpu at 0x0000000003971F48>, 
            #  'ACK2RETURN': ' 1D-1E-...
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2hpu-failed-by-copy2imagetobuffer4hpu: " + str(eventinfo2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copyimagetobuffer2hpu ['EVENT'] = eventinfo2hpu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2hpu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2hpu-trying-by-copy2imagetobuffer4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2hpu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2hpu-set-by-copy2imagetobuffer4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2hpu-failed-by-copy2imagetobuffer4hpu: " + str(eventwaitlist2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copyimagetobuffer2hpu ['EVENTWAITLIST'] = eventwaitlist2hpu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2hpu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2hpu-trying-by-copy2imagetobuffer4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2hpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2hpu = eventwaitlistquantity2hpu + 1                                                                      
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2hpu-set-by-copy2imagetobuffer4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2hpu-failed-by-copy2imagetobuffer4hpu: " + str(eventwaitlistquantity2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copyimagetobuffer2hpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2hpu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (14) ORIGIN FOR SOURCE - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        origin4source2hpu = {}
        
        message2print = "{(Step 14A/70) trying" + "; origin4source2hpu-trying-by-copy2imagetobuffer4hpu: " + str(origin4source2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4source2hpu['X'] = 0 
            origin4source2hpu['Y'] = 0  
            origin4source2hpu['Z'] = 0                                                                                                                          
            message2print = "{(Step 14B/70) set" + "; origin4source2hpu-set-by-copy2imagetobuffer4hpu to: " + str(origin4source2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 0, 'X': 0, 'Z': 0} 
            code2ack  = code2ack + '14B-' 
        except:
            message2print = "{(Step 14Z/70) failed" + "; origin4source2hpu-failed-by-copy2imagetobuffer4hpu: " + str(origin4source2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '14Z-'
    
        task4copyimagetobuffer2hpu ['SOURCEORIGIN'] = origin4source2hpu
        
       
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2hpu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2hpu-trying-by-copy2imagetobuffer4hpu: " + str(region4copy2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2hpu['X'] = 10 
            region4copy2hpu['Y'] = 10  
            region4copy2hpu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2hpu-set-by-copy2imagetobuffer4hpu to: " + str(region4copy2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10} 
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2hpu-failed-by-copy2imagetobuffer4hpu: " + str(region4copy2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copyimagetobuffer2hpu ['REGION'] = region4copy2hpu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER                     
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2hpu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2hpu-set-by-copy2imagetobuffer4hpu to: " + str(offset4copy2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '17A-'                
        task4copyimagetobuffer2hpu ['OFFSET'] = offset4copy2hpu            
                    
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
    else:
        pass
                    
    # ---------------------------------------------------------------------------------------------
    # ANALYZE TASK - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2hpu.keys()
    message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2imagetobuffer4hpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}


    if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
        pass
    else:                
        # ---------------------------------------------------------------------------------------------
        # (21B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4hpu = trace2image4hpu                                                             # CALLBACK FUNCTION FOR HPU
        
        message2print = "{(Step 21B1/70) initialized" + "; function2callback4hpu-created-by-copy2imagetobuffer4hpu to: " + str(function2callback4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
            
        code2ack  = code2ack + '21B1-'
           
        args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4hpu"                                                        # MODULE OF HPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4hpu"                                                        # FUNCTION OF HPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4hpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR HPUIZED OPENCL TO RETURN    

        message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4hpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 

        code2ack  = code2ack + '21B2-'
        
        # ---------------------------------------------------------------------------------------------
        # (21C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2hpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO HPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2hpu['FUNCTION2CALLBACK'] = function2callback4hpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '21C-'
        message2print = "{(Step 21C/70) initialized" + "; task4copyimagetobuffer2hpu-created-by-copy2imagetobuffer4hpu to: " + str(task4copyimagetobuffer2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #
             
    if 'PLATFORM' in keys2task:
        pass
    else:  
        # ---------------------------------------------------------------------------------------------
        # (22) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4hpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        try:
            platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4hpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                 
            code2ack  = code2ack + '22B-'
        except:
            message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4hpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '22Z-'
                 
        task4copyimagetobuffer2hpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
    if 'CONTEXT' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (23) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2hpu = {}
        message2print = "{(Step 23A/70) trying" + "; context2hpu-trying-by-copy2imagetobuffer4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2hpu = get2context4hpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
            
            message2print = "{(Step 23B/70) got" + "; context2hpu-got-by-copy2imagetobuffer4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '23B-'         
        except:
            message2print = "{(Step 23Z/70) failed" + "; context2hpu-failed-by-copy2imagetobuffer4hpu: " + str(context2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '23Z-'
            
        task4copyimagetobuffer2hpu ['CONTEXT'] = context2hpu           
    
    if 'DEVICE' in keys2task:
        pass
    else:                   
        # ---------------------------------------------------------------------------------------------
        # (24) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2hpu = {}
        
        message2print = "{(Step 24A/70) trying" + "; device2hpu-trying-by-copy2imagetobuffer4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2hpu = get2hpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 24B/70) got" + "; device2hpu-got-by-copy2imagetobuffer4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
                
            code2ack  = code2ack + '24B-'  
        except:
            message2print = "{(Step 24Z/70) failed" + "; device2hpu-failed-by-copy2imagetobuffer4hpu: " + str(device2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '24Z-'
        
        task4copyimagetobuffer2hpu ['DEVICE'] = device2hpu
    
    if 'IMAGESOURCE' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (25) SOURCE IMAGE - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        sourceimageinfo2hpu = {}
         
        message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2hpu-trying-by-copy2imagetobuffer4hpu: " + str(sourceimageinfo2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            sourceimageinfo2hpu = get2imageinfo4hpu()                                                                         
            
            message2print = "{(Step 25B/70) got" + "; sourceimageinfo2hpu-got-by-copy2imagetobuffer4hpu: " + str(sourceimageinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '25B-'  
        except:
            message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2hpu-failed-by-copy2imagetobuffer4hpu: " + str(sourceimageinfo2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '25Z-'          
                  
        task4copyimagetobuffer2hpu ['IMAGESOURCE'] = sourceimageinfo2hpu         
    
    if 'BUFFERDESTINATION' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (26) DESTINATION BUFFER - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        destinationbufferinfo2hpu = {}
         
        message2print = "{(Step 26A/70) trying" + "; destinationbufferinfo2hpu-trying-by-copy2imagetobuffer4hpu: " + str(destinationbufferinfo2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            destinationbufferinfo2hpu = get2bufferinfo4hpu()                                                                         
            
            message2print = "{(Step 26B/70) got" + "; destinationbufferinfo2hpu-got-by-copy2imagetobuffer4hpu: " + str(destinationbufferinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '26B-'  
        except:
            message2print = "{(Step 26Z/70) failed" + "; destinationbufferinfo2hpu-failed-by-copy2imagetobuffer4hpu: " + str(destinationbufferinfo2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '26Z-'          
                  
        task4copyimagetobuffer2hpu ['BUFFERDESTINATION'] = destinationbufferinfo2hpu    
                                
    if 'QUEUE' in keys2task:
        pass
    else:   
        # ---------------------------------------------------------------------------------------------
        # (27) QUEUE - REQUIRED BY HPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4hpu = None
        
        message2print = "{(Step 27A/70) trying" + "; queue2copyimagetobuffer4hpu-trying-by-copy2imagetobuffer4hpu: " + str(queue2copyimagetobuffer4hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4hpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 27B/70) got" + "; queue2copyimagetobuffer4hpu-got-by-copy2imagetobuffer4hpu: " + str(queue2copyimagetobuffer4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '27B-'  
        except:
            message2print = "{(Step 27Z/70) failed" + "; queue2copyimagetobuffer4hpu-failed-by-copy2imagetobuffer4hpu: " + str(queue2copyimagetobuffer4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '27Z-'
            
        task4copyimagetobuffer2hpu ['QUEUE'] = queue2copyimagetobuffer4hpu
        
    if 'PIPE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) PIPE - REQUIRED BY HPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2hpu = None
       
        message2print = "{(Step 28A/70) trying" + "; pipe2hpu-trying-by-copy2imagetobuffer4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2hpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 28B/70) got" + "; pipe2hpu-got-by-copy2imagetobuffer4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '28B-'  
        except:
            message2print = "{(Step 28Z/70) failed" + "; pipe2hpu-failed-by-copy2imagetobuffer4hpu: " + str(pipe2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '28Z-'
            
        task4copyimagetobuffer2hpu ['PIPE'] = pipe2hpu 
        
    if 'COMMANDQUEUE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2hpu = {}
       
        message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2hpu-trying-by-copy2imagetobuffer4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2hpu = get2commandqueueinfo4hpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 29B/70) got" + "; commandqueueinfo2hpu-got-by-copy2imagetobuffer4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '29B-'  
        except:
            message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2hpu-failed-by-copy2imagetobuffer4hpu: " + str(commandqueueinfo2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '29Z-'
    
        task4copyimagetobuffer2hpu ['COMMANDQUEUE'] = commandqueueinfo2hpu
    
         
    if 'EVENT' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (31) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2hpu = {}
       
        message2print = "{(Step 31A/70) trying" + "; eventinfo2hpu-trying-by-copy2imagetobuffer4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2hpu = get2eventinfo4hpu()                                                                    
            message2print = "{(Step 31B/70) got" + "; eventinfo2hpu-got-by-copy2imagetobuffer4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '31B-' 
        except:
            message2print = "{(Step 31Z/70) failed" + "; eventinfo2hpu-failed-by-copy2imagetobuffer4hpu: " + str(eventinfo2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '31Z-'
    
        task4copyimagetobuffer2hpu ['EVENT'] = eventinfo2hpu 
    
         
    if 'EVENTWAITLIST' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (32) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2hpu = {}
       
        message2print = "{(Step 32A/70) trying" + "; eventwaitlist2hpu-trying-by-copy2imagetobuffer4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2hpu = {}                                                                   
            message2print = "{(Step 32B/70) set" + "; eventwaitlist2hpu-set-by-copy2imagetobuffer4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '32B-' 
        except:
            message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2hpu-failed-by-copy2imagetobuffer4hpu: " + str(eventwaitlist2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '32Z-'
    
        task4copyimagetobuffer2hpu ['EVENTWAITLIST'] = eventwaitlist2hpu                     
          
    if 'EVENTWAITLISTQUANTITY' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2hpu = 0
       
        message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2hpu-trying-by-copy2imagetobuffer4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2hpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2hpu = eventwaitlistquantity2hpu + 1                                                                   
            message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2hpu-set-by-copy2imagetobuffer4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '33B-' 
        except:
            message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2hpu-failed-by-copy2imagetobuffer4hpu: " + str(eventwaitlistquantity2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '33Z-'
    
        task4copyimagetobuffer2hpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2hpu       
                                         
                
    if 'SOURCEORIGIN' in keys2task:
        pass
    else:                    
        # ---------------------------------------------------------------------------------------------
        # (34) SOURCEORIGIN - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        source4origin2hpu = {}
        
        message2print = "{(Step 34A/70) trying" + "; source4origin2hpu-trying-by-copy2imagetobuffer4hpu: " + str(source4origin2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            source4origin2hpu['X'] = 0 
            source4origin2hpu['Y'] = 0  
            source4origin2hpu['Z'] = 0                                                                                                                          
            message2print = "{(Step 34B/70) set" + "; source4origin2hpu-set-by-copy2imagetobuffer4hpu to: " + str(source4origin2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '34B-' 
        except:
            message2print = "{(Step 34Z/70) failed" + "; source4origin2hpu-failed-by-copy2imagetobuffer4hpu: " + str(source4origin2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '34Z-'
    
        task4copyimagetobuffer2hpu ['SOURCEORIGIN'] = source4origin2hpu
             
   
             
    if 'REGION' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (36) REGION - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        region2hpu = {}
        
        message2print = "{(Step 36A/70) trying" + "; region2hpu-trying-by-copy2imagetobuffer4hpu: " + str(region2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            region2hpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
            region2hpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
            region2hpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
            message2print = "{(Step 36B/70) set" + "; region2hpu-set-by-copy2imagetobuffer4hpu to: " + str(region2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '36B-' 
        except:
            message2print = "{(Step 36Z/70) failed" + "; region2hpu-failed-by-copy2imagetobuffer4hpu: " + str(region2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '36Z-'
    
        task4copyimagetobuffer2hpu ['REGION'] = region2hpu
    
         
    if 'OFFSET' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (37) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        offset2hpu = 0
        message2print = "{(Step 37A/70) set" + "; offset2hpu-set-by-copy2imagetobuffer4hpu to: " + str(offset2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        task4copyimagetobuffer2hpu ['OFFSET'] = offset2hpu        
        code2ack  = code2ack + '37A-' 
        
    # ---------------------------------------------------------------------------------------------
    # EOF TASK
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2hpu.keys()
    message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2imagetobuffer4hpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
               


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimagetobuffer4hpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2hpu = False
    channels2hpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2hpu-trying-by-copy2imagetobuffer4hpu: " + str(channels2hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copyimagetobuffer4hpu, publisherchannel2copyimagetobuffer4hpu = task4copyimagetobuffer2hpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copyimagetobuffer4hpu-got-by-copy2imagetobuffer4hpu: " + str(consumerchannel2copyimagetobuffer4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 660>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copyimagetobuffer4hpu-got-by-copy2imagetobuffer4hpu: " + str(publisherchannel2copyimagetobuffer4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 664>                     
        ack4channels2hpu = True
        
        control2copyimagetobuffer4hpu['CONSUMERCHANNEL']  = consumerchannel2copyimagetobuffer4hpu
        
        
        control2copyimagetobuffer4hpu['PUBLISHERCHANNEL'] = publisherchannel2copyimagetobuffer4hpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2hpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copyimagetobuffer4hpu = False
    queue2copyimagetobuffer4hpu = None
    message2print = "{(Step 41A/70) trying" + "queue2copyimagetobuffer4hpu-trying-by-copy2imagetobuffer4hpu: " + str(queue2copyimagetobuffer4hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copyimagetobuffer4hpu = task4copyimagetobuffer2hpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copyimagetobuffer4hpu-got-by-copy2imagetobuffer4hpu: " + str(queue2copyimagetobuffer4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004465E48>
              
        ack4queue2copyimagetobuffer4hpu = True
        control2copyimagetobuffer4hpu['QUEUE']  = queue2copyimagetobuffer4hpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copyimagetobuffer4hpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4hpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4hpu-trying-by-copy2imagetobuffer4hpu: " + str(dummy2image4hpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4hpu = DummyProcess4Xi(target=manoeuvre2copyimagetobuffer4hpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copyimagetobuffer4hpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copyimagetobuffer4hpu, publisherchannel2copyimagetobuffer4hpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4hpu-created-by-copy2imagetobuffer4hpu: " + str(dummy2image4hpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2imagetobuffer4hpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-14B-16B-17A-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2imagetobuffer4hpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimagetobuffer4hpu-returning-by-copy2imagetobuffer4hpu: " + str(control2copyimagetobuffer4hpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004465E48>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 664>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 660>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2copyimagetobuffer4hpu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2buffertoimage4hpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (80) ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4HPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 80A/700) OF HPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4HPU ', time4xi())    
#   response4copy2buffertoimage = {} 
#   task4copy2buffertoimage = None
#   try:                               
#       response4copy2buffertoimage = copy2buffertoimage4hpu(task4copy2buffertoimage)                                                                                                          
#       print('(Step 80B/700) OF HPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4HPU : ', response4copy2buffertoimage)
#       queue2copybuffertoimage4hpu      = response4copy2buffertoimage['QUEUE']
#       channel2copybuffertoimagepublish4hpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimageconsume4hpu = response4copy2buffertoimage['CONSUMERCHANNEL']
#   except:
#       print('(Step 80Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4HPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (81) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4HPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 81A/700) OF HPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4HPU ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4hpu = "<copy4xi>copy2copybuffertoimage4hpu</copy4xi>"
#       channel2copybuffertoimagepublish4hpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4hpu.send(message2copybuffertoimagesend4hpu)
#       print("(Step 81B/700) OF HPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4HPU: ", channel2copybuffertoimagepublish4hpu, " MESSAGE2SEND4HPU: " , message2copybuffertoimagesend4hpu)                                                                                                
#   except:
#       print('(Step 81Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4HPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (82) LET MANOEUVRE2COPYIMAGETOBUFFER4HPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 82C/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4hpu = manoeuvre2copybuffertoimage4hpu(queue2copybuffertoimage4hpu, channel2copybuffertoimageconsume4hpu, channel2copybuffertoimagepublish4hpu)
#       print("(Step 82D/700) OF HPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", manoeuvre2copybuffertoimage4hpu)
#   except:
#       print("(Step 82Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (83) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 83A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4HPU ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4hpu = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4hpu = channel2copybuffertoimageconsume4hpu.recv()
#       print("(Step 83B/700) OF HPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4HPU: ", channel2copybuffertoimageconsume4hpu, " CONTROL & COMMAND: ", message2copyimagereceive4hpu)                                                                                                  
#       # 
#   except:
#       print('(Step 83Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4HPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (84) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4HPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 84A/700) OF HPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4HPU ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4hpu = "<copy4xi>copy2copybuffertoimage4hpu</copy4xi>"
#       channel2copybuffertoimagepublish4hpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4hpu.send(message2copybuffertoimagesend4hpu)
#       print("(Step 84B/700) OF HPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4HPU: ", channel2copybuffertoimagepublish4hpu, " MESSAGE2SEND4HPU: " , message2copybuffertoimagesend4hpu)                                                                                                
#       #  
#   except:
#        print('(Step 84Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4HPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (85) LET MANOEUVRE2COPYIMAGETOBUFFER4HPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 85A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4hpu = manoeuvre2copybuffertoimage4hpu(queue2copybuffertoimage4hpu, channel2copybuffertoimageconsume4hpu, channel2copybuffertoimagepublish4hpu)
#       print("(Step 85B/700) OF HPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", manoeuvre2copybuffertoimage4hpu)
#   except:
#       print("(Step 85Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (86) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 86A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4HPU ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4hpu = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4hpu = channel2copybuffertoimageconsume4hpu.recv()
#       print("(Step 86B/700) OF HPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4HPU: ", channel2copybuffertoimageconsume4hpu, " CONTROL & COMMAND: ", message2copyimagereceive4hpu)                                                                                                  
#       # 
#   except:
#       print('(Step 86Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4HPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (87) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 87A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copybuffertoimage4dragon.put(path2dragon)             
#       print("(Step 87B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 87Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (88) LET MANOEUVRE2COPYIMAGETOBUFFER4HPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 88A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4hpu = manoeuvre2copybuffertoimage4hpu(queue2copybuffertoimage4hpu, channel2copybuffertoimageconsume4hpu, channel2copybuffertoimagepublish4hpu)
#       print("(Step 88B/700) OF HPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", manoeuvre2copybuffertoimage4hpu)
#   except:
#       print("(Step 88Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (89) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 89A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4hpu = channel2copybuffertoimageconsume4hpu.recv()
#       print("(Step 89D/700) OF HPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4HPU: ", channel2copybuffertoimageconsume4hpu, " CONTROL & COMMAND: ", message2copyimagereceive4hpu)                                                                                                   
#   except:
#       print('(Step 89Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (90) LET MANOEUVRE2COPYIMAGETOBUFFER4HPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 90A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4hpu = manoeuvre2copybuffertoimage4hpu(queue2copybuffertoimage4hpu, channel2copybuffertoimageconsume4hpu, channel2copybuffertoimagepublish4hpu)
#       print("(Step 90B/700) OF HPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4HPU ", response2manoeuvreimagecopytobuffer4hpu)
#   except:
#       print("(Step 90Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4HPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2buffertoimage4hpu(task4copybuffertoimage2hpu=None):

    '''
    copies buffer to image for HPU
    '''
    global hpu2image4xi                                                                                             # 2D-IMAGES FOR HPU  
    global hpu2buffer4xi                                                                                            # BUFFER FOR HPU
    
    global control2copybuffertoimage4hpu                                                                            # COMMAND AND CONTROL FOR HPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2buffertoimage4hpu"
    method2print = "copy2buffertoimage4hpu"

    message2print = "{(Step 1A/33) started" + "; copy2buffertoimage4hpu-started: " + str(copy2buffertoimage4hpu)  + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                                
    # <function copy2buffertoimage4hpu at 0x0000000003869548>
    
    message2print = "{(Step 1B/33) passed" + "; task4copybuffertoimage2hpu-to-copy2buffertoimage4hpu: " + str(task4copybuffertoimage2hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE HPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if hpu2image4xi is None:
            hpu2image4xi = create2image4hpu()                                                                              # INITIALIZE IMAGE FOR HPU 
            message2print = "{(Step 1C/70) created" + "; hpu2image4xi-created-by-copy2buffertoimage4hpu to: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4hpu-skipped-by-copy2buffertoimage4hpu: " + str(copy2buffertoimage4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            hpu2image4xi = create2image4hpu()
            message2print = "{(Step 1E/70) set" + "; hpu2image4xi-set-by-copy2buffertoimage4hpu to: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            hpu2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; hpu2image4xi-yielded-by-copy2buffertoimage4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE HPU2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if hpu2buffer4xi is None:
            hpu2buffer4xi = create2buffer4hpu()                                                                     # INITIALIZE BUFFER FOR HPU 
            message2print = "{(Step 1F/70) created" + "; hpu2buffer4xi-created-by-copy2buffertoimage4hpu to: " + str(hpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4hpu-skipped-by-copy2buffertoimage4hpu: " + str(copy2buffertoimage4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            hpu2buffer4xi = create2buffer4hpu()
            message2print = "{(Step 1H/70) set" + "; hpu2buffer4xi-set-by-copy2buffertoimage4hpu to: " + str(hpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            hpu2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; hpu2buffer4xi-yielded-by-copy2buffertoimage4hpu: " + str(hpu2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASKS           
    # -----------------------------------------------------------------------------------------          
    if task4copybuffertoimage2hpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
        
        task4copybuffertoimage2hpu = {}                                                                                                                                                                                       
        
        message2print = "{(Step 1A/70) initialized" + "; task4copybuffertoimage2hpu-created-by-copy2buffertoimage4hpu to: " + str(task4copybuffertoimage2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
        
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4hpu = trace2image4hpu                                                                 # CALLBACK FUNCTION FOR HPU
        
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4hpu-created-by-copy2buffertoimage4hpu to: " + str(function2callback4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4hpu at 0x0000000004645548>
        
        code2ack  = code2ack + '1B1-'
       
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2buffertoimage4hpu"                                                     # MODULE OF HPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2buffertoimage4hpu"                                                     # FUNCTION OF HPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4hpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR HPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4hpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4hpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}',
        #  'CLASS': 'copy2buffertoimage4hpu', 
        #  'METHOD': 'copy2buffertoimage4hpu'}; time: 04:41:15 10/04/11 Pacific Daylight Time}

        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copybuffertoimage2hpu['DATA2CALLBACK'] = args2callback                                             # USER DATA PASSED TO HPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copybuffertoimage2hpu['FUNCTION2CALLBACK'] = function2callback4hpu                                 # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copybuffertoimage2hpu-created-by-copy2buffertoimage4hpu to: " + str(task4copybuffertoimage2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4hpu at 0x0000000004645548>, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4hpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2buffertoimage4hpu', 
        #  'METHOD': 'copy2buffertoimage4hpu'}}; time: 04:41:15 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4hpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2buffertoimage4hpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 
            #  'ack2return': ' 2C-2E-2F-2G-', 
            #  'nack2return': ' ', 
            #  'available2platform': 1, 
            #  'max2entries': 1}; time: 04:41:15 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4hpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copybuffertoimage2hpu['PLATFORM'] = platform2xi                                                    # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2hpu = {}
        message2print = "{(Step 3A/70) trying" + "; context2hpu-trying-by-copy2buffertoimage4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2hpu = get2context4hpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2hpu-got-by-copy2buffertoimage4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2hpu-failed-by-copy2buffertoimage4hpu: " + str(context2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
                
        task4copybuffertoimage2hpu ['CONTEXT'] = context2hpu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2hpu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2hpu-trying-by-copy2buffertoimage4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2hpu = get2hpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2hpu-got-by-copy2buffertoimage4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, ...
               
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2hpu-failed-by-copy2buffertoimage4hpu: " + str(device2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
            
        task4copybuffertoimage2hpu ['DEVICE'] = device2hpu
            
                   
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE DESTINATION OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagedestinationinfo2hpu = {}
             
        message2print = "{(Step 5A/70) trying" + "; imagedestinationinfo2hpu-trying-by-copy2buffertoimage4hpu: " + str(imagedestinationinfo2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagedestinationinfo2hpu = get2imageinfo4hpu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagedestinationinfo2hpu-got-by-copy2buffertoimage4hpu: " + str(imagedestinationinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004105358>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4hpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4hpu', 
            #  'METHOD': 'create2context4hpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4hpu at 0x00000000037FF348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}, 
            #  'PIPE': (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>), 
            #  'COUNT2REFERENCE': 1, 
            #  'FUNCTION2CALLBACK': <function trace2image4hpu at 0x0000000004645548>, 
            #  'COMMANDQUEUE': ...
            
            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagedestinationinfo2hpu-failed-by-copy2buffertoimage4hpu: " + str(imagedestinationinfo2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copybuffertoimage2hpu ['IMAGEDESTINATION'] = imagedestinationinfo2hpu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER SOURCE OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        buffersourceinfo2hpu = {}
         
        message2print = "{(Step 6A/70) trying" + "; buffersourceinfo2hpu-trying-by-copy2buffertoimage4hpu: " + str(buffersourceinfo2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            buffersourceinfo2hpu = get2bufferinfo4hpu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; buffersourceinfo2hpu-got-by-copy2buffertoimage4hpu: " + str(buffersourceinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4hpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4hpu', 
            #  'METHOD': 'create2buffer4hpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4hpu at 0x0000000004631B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000040EBD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': ...
            
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; buffersourceinfo2hpu-failed-by-copy2buffertoimage4hpu: " + str(buffersourceinfo2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copybuffertoimage2hpu ['BUFFERSOURCE'] = buffersourceinfo2hpu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY HPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copybuffertoimage4hpu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copybuffertoimage4hpu-trying-by-copy2buffertoimage4hpu: " + str(queue2copybuffertoimage4hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copybuffertoimage4hpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copybuffertoimage4hpu-got-by-copy2buffertoimage4hpu: " + str(queue2copybuffertoimage4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x000000000492B208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copybuffertoimage4hpu-failed-by-copy2buffertoimage4hpu: " + str(queue2copybuffertoimage4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copybuffertoimage2hpu ['QUEUE'] = queue2copybuffertoimage4hpu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY HPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2hpu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2hpu-trying-by-copy2buffertoimage4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2hpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2hpu-got-by-copy2buffertoimage4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 708>, <read-write PipeConnection, handle 712>) 
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2hpu-failed-by-copy2buffertoimage4hpu: " + str(pipe2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copybuffertoimage2hpu ['PIPE'] = pipe2hpu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2hpu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2hpu-trying-by-copy2buffertoimage4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2hpu = get2commandqueueinfo4hpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2hpu-got-by-copy2buffertoimage4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2hpu-failed-by-copy2buffertoimage4hpu: " + str(commandqueueinfo2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copybuffertoimage2hpu ['COMMANDQUEUE'] = commandqueueinfo2hpu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2hpu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2hpu-trying-by-copy2buffertoimage4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2hpu = get2eventinfo4hpu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2hpu-got-by-copy2buffertoimage4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2hpu-failed-by-copy2buffertoimage4hpu: " + str(eventinfo2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copybuffertoimage2hpu ['EVENT'] = eventinfo2hpu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2hpu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2hpu-trying-by-copy2buffertoimage4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2hpu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2hpu-set-by-copy2buffertoimage4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2hpu-failed-by-copy2buffertoimage4hpu: " + str(eventwaitlist2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copybuffertoimage2hpu ['EVENTWAITLIST'] = eventwaitlist2hpu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2hpu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2hpu-trying-by-copy2buffertoimage4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2hpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2hpu = eventwaitlistquantity2hpu + 1                                                                     
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2hpu-set-by-copy2buffertoimage4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2hpu-failed-by-copy2buffertoimage4hpu: " + str(eventwaitlistquantity2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copybuffertoimage2hpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2hpu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (15) ORIGIN FOR DESTINATION - REQUIRED                             
        # ---------------------------------------------------------------------------------------------              
        origin4destination2hpu = {}
        
        message2print = "{(Step 15A/70) trying" + "; origin4destination2hpu-trying-by-copy2buffertoimage4hpu: " + str(origin4destination2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4destination2hpu['X'] = 100 
            origin4destination2hpu['Y'] = 100  
            origin4destination2hpu['Z'] = 100                                                                                                                          
            message2print = "{(Step 15B/70) set" + "; origin4destination2hpu-set-by-copy2buffertoimage4hpu to: " + str(origin4destination2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 100, 'X': 100, 'Z': 100}
            code2ack  = code2ack + '15B-' 
        except:
            message2print = "{(Step 15Z/70) failed" + "; origin4destination2hpu-failed-by-copy2buffertoimage4hpu: " + str(origin4destination2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '15Z-'
    
        task4copybuffertoimage2hpu ['DESTINATIONORIGIN'] = origin4destination2hpu
                    
        
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2hpu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2hpu-trying-by-copy2buffertoimage4hpu: " + str(region4copy2hpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2hpu['X'] = 10 
            region4copy2hpu['Y'] = 10  
            region4copy2hpu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2hpu-set-by-copy2buffertoimage4hpu to: " + str(region4copy2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10}
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2hpu-failed-by-copy2buffertoimage4hpu: " + str(region4copy2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copybuffertoimage2hpu ['REGION'] = region4copy2hpu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER                   
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2hpu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2hpu-set-by-copy2buffertoimage4hpu to: " + str(offset4copy2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        
        task4copybuffertoimage2hpu ['OFFSET'] = offset4copy2hpu            
                                
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
        
        # ---------------------------------------------------------------------------------------------
        # ANALYZE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2hpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2buffertoimage4hpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:41:15 10/04/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4hpu = trace2image4hpu                                                             # CALLBACK FUNCTION FOR HPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4hpu-created-by-copy2buffertoimage4hpu to: " + str(function2callback4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2buffertoimage4hpu"                                                        # MODULE OF HPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2buffertoimage4hpu"                                                        # FUNCTION OF HPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4hpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR HPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4hpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4copybuffertoimage2hpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO HPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4copybuffertoimage2hpu['FUNCTION2CALLBACK'] = function2callback4hpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4copybuffertoimage2hpu-created-by-copy2buffertoimage4hpu to: " + str(task4copybuffertoimage2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4hpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2buffertoimage4hpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4hpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4copybuffertoimage2hpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2hpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2hpu-trying-by-copy2buffertoimage4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2hpu = get2context4hpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2hpu-got-by-copy2buffertoimage4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2hpu-failed-by-copy2buffertoimage4hpu: " + str(context2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4copybuffertoimage2hpu ['CONTEXT'] = context2hpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2hpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2hpu-trying-by-copy2buffertoimage4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2hpu = get2hpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2hpu-got-by-copy2buffertoimage4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2hpu-failed-by-copy2buffertoimage4hpu: " + str(device2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4copybuffertoimage2hpu ['DEVICE'] = device2hpu
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2hpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; destinationimageinfo2hpu-trying-by-copy2buffertoimage4hpu: " + str(destinationimageinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2hpu = get2imageinfo4hpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; destinationimageinfo2hpu-got-by-copy2buffertoimage4hpu: " + str(destinationimageinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; destinationimageinfo2hpu-failed-by-copy2buffertoimage4hpu: " + str(destinationimageinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4copybuffertoimage2hpu ['IMAGEDESTINATION'] = destinationimageinfo2hpu         
        
        if 'BUFFERSOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) SOURCE BUFFER - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourcebufferinfo2hpu = {}
             
            message2print = "{(Step 26A/70) trying" + "; sourcebufferinfo2hpu-trying-by-copy2buffertoimage4hpu: " + str(sourcebufferinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourcebufferinfo2hpu = get2bufferinfo4hpu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; sourcebufferinfo2hpu-got-by-copy2buffertoimage4hpu: " + str(sourcebufferinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; sourcebufferinfo2hpu-failed-by-copy2buffertoimage4hpu: " + str(sourcebufferinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4copybuffertoimage2hpu ['BUFFERSOURCE'] = sourcebufferinfo2hpu    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY HPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2copybuffertoimage4hpu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2copybuffertoimage4hpu-trying-by-copy2buffertoimage4hpu: " + str(queue2copybuffertoimage4hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2copybuffertoimage4hpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2copybuffertoimage4hpu-got-by-copy2buffertoimage4hpu: " + str(queue2copybuffertoimage4hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2copybuffertoimage4hpu-failed-by-copy2buffertoimage4hpu: " + str(queue2copybuffertoimage4hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4copybuffertoimage2hpu ['QUEUE'] = queue2copybuffertoimage4hpu
            
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY HPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2hpu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2hpu-trying-by-copy2buffertoimage4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2hpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2hpu-got-by-copy2buffertoimage4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2hpu-failed-by-copy2buffertoimage4hpu: " + str(pipe2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4copybuffertoimage2hpu ['PIPE'] = pipe2hpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2hpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2hpu-trying-by-copy2buffertoimage4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2hpu = get2commandqueueinfo4hpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2hpu-got-by-copy2buffertoimage4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2hpu-failed-by-copy2buffertoimage4hpu: " + str(commandqueueinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4copybuffertoimage2hpu ['COMMANDQUEUE'] = commandqueueinfo2hpu
        
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2hpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2hpu-trying-by-copy2buffertoimage4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2hpu = get2eventinfo4hpu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2hpu-got-by-copy2buffertoimage4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2hpu-failed-by-copy2buffertoimage4hpu: " + str(eventinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4copybuffertoimage2hpu ['EVENT'] = eventinfo2hpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2hpu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2hpu-trying-by-copy2buffertoimage4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2hpu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2hpu-set-by-copy2buffertoimage4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2hpu-failed-by-copy2buffertoimage4hpu: " + str(eventwaitlist2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4copybuffertoimage2hpu ['EVENTWAITLIST'] = eventwaitlist2hpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2hpu = 0
           
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2hpu-trying-by-copy2buffertoimage4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2hpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2hpu = eventwaitlistquantity2hpu + 1                                                                     
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2hpu-set-by-copy2buffertoimage4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2hpu-failed-by-copy2buffertoimage4hpu: " + str(eventwaitlistquantity2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4copybuffertoimage2hpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2hpu       
                                             
  
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2hpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; destination4origin2hpu-trying-by-copy2buffertoimage4hpu: " + str(destination4origin2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2hpu['X'] = 0 
                destination4origin2hpu['Y'] = 0  
                destination4origin2hpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; destination4origin2hpu-set-by-copy2buffertoimage4hpu to: " + str(destination4origin2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; destination4origin2hpu-failed-by-copy2buffertoimage4hpu: " + str(destination4origin2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4copybuffertoimage2hpu ['DESTINATIONORIGIN'] = destination4origin2hpu
             
     
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2hpu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2hpu-trying-by-copy2buffertoimage4hpu: " + str(region2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2hpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2hpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2hpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2hpu-set-by-copy2buffertoimage4hpu to: " + str(region2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2hpu-failed-by-copy2buffertoimage4hpu: " + str(region2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4copybuffertoimage2hpu ['REGION'] = region2hpu
        
             
        if 'OFFSET' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (37) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            offset2hpu = 0
            message2print = "{(Step 37A/70) set" + "; offset2hpu-set-by-copy2buffertoimage4hpu to: " + str(offset2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            task4copybuffertoimage2hpu ['OFFSET'] = offset2hpu        
            code2ack  = code2ack + '37A-' 
                    
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2hpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2buffertoimage4hpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:52:11 10/04/11 Pacific Daylight Time}
                


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copybuffertoimage4hpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2hpu = False
    channels2hpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2hpu-trying-by-copy2buffertoimage4hpu: " + str(channels2hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copybuffertoimage4hpu, publisherchannel2copybuffertoimage4hpu = task4copybuffertoimage2hpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copybuffertoimage4hpu-got-by-copy2buffertoimage4hpu: " + str(consumerchannel2copybuffertoimage4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 704>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copybuffertoimage4hpu-got-by-copy2buffertoimage4hpu: " + str(publisherchannel2copybuffertoimage4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 708>                      
        ack4channels2hpu = True
        
        control2copybuffertoimage4hpu['CONSUMERCHANNEL']  = consumerchannel2copybuffertoimage4hpu
        
        
        control2copybuffertoimage4hpu['PUBLISHERCHANNEL'] = publisherchannel2copybuffertoimage4hpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2hpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copybuffertoimage4hpu = False
    queue2copybuffertoimage4hpu = None
    message2print = "{(Step 41A/70) trying" + "queue2copybuffertoimage4hpu-trying-by-copy2buffertoimage4hpu: " + str(queue2copybuffertoimage4hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copybuffertoimage4hpu = task4copybuffertoimage2hpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copybuffertoimage4hpu-got-by-copy2buffertoimage4hpu: " + str(queue2copybuffertoimage4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003DF2198>
              
        ack4queue2copybuffertoimage4hpu = True
        control2copybuffertoimage4hpu['QUEUE']  = queue2copybuffertoimage4hpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copybuffertoimage4hpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4hpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4hpu-trying-by-copy2buffertoimage4hpu: " + str(dummy2image4hpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4hpu = DummyProcess4Xi(target=manoeuvre2copybuffertoimage4hpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copybuffertoimage4hpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copybuffertoimage4hpu, publisherchannel2copybuffertoimage4hpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4hpu-created-by-copy2buffertoimage4hpu: " + str(dummy2image4hpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2buffertoimage4hpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2buffertoimage4hpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
        
    message2print = "{(Step 70C/70) returning" + "; control2copybuffertoimage4hpu-returning-by-copy2buffertoimage4hpu: " + str(control2copybuffertoimage4hpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003DF2198>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 708>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 704>}; time: 04:52:11 10/04/11 Pacific Daylight Time}

        
    return control2copybuffertoimage4hpu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "read2image4hpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4HPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF HPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4HPU ', time4xi())    
#   respon4read2image = {} 
#   task4read2image = None
#   try:                               
#       response4read2image = read2image4hpu(task4read2image)                                                                                                          
#       print('(Step 16B/700) OF HPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4HPU : ', response4read2image)
#       queue2read4hpu      = response4read2image['QUEUE']
#       channel2readpublish4hpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readconsume4hpu = response4read2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4HPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4HPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF HPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4HPU ', time4xi())     
#   try:                               
#       message2readsend4hpu = "<copy4xi>copy2read4hpu</copy4xi>"
#       channel2readpublish4hpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4hpu.send(message2readsend4hpu)
#       print("(Step 17B/700) OF HPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4HPU: ", channel2readpublish4hpu, " MESSAGE2SEND4HPU: " , message2readsend4hpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF HPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4HPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4HPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi()) 
#   try:        
#       response2manoeuvre4hpu = manoeuvre2readimage4hpu(queue2read4hpu, channel2readconsume4hpu, channel2readpublish4hpu)
#       print("(Step 18D/700) OF HPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", manoeuvre2readimage4hpu)
#   except:
#       print("(Step 18Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4HPU ', time4xi())     
#   try:
#       channel2readconsume4hpu = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4hpu = channel2readconsume4hpu.recv()
#       print("(Step 19B/700) OF HPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4HPU: ", channel2readconsume4hpu, " CONTROL & COMMAND: ", message2readreceive4hpu)                                                                                                  
#       # <roger4xi>roger2read4hpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4HPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4HPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF HPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4HPU ', time4xi())     
#   try:                               
#       message2readsend4hpu = "<ready4xi>ready2read4hpu</ready4xi>"
#       channel2readpublish4hpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4hpu.send(message2readsend4hpu)
#       print("(Step 20B/700) OF HPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4HPU: ", channel2readpublish4hpu, " MESSAGE2SEND4HPU: " , message2readsend4hpu)                                                                                                
#       # <ready4xi>ready2read4hpu</ready4xi> 
#   except:
#        print('(Step 20Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4HPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4HPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi()) 
#   try:        
#       response2manoeuvre4hpu = manoeuvre2readimage4hpu(queue2read4hpu, channel2readconsume4hpu, channel2readpublish4hpu)
#       print("(Step 21B/700) OF HPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", manoeuvre2readimage4hpu)
#   except:
#       print("(Step 21Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4HPU ', time4xi())     
#   try:
#       channel2readconsume4hpu = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4hpu = channel2readconsume4hpu.recv()
#       print("(Step 22B/700) OF HPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4HPU: ", channel2readconsume4hpu, " CONTROL & COMMAND: ", message2readreceive4hpu)                                                                                                  
#       # <ready4xi>ready2read4hpu</ready4xi> 
#   except:
#       print('(Step 22Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4HPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2read4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4HPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi()) 
#   try:        
#       response2manoeuvre4hpu = manoeuvre2readimage4hpu(queue2read4hpu, channel2readconsume4hpu, channel2readpublish4hpu)
#       print("(Step 25B/700) OF HPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", manoeuvre2readimage4hpu)
#   except:
#       print("(Step 25Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2readreceive4hpu = channel2readconsume4hpu.recv()
#       print("(Step 26D/700) OF HPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4HPU: ", channel2readconsume4hpu, " CONTROL & COMMAND: ", message2readreceive4hpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4hpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4HPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi()) 
#   try:        
#       response2manoeuvre4hpu = manoeuvre2readimage4hpu(queue2read4hpu, channel2readconsume4hpu, channel2readpublish4hpu)
#       print("(Step 27B/700) OF HPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4HPU ", response2manoeuvre4hpu)
#   except:
#       print("(Step 27Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi())   
#">

# -------------------------------------------------------------------------------------------------   
def read2image4hpu(task4imageread2hpu=None):

    '''
    reads 2D images for HPU
    '''
    global hpu2image4xi                                                     # 2D-IMAGES FOR HPU  
    global control2readimage4hpu                                              # COMMAND AND CONTROL FOR HPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "read2image4hpu"
    method2print = "read2image4hpu"

    message2print = "{(Step 1A/33) started" + "; read2image4hpu-started: " + str(read2image4hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function read2image4hpu at 0x00000000039595C8> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageread2hpu-to-read2image4hpu: " + str(task4imageread2hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE HPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if hpu2image4xi is None:
            hpu2image4xi = create2image4hpu()                                                                              # INITIALIZE IMAGE FOR HPU 
            message2print = "{(Step 1C/70) created" + "; hpu2image4xi-created-by-read2image4hpu to: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4hpu-skipped-by-read2image4hpu: " + str(read2image4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                 
    except:
        try:
            hpu2image4xi = create2image4hpu()
            message2print = "{(Step 1E/70) set" + "; hpu2image4xi-set-by-read2image4hpu to: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            hpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; hpu2image4xi-yielded-by-read2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imageread2hpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imageread2hpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imageread2hpu-created-by-read2image4hpu to: " + str(task4imageread2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4hpu = trace2image4hpu                                                              # CALLBACK FUNCTION FOR HPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4hpu-created-by-read2image4hpu to: " + str(function2callback4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4hpu at 0x0000000004737548>;
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4hpu"                                                          # MODULE OF HPUIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4hpu"                                                          # FUNCTION OF HPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4hpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR HPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-read2image4hpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; read2image4hpu-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4hpu', 
            #  'METHOD': 'read2image4hpu'}; time: 05:06:38 10/04/11 Pacific Daylight Time} 

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2hpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO HPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2hpu['FUNCTION2CALLBACK'] = function2callback4hpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imageread2hpu-created-by-read2image4hpu to: " + str(task4imageread2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4hpu at 0x0000000004737548>, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; read2image4hpu-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4hpu', 
            #  'METHOD': 'read2image4hpu'}}; time: 05:06:38 10/04/11 Pacific Daylight Time}
             
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-read2image4hpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-read2image4hpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-read2image4hpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imageread2hpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2hpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2hpu-trying-by-read2image4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2hpu = get2context4hpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2hpu-got-by-read2image4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
    
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2hpu-failed-by-read2image4hpu: " + str(context2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imageread2hpu ['CONTEXT'] = context2hpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2hpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2hpu-trying-by-read2image4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2hpu = get2hpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2hpu-got-by-read2image4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 
                #  'ID2PLATFORM': '0000000001509568', 
                #  'PROFILING2QUEUE': 'ACK', 
                #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
                #  'NATIVE2WIDTHLONG4VECTOR': 2 ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2hpu-failed-by-read2image4hpu: " + str(device2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imageread2hpu ['DEVICE'] = device2hpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2hpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2hpu-trying-by-read2image4hpu: " + str(imageinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2hpu = get2imageinfo4hpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2hpu-got-by-read2image4hpu: " + str(imageinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041F9320>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2hpu-failed-by-read2image4hpu: " + str(imageinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imageread2hpu ['IMAGE'] = imageinfo2hpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY HPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2hpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2hpu-trying-by-read2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2hpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2hpu-got-by-read2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000004607160> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2hpu-failed-by-read2image4hpu: " + str(queue2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imageread2hpu ['QUEUE'] = queue2hpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY HPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2hpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2hpu-trying-by-read2image4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2hpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2hpu-got-by-read2image4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 408>, <read-write PipeConnection, handle 412>) 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2hpu-failed-by-read2image4hpu: " + str(pipe2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imageread2hpu ['PIPE'] = pipe2hpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2hpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2hpu-trying-by-read2image4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2hpu = get2commandqueueinfo4hpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2hpu-got-by-read2image4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2hpu-failed-by-read2image4hpu: " + str(commandqueueinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imageread2hpu ['COMMANDQUEUE'] = commandqueueinfo2hpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY HPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2hpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2hpu-trying-by-read2image4hpu: " + str(bufferinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2hpu = get2bufferinfo4hpu()                                                                   # SYSTEM CREATES BUFFER FOR HPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2hpu-got-by-read2image4hpu: " + str(bufferinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2hpu-failed-by-read2image4hpu: " + str(bufferinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imageread2hpu ['BUFFER'] = bufferinfo2hpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2hpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2hpu-trying-by-read2image4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2hpu = get2eventinfo4hpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2hpu-got-by-read2image4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2hpu-failed-by-read2image4hpu: " + str(eventinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imageread2hpu ['EVENT'] = eventinfo2hpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2hpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2hpu-trying-by-read2image4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2hpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2hpu-set-by-read2image4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {}
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2hpu-failed-by-read2image4hpu: " + str(eventwaitlist2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imageread2hpu ['EVENTWAITLIST'] = eventwaitlist2hpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2hpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2hpu-trying-by-read2image4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2hpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2hpu = eventwaitlistquantity2hpu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2hpu-set-by-read2image4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2hpu-failed-by-read2image4hpu: " + str(eventwaitlistquantity2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imageread2hpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2hpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2hpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2hpu-trying-by-read2image4hpu: " + str(blocking2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2hpu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2hpu-set-by-read2image4hpu to: " + str(blocking2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2hpu-failed-by-read2image4hpu: " + str(blocking2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imageread2hpu ['BLOCKING'] = blocking2hpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2hpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2hpu-trying-by-read2image4hpu: " + str(origin2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2hpu['X'] = 0 
                origin2hpu['Y'] = 0  
                origin2hpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2hpu-set-by-read2image4hpu to: " + str(origin2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2hpu-failed-by-read2image4hpu: " + str(origin2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imageread2hpu ['ORIGIN'] = origin2hpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2hpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2hpu-trying-by-read2image4hpu: " + str(region2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2hpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2hpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2hpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2hpu-set-by-read2image4hpu to: " + str(region2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2hpu-failed-by-read2image4hpu: " + str(region2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imageread2hpu ['REGION'] = region2hpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2hpu = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2hpu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2hpu-trying-by-read2image4hpu: " + str(rowpitch2hpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2hpu = int(width2hpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2hpu-set-by-read2image4hpu to: " + str(rowpitch2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2hpu-failed-by-read2image4hpu: " + str(rowpitch2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imageread2hpu ['ROWPITCH'] = rowpitch2hpu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2hpu = None
            height2hpu = 800
            size2hpu = int(width2hpu * height2hpu)                                                                  # CALCULATED MEMORY
            if size2hpu == 0:
                size2hpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2hpu-trying-by-read2image4hpu: " + str(pointer2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2hpu = Arena4Xi(size2hpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2hpu-got-by-read2image4hpu: " + str(arena2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x0000000004449EF0>
                
                size2arena = arena2hpu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-read2buffer4hpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000
                
                buffer2arena = arena2hpu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-read2buffer4hpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2hpu:
                    size2hpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2hpu-set-by-read2buffer4hpu to: " + str(size2hpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2hpu = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2hpu-set-by-read2buffer4hpu to: " + str(pointer2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2hpu-failed-by-read2image4hpu: " + str(pointer2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imageread2hpu ['POINTER'] = pointer2hpu                            
            task4imageread2hpu ['SIZE']    = size2hpu                                                               # AUTOMATIC BY HPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2hpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2hpu-trying-by-read2image4hpu: " + str(slicepitch2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2hpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2hpu-set-by-read2image4hpu to: " + str(slicepitch2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2hpu-failed-by-read2image4hpu: " + str(slicepitch2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imageread2hpu ['SLICEPITCH'] = slicepitch2hpu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2hpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-read2image4hpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4hpu = trace2image4hpu                                                              # CALLBACK FUNCTION FOR HPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4hpu-created-by-read2image4hpu to: " + str(function2callback4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4hpu"                                                          # MODULE OF HPUIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4hpu"                                                          # FUNCTION OF HPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4hpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR HPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-read2image4hpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2hpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO HPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2hpu['FUNCTION2CALLBACK'] = function2callback4hpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imageread2hpu-created-by-read2image4hpu to: " + str(task4imageread2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-read2image4hpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-read2image4hpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-read2image4hpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imageread2hpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2hpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2hpu-trying-by-read2image4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2hpu = get2context4hpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2hpu-got-by-read2image4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2hpu-failed-by-read2image4hpu: " + str(context2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imageread2hpu ['CONTEXT'] = context2hpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2hpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2hpu-trying-by-read2image4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2hpu = get2hpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2hpu-got-by-read2image4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2hpu-failed-by-read2image4hpu: " + str(device2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imageread2hpu ['DEVICE'] = device2hpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2hpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2hpu-trying-by-read2image4hpu: " + str(imageinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2hpu = get2imageinfo4hpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2hpu-got-by-read2image4hpu: " + str(imageinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2hpu-failed-by-read2image4hpu: " + str(imageinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imageread2hpu ['IMAGE'] = imageinfo2hpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY HPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2hpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2hpu-trying-by-read2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2hpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2hpu-got-by-read2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2hpu-failed-by-read2image4hpu: " + str(queue2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imageread2hpu ['QUEUE'] = queue2hpu
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY HPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2hpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2hpu-trying-by-read2image4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2hpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2hpu-got-by-read2image4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2hpu-failed-by-read2image4hpu: " + str(pipe2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imageread2hpu ['PIPE'] = pipe2hpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2hpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2hpu-trying-by-read2image4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2hpu = get2commandqueueinfo4hpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2hpu-got-by-read2image4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2hpu-failed-by-read2image4hpu: " + str(commandqueueinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imageread2hpu ['COMMANDQUEUE'] = commandqueueinfo2hpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY HPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2hpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2hpu-trying-by-read2image4hpu: " + str(bufferinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2hpu = get2bufferinfo4hpu()                                                                   # SYSTEM CREATES BUFFER FOR HPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2hpu-got-by-read2image4hpu: " + str(bufferinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2hpu-failed-by-read2image4hpu: " + str(bufferinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imageread2hpu ['BUFFER'] = bufferinfo2hpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2hpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2hpu-trying-by-read2image4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2hpu = get2eventinfo4hpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2hpu-got-by-read2image4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2hpu-failed-by-read2image4hpu: " + str(eventinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imageread2hpu ['EVENT'] = eventinfo2hpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2hpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2hpu-trying-by-read2image4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2hpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2hpu-set-by-read2image4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2hpu-failed-by-read2image4hpu: " + str(eventwaitlist2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imageread2hpu ['EVENTWAITLIST'] = eventwaitlist2hpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2hpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2hpu-trying-by-read2image4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2hpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2hpu = eventwaitlistquantity2hpu + 1                                                                      
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2hpu-set-by-read2image4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2hpu-failed-by-read2image4hpu: " + str(eventwaitlistquantity2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imageread2hpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2hpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2hpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2hpu-trying-by-read2image4hpu: " + str(blocking2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2hpu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2hpu-set-by-read2image4hpu to: " + str(blocking2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2hpu-failed-by-read2image4hpu: " + str(blocking2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imageread2hpu ['BLOCKING'] = blocking2hpu             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2hpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2hpu-trying-by-read2image4hpu: " + str(origin2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2hpu['X'] = 0 
                origin2hpu['Y'] = 0  
                origin2hpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2hpu-set-by-read2image4hpu to: " + str(origin2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2hpu-failed-by-read2image4hpu: " + str(origin2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imageread2hpu ['ORIGIN'] = origin2hpu
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2hpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2hpu-trying-by-read2image4hpu: " + str(region2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2hpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2hpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2hpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2hpu-set-by-read2image4hpu to: " + str(region2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2hpu-failed-by-read2image4hpu: " + str(region2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imageread2hpu ['REGION'] = region2hpu
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2hpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2hpu-trying-by-read2image4hpu: " + str(rowpitch2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2hpu = int(width2hpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2hpu-set-by-read2image4hpu to: " + str(rowpitch2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2hpu-failed-by-read2image4hpu: " + str(rowpitch2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imageread2hpu ['ROWPITCH'] = rowpitch2hpu     
             
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2hpu = None
            size2hpu = int(width2hpu * height2hpu)                                                                  # CALCULATED MEMORY
            if size2hpu == 0:
                size2hpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2hpu-trying-by-read2image4hpu: " + str(pointer2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2hpu = Arena4Xi(size2hpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2hpu-got-by-read2image4hpu: " + str(arena2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2hpu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-read2buffer4hpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2hpu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-read2buffer4hpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2hpu:
                    size2hpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2hpu-set-by-read2buffer4hpu to: " + str(size2hpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2hpu = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2hpu-set-by-read2buffer4hpu to: " + str(pointer2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2hpu-failed-by-read2image4hpu: " + str(pointer2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imageread2hpu ['POINTER'] = pointer2hpu                            
            task4imageread2hpu ['SIZE']    = size2hpu                                                               # AUTOMATIC BY HPUXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2hpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2hpu-trying-by-read2image4hpu: " + str(slicepitch2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2hpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2hpu-set-by-read2image4hpu to: " + str(slicepitch2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2hpu-failed-by-read2image4hpu: " + str(slicepitch2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imageread2hpu ['SLICEPITCH'] = slicepitch2hpu
                              
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2hpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-read2image4hpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}                
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imageread2hpu-yielded-by-read2image4hpu: " + str(task4imageread2hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2readimage4hpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2hpu = False
    channels2hpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2hpu-trying-by-read2image4hpu: " + str(channels2hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>) 
        consumerchannel2hpu, publisherchannel2hpu = task4imageread2hpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2hpu-got-by-read2image4hpu: " + str(consumerchannel2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2hpu-got-by-read2image4hpu: " + str(publisherchannel2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #                      
        ack4channels2hpu = True
        
        control2readimage4hpu['CONSUMERCHANNEL']  = consumerchannel2hpu
        
        
        control2readimage4hpu['PUBLISHERCHANNEL'] = publisherchannel2hpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2hpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2hpu = False
    queue2hpu = None
    message2print = "{(Step 41A/70) trying" + "; queue2hpu-trying-by-read2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2hpu = task4imageread2hpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2hpu-got-by-read2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
              
        ack4queue2hpu = True
        control2readimage4hpu['QUEUE']  = queue2hpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2hpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4hpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4hpu-trying-by-read2image4hpu: " + str(dummy2image4hpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4hpu = DummyProcess4Xi(target=manoeuvre2readimage4hpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2hpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2hpu, publisherchannel2hpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4hpu-created-by-read2image4hpu: " + str(dummy2image4hpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'

    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-read2image4hpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-read2image4hpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #   
        
    message2print = "{(Step 70C/70) returning" + "; control2readimage4hpu-returning-by-read2image4hpu: " + str(control2readimage4hpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003FE3B70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 416>, 
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 412>}
        
    return control2readimage4hpu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "map2image4hpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # ESTABLISH CONTROL & COMMAND                 
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 92A/700) OF HPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4HPU ', time4xi())           
#                    
#   response4map2image = {} 
#   task4map2image = None                                                                                  # WE ONLY WANT TO ESTABLISH CONTROL AND CONTROL FIRST
#
#   try:                               
#       response4map2image = map2image4hpu(task4map2image)                                                                                                          
#       print('(Step 92B/700) OF HPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH MAP2IMAGE4HPU : ', response4map2image)
#    
#       queue4map2image4hpu      = response4map2image['QUEUE']
#       channel4map2imagepublish4hpu = response4map2image['PUBLISHERCHANNEL']
#       channel4map2imageconsume4hpu = response4map2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 92Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4HPU ', time4xi()) 
#        
#   # -----------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGE4HPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------------------      
#   print('(Step 93A/700) OF HPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4HPU ', time4xi())      
#   try:                               
#       message2send4hpu = "<copy4xi>copy2mapimage4hpu</copy4xi>"
#       channel4map2imagepublish4hpu.send(message2send4hpu)
#       print("(Step 93B/700) OF HPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4HPU: ", channel4map2imagepublish4hpu, " MESSAGE2SEND4HPU: " , message2send4hpu)                                                                                                
#   except:
#       print('(Step 93Z1/700) OF HPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4HPU ', time4xi())   
# 
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4HPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4hpu is True:
#       print("(Step 94A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4HPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4hpu(queue4map2image4hpu, channel4map2imageconsume4hpu, channel4map2imagepublish4hpu)
#           print("(Step 94B/700) OF HPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4HPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#        except:
#           print("(Step 94Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4HPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#
#   # ---------------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND ROGER VIA PIPE                  
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 95A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4HPU ', time4xi())      
#   try:
#       message2receive4hpu = channel4map2imageconsume4hpu.recv()
#       print("(Step 95B/700) OF HPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4HPU: ", channel4map2imageconsume4hpu, " CONTROL & COMMAND: ", message2receive4hpu)                                                                                                  
#   except:
#       print('(Step 95Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4HPU ', time4xi())          
#   
#   # ------------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGE4HPU TO RECEIVE                
#   # ------------------------------------------------------------------------------------------------------------      
#   print('(Step 96A/700) OF HPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4HPU ', time4xi())     
#   try:                               
#       message2send4hpu = "<ready4xi>ready2mapimage4hpu</ready4xi>"
#       channel4map2imagepublish4hpu.send(message2send4hpu)
#       print("(Step 96B/700) OF HPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4HPU: ", channel4map2imagepublish4hpu, " MESSAGE2SEND4HPU: " , message2send4hpu)                                                                                                
#   except:
#       print('(Step 84Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4HPU ', time4xi())                                                
# 
#   # -------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4HPU  - LET IT RECEIVE READY AND SEND READY                
#   # -------------------------------------------------------------------------------------------------------       
#   if ack2simulate4hpu is True:
#       print("(Step 97A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4HPU ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4hpu(queue4map2image4hpu, channel4map2imageconsume4hpu, channel4map2imagepublish4hpu)
#           print("(Step 97B/700) OF HPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4HPU ", manoeuvre2copyimage4hpu)
#       except:
#           print("(Step 97Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4HPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#            
#    
#   # ---------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND READY VIA PIPE                
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 98A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4HPU ', time4xi())     
#   try:
#       message2receive4hpu = channel4map2imageconsume4hpu.recv()
#       print("(Step 98B/700) OF HPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4HPU: ", channel4map2imageconsume4hpu, " CONTROL & COMMAND: ", message2receive4hpu)                                                                                                  
#   except:
#       print('(Step 86Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4HPU ', time4xi())          
#                         
#   # -----------------------------------------------------------------------------------------
#   # IMAGE HPUXI COPY OPERATIONS 
#   # -----------------------------------------------------------------------------------------         
#   print('(Step 99A/700) OF HPU2IMAGE4XI5TEST - TRYING FILE OPERATIONS WITH IMAGE HPUXI ', time4xi())       
#   try:        
#       path2dragon = '..//..//..//..//..//web//images//png//dragonxi.png'
#       file2dragon = open(path2dragon)
#       print("(Step 99B/700) OF HPU2IMAGE4XI5TEST - FILENO: ", file2hpu.fileno( ))  
#       print("(Step 99C/700) OF HPU2IMAGE4XI5TEST - MODE: ", file2hpu.mode)
#       print("(Step 99D/700) OF HPU2IMAGE4XI5TEST - NAME: ", file2hpu.name)
#       offset2hpu = 10
#       file2hpu.seek(offset2hpu)
#       print("(Step 99E/700) OF HPU2IMAGE4XI5TEST - PERFORMED SEEK WITH OFFSET: ", offset2hpu)
#       file2hpu.close()
#       print("(Step 99F/700) OF DRAGON2IMAGE4XI5TEST - CLOSED IMAGE FILE: ", path2dragon)  
#   except:
#       print("(Step 87Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM FILE OPERATIONS WITH IMAGE: ", path2dragon)        
# 
#   # -----------------------------------------------------------------------------------------
#   # PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 100A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue4map2image4dragon.put(path2dragon)             
#       print("(Step 100B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#   except:
#       print("(Step 88Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   
#   # -------------------------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4HPU  - STATE III - GETS FROM QUEUE AND SENDS EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4hpu is True:
#       print("(Step 101A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4HPU ", time4xi()) 
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4hpu(queue4map2image4hpu, channel4map2imageconsume4hpu, channel4map2imagepublish4hpu)
#           print("(Step 101B/700) OF HPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4HPU ", manoeuvre2copyimage4hpu)
#       except:
#           print("(Step 101Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4HPU ", time4xi())     
#   else:
#       pass # trust activated thread                     
#                 
#                                  
#   # ----------------------------------------------------------------------------------------------------
#   # RECEIVE ACK OR NACK ABOUT IMAGE OPERATION CONTROL & COMMAND VIA PIPE                
#   # ----------------------------------------------------------------------------------------------------          
#   print('(Step 102A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2receive4hpu = channel4map2imageconsume4hpu.recv()
#       print("(Step 102B/700) OF HPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4HPU: ", channel4map2imageconsume4hpu, " CONTROL & COMMAND: ", message2receive4hpu)                                                                                                   
#   except:
#       print('(Step 90Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#
#   
#   # ----------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4HPU  - LET MANOEUVRE2COPYIMAGE4HPU TO GO TO STATE 0               
#   # ----------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4hpu is True:
#       print("(Step 103A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4HPU ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4hpu(queue4map2image4hpu, channel4map2imageconsume4hpu, channel4map2imagepublish4hpu)
#           print("(Step 103B/700) OF HPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4HPU ", response2manoeuvre4copy)
#       except:
#           print("(Step 103Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4HPU ", time4xi())     
#   else:
#       pass # trust activated thread  
#                                           
#">

# -------------------------------------------------------------------------------------------------   
def map2image4hpu(task4imagemap2hpu=None):

    '''
    maps 2D images for HPU
    '''
    global hpu2image4xi                                                                                             # 2D-IMAGES FOR HPU  
    global control2mapimage4hpu                                                                                     # COMMAND AND CONTROL FOR HPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "map2image4hpu"
    method2print = "map2image4hpu"

    message2print = "{(Step 1A/33) started" + "; map2image4hpu-started: " + str(map2image4hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 1B/33) passed" + "; task4imagemap2hpu-to-map2image4hpu: " + str(task4imagemap2hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE HPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if hpu2image4xi is None:
            hpu2image4xi = create2image4hpu()                                                                              # INITIALIZE IMAGE FOR HPU 
            message2print = "{(Step 1C/70) created" + "; hpu2image4xi-created-by-map2image4hpu to: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4hpu-skipped-by-map2image4hpu: " + str(map2image4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                
    except:
        try:
            hpu2image4xi = create2image4hpu()
            message2print = "{(Step 1E/70) set" + "; hpu2image4xi-set-by-map2image4hpu to: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            hpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; hpu2image4xi-yielded-by-map2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagemap2hpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR MAP - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagemap2hpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagemap2hpu-created-by-map2image4hpu to: " + str(task4imagemap2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4hpu = trace2image4hpu                                                              # CALLBACK FUNCTION FOR HPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4hpu-created-by-map2image4hpu to: " + str(function2callback4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4hpu"                                                          # MODULE OF HPUIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4hpu"                                                          # FUNCTION OF HPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4hpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR HPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-map2image4hpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2hpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO HPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2hpu['FUNCTION2CALLBACK'] = function2callback4hpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagemap2hpu-created-by-map2image4hpu to: " + str(task4imagemap2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-map2image4hpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-map2image4hpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-map2image4hpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagemap2hpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2hpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2hpu-trying-by-map2image4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2hpu = get2context4hpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2hpu-got-by-map2image4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2hpu-failed-by-map2image4hpu: " + str(context2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagemap2hpu ['CONTEXT'] = context2hpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2hpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2hpu-trying-by-map2image4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2hpu = get2hpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2hpu-got-by-map2image4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2hpu-failed-by-map2image4hpu: " + str(device2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagemap2hpu ['DEVICE'] = device2hpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2hpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2hpu-trying-by-map2image4hpu: " + str(imageinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2hpu = get2imageinfo4hpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2hpu-got-by-map2image4hpu: " + str(imageinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2hpu-failed-by-map2image4hpu: " + str(imageinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagemap2hpu ['IMAGE'] = imageinfo2hpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY HPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2hpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2hpu-trying-by-map2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2hpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2hpu-got-by-map2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2hpu-failed-by-map2image4hpu: " + str(queue2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagemap2hpu ['QUEUE'] = queue2hpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY HPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2hpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2hpu-trying-by-map2image4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2hpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2hpu-got-by-map2image4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2hpu-failed-by-map2image4hpu: " + str(pipe2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagemap2hpu ['PIPE'] = pipe2hpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2hpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2hpu-trying-by-map2image4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2hpu = get2commandqueueinfo4hpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2hpu-got-by-map2image4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2hpu-failed-by-map2image4hpu: " + str(commandqueueinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagemap2hpu ['COMMANDQUEUE'] = commandqueueinfo2hpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY HPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2hpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2hpu-trying-by-map2image4hpu: " + str(bufferinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2hpu = get2bufferinfo4hpu()                                                                   # SYSTEM CREATES BUFFER FOR HPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2hpu-got-by-map2image4hpu: " + str(bufferinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2hpu-failed-by-map2image4hpu: " + str(bufferinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagemap2hpu ['BUFFER'] = bufferinfo2hpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2hpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2hpu-trying-by-map2image4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2hpu = get2eventinfo4hpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2hpu-got-by-map2image4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2hpu-failed-by-map2image4hpu: " + str(eventinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagemap2hpu ['EVENT'] = eventinfo2hpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2hpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2hpu-trying-by-map2image4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2hpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2hpu-set-by-map2image4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2hpu-failed-by-map2image4hpu: " + str(eventwaitlist2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagemap2hpu ['EVENTWAITLIST'] = eventwaitlist2hpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2hpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2hpu-trying-by-map2image4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2hpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2hpu = eventwaitlistquantity2hpu + 1                                                                      
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2hpu-set-by-map2image4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2hpu-failed-by-map2image4hpu: " + str(eventwaitlistquantity2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagemap2hpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2hpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2hpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2hpu-trying-by-map2image4hpu: " + str(blocking2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2hpu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2hpu-set-by-map2image4hpu to: " + str(blocking2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2hpu-failed-by-map2image4hpu: " + str(blocking2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagemap2hpu ['BLOCKING'] = blocking2hpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2hpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2hpu-trying-by-map2image4hpu: " + str(origin2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2hpu['X'] = 0 
                origin2hpu['Y'] = 0  
                origin2hpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2hpu-set-by-map2image4hpu to: " + str(origin2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2hpu-failed-by-map2image4hpu: " + str(origin2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagemap2hpu ['ORIGIN'] = origin2hpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2hpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2hpu-trying-by-map2image4hpu: " + str(region2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2hpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2hpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2hpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2hpu-set-by-map2image4hpu to: " + str(region2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2hpu-failed-by-map2image4hpu: " + str(region2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagemap2hpu ['REGION'] = region2hpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2hpu = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2hpu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2hpu-trying-by-map2image4hpu: " + str(rowpitch2hpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2hpu = int(width2hpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2hpu-set-by-map2image4hpu to: " + str(rowpitch2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2hpu-failed-by-map2image4hpu: " + str(rowpitch2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagemap2hpu ['ROWPITCH'] = rowpitch2hpu     
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2hpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2hpu-trying-by-map2image4hpu: " + str(slicepitch2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2hpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2hpu-set-by-map2image4hpu to: " + str(slicepitch2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2hpu-failed-by-map2image4hpu: " + str(slicepitch2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagemap2hpu ['SLICEPITCH'] = slicepitch2hpu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) FLAGS - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            flags2hpu = 'CL_MAP_READ'                                                                
     
            message2print = "{(Step 19A/70) trying" + "; flags2hpu-trying-by-map2image4hpu: " + str(flags2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2hpu = 'CL_MAP_WRITE'  # THE REGION SPECIFIED BY (ORIGIN, REGION) IN THE IMAGE OBJECT IS BEING MAPPED FOR WRITING                                                                                                                     
                message2print = "{(Step 19B/70) set" + "; flags2hpu-set-by-map2image4hpu to: " + str(flags2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '19B-' 
            except:
                message2print = "{(Step 19Z/70) failed" + "; flags2hpu-failed-by-map2image4hpu: " + str(flags2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '19Z-'
        
            task4imagemap2hpu ['FLAGS'] = flags2hpu 
                                  
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2hpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-map2image4hpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}
 

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4hpu = trace2image4hpu                                                              # CALLBACK FUNCTION FOR HPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4hpu-created-by-map2image4hpu to: " + str(function2callback4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4hpu"                                                          # MODULE OF HPUIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4hpu"                                                          # FUNCTION OF HPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4hpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR HPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-map2image4hpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2hpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO HPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2hpu['FUNCTION2CALLBACK'] = function2callback4hpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagemap2hpu-created-by-map2image4hpu to: " + str(task4imagemap2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-map2image4hpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-map2image4hpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-map2image4hpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagemap2hpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2hpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2hpu-trying-by-map2image4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2hpu = get2context4hpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2hpu-got-by-map2image4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2hpu-failed-by-map2image4hpu: " + str(context2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagemap2hpu ['CONTEXT'] = context2hpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2hpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2hpu-trying-by-map2image4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2hpu = get2hpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2hpu-got-by-map2image4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2hpu-failed-by-map2image4hpu: " + str(device2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagemap2hpu ['DEVICE'] = device2hpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2hpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2hpu-trying-by-map2image4hpu: " + str(imageinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2hpu = get2imageinfo4hpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2hpu-got-by-map2image4hpu: " + str(imageinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2hpu-failed-by-map2image4hpu: " + str(imageinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagemap2hpu ['IMAGE'] = imageinfo2hpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY HPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2hpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2hpu-trying-by-map2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2hpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2hpu-got-by-map2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2hpu-failed-by-map2image4hpu: " + str(queue2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagemap2hpu ['QUEUE'] = queue2hpu
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY HPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2hpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2hpu-trying-by-map2image4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2hpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2hpu-got-by-map2image4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2hpu-failed-by-map2image4hpu: " + str(pipe2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagemap2hpu ['PIPE'] = pipe2hpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2hpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2hpu-trying-by-map2image4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2hpu = get2commandqueueinfo4hpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2hpu-got-by-map2image4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2hpu-failed-by-map2image4hpu: " + str(commandqueueinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagemap2hpu ['COMMANDQUEUE'] = commandqueueinfo2hpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY HPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2hpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2hpu-trying-by-map2image4hpu: " + str(bufferinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2hpu = get2bufferinfo4hpu()                                                                   # SYSTEM CREATES BUFFER FOR HPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2hpu-got-by-map2image4hpu: " + str(bufferinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2hpu-failed-by-map2image4hpu: " + str(bufferinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagemap2hpu ['BUFFER'] = bufferinfo2hpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2hpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2hpu-trying-by-map2image4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2hpu = get2eventinfo4hpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2hpu-got-by-map2image4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2hpu-failed-by-map2image4hpu: " + str(eventinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagemap2hpu ['EVENT'] = eventinfo2hpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2hpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2hpu-trying-by-map2image4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2hpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2hpu-set-by-map2image4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2hpu-failed-by-map2image4hpu: " + str(eventwaitlist2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagemap2hpu ['EVENTWAITLIST'] = eventwaitlist2hpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2hpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2hpu-trying-by-map2image4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2hpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2hpu = eventwaitlistquantity2hpu + 1                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2hpu-set-by-map2image4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2hpu-failed-by-map2image4hpu: " + str(eventwaitlistquantity2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagemap2hpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2hpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2hpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2hpu-trying-by-map2image4hpu: " + str(blocking2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2hpu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2hpu-set-by-map2image4hpu to: " + str(blocking2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2hpu-failed-by-map2image4hpu: " + str(blocking2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagemap2hpu ['BLOCKING'] = blocking2hpu             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2hpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2hpu-trying-by-map2image4hpu: " + str(origin2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2hpu['X'] = 0 
                origin2hpu['Y'] = 0  
                origin2hpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2hpu-set-by-map2image4hpu to: " + str(origin2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2hpu-failed-by-map2image4hpu: " + str(origin2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagemap2hpu ['ORIGIN'] = origin2hpu
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2hpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2hpu-trying-by-map2image4hpu: " + str(region2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2hpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2hpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2hpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2hpu-set-by-map2image4hpu to: " + str(region2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2hpu-failed-by-map2image4hpu: " + str(region2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagemap2hpu ['REGION'] = region2hpu
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2hpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2hpu-trying-by-map2image4hpu: " + str(rowpitch2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2hpu = int(width2hpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2hpu-set-by-map2image4hpu to: " + str(rowpitch2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2hpu-failed-by-map2image4hpu: " + str(rowpitch2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagemap2hpu ['ROWPITCH'] = rowpitch2hpu     
   
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2hpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2hpu-trying-by-map2image4hpu: " + str(slicepitch2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2hpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2hpu-set-by-map2image4hpu to: " + str(slicepitch2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2hpu-failed-by-map2image4hpu: " + str(slicepitch2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagemap2hpu ['SLICEPITCH'] = slicepitch2hpu
   
        if 'FLAGS' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (39) FLAGS - REQUIRED                   
            # --------------------------------------------------------------------------------------------------------             
            flags2hpu = 'CL_MAP_READ'                                                                                                             
           
            message2print = "{(Step 39A/70) trying" + "; flags2hpu-trying-by-map2image4hpu: " + str(flags2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2hpu = 'CL_MAP_WRITE'                                                          
                message2print = "{(Step 39B/70) set" + "; flags2hpu-set-by-map2image4hpu to: " + str(flags2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '39B-'                
                 
            except:
                message2print = "{(Step 39Z/70) failed" + "; flags2hpu-failed-by-map2image4hpu: " + str(flags2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '39Z-'
            
            task4imagemap2hpu ['FLAGS'] = flags2hpu
                                          
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2hpu.keys()
        message2print = "{(Step 40A/70) set" + "; keys2task-set-by-map2image4hpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}

                       
    except: 
        message2print = "{(Step 40Z/70) yielded" + "; task4imagemap2hpu-yielded-by-map2image4hpu: " + str(task4imagemap2hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '40Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2mapimage4hpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2hpu = False
    channels2hpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2hpu-trying-by-map2image4hpu: " + str(channels2hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2hpu, publisherchannel2hpu = task4imagemap2hpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2hpu-got-by-map2image4hpu: " + str(consumerchannel2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 748>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2hpu-got-by-map2image4hpu: " + str(publisherchannel2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 752>                     
        ack4channels2hpu = True
        
        control2mapimage4hpu['CONSUMERCHANNEL']  = consumerchannel2hpu
        
        
        control2mapimage4hpu['PUBLISHERCHANNEL'] = publisherchannel2hpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2hpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2hpu = False
    queue2hpu = None
    message2print = "{(Step 41A/70) trying" + "queue2hpu-trying-by-map2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2hpu = task4imagemap2hpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2hpu-got-by-map2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004B3FB70> 
              
        ack4queue2hpu = True
        control2mapimage4hpu['QUEUE']  = queue2hpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2hpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4hpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4hpu-trying-by-map2image4hpu: " + str(dummy2image4hpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4hpu = DummyProcess4Xi(target=manoeuvre2mapimage4hpu,                                            # in this module to handle queue and pipe channels
        #                                   args =(queue2hpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2hpu, publisherchannel2hpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4hpu-created-by-map2image4hpu: " + str(dummy2image4hpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-map2image4hpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-map2image4hpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-18B-19B-40D-41C- 
        
    message2print = "{(Step 70C/70) returning" + "; control2mapimage4hpu-returning-by-map2image4hpu: " + str(control2mapimage4hpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004B3FB70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 752>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 748>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2mapimage4hpu
                    
#</function4xi> 

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimage4hpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4HPU - LET IT TO RECEIVE COPY MESSAGE AND TO SEND ROGER MESSAGE VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4hpu is True:
#       print("(Step 42A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4HPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimage4hpu(queue2copy4hpu, channel2copyconsume4hpu, channel2copypublish4hpu)
#           print("(Step 42B/700) OF HPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4HPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4HPU': '<roger4xi>roger2copy4hpu</roger4xi>', 'ACK2INFO4HPU': False, 'MESSAGE2GET4HPU': '<default4xi>default2copy4hpu</default4xi>', 
#           #  'MESSAGE2RECEIVE4HPU': '<copy4xi>copy2copy4hpu</copy4xi>', 
#           'CODE2ACK4HPU': ' 0D-1C-', 'MESSAGE2PUT4HPU': '<default4xi>default2copy4hpu</default4xi>', 'CODE2NACK4HPU': ' ', 'STATE2MANOEUVRE4HPU': 1}
#        except:
#           print("(Step 42Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4HPU ", time4xi())     
#   else:
#       pass # trust activated thread
#">            

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimage4hpu(queue2copymanoeuvre4hpu, consumerchannel2copymanoeuvre4hpu, publisherchannel2copymanoeuvre4hpu):
    '''
    manoeuvres copy control and command
    '''
    global state2copyimage4hpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagestate4hpu                                                                                        # MAX FOR STATE
    global ack2copyimageinfo4hpu                                                                                         # ACK FOR IMAGE INFO
    global message2copyimageget4hpu                                 
    global message2copyimageput4hpu
    global message2copyimagereceive4hpu
    global message2copyimagesend4hpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimage4hpu" 
    method2print = "manoeuvre2copyimage4hpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimage4hpu-started: " + str(manoeuvre2copyimage4hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2copyimage4hpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2copymanoeuvre4hpu-to-manoeuvre2copyimage4hpu: " + str(queue2copymanoeuvre4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copymanoeuvre4hpu-to-manoeuvre2copyimage4hpu: " + str(consumerchannel2copymanoeuvre4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copymanoeuvre4hpu-to-manoeuvre2copyimage4hpu: " + str(publisherchannel2copymanoeuvre4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copy = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimage4hpu-try-by-manoeuvre2copyimage4hpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimage4hpu is None:
            state2copyimage4hpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimage4hpu-initialized-by-manoeuvre2copyimage4hpu to: " + str(state2copyimage4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimage4hpu-used-by-manoeuvre2copyimage4hpu: " + str(state2copyimage4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimage4hpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimage4hpu-initialized-by-manoeuvre2copyimage4hpu to: " + str(state2copyimage4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4hpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimageinfo4hpu-initialized-by-manoeuvre2copyimage4hpu to: " + str(ack2copyimageinfo4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4hpu     = '<default4xi>default2copy4hpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimageget4hpu-initialized-by-manoeuvre2copyimage4hpu to: " + str(message2copyimageget4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4hpu</default4xi>
        message2copyimageput4hpu     = '<default4xi>default2copy4hpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimageput4hpu-initialized-by-manoeuvre2copyimage4hpu to: " + str(message2copyimageput4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4hpu</default4xi>
        message2copyimagereceive4hpu = '<default4xi>default2copy4hpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagereceive4hpu-initialized-by-manoeuvre2copyimage4hpu to: " + str(message2copyimagereceive4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4hpu</default4xi>
        message2copyimagesend4hpu    = '<default4xi>default2copy4hpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagesend4hpu-initialized-by-manoeuvre2copyimage4hpu to: " + str(message2copyimagesend4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4hpu</default4xi>
        max2copyimagestate4hpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagestate4hpu-initialized-by-manoeuvre2copyimage4hpu to: " + str( max2copyimagestate4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimage4hpu = state2copyimage4hpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimage4hpu-added-by-manoeuvre2copyimage4hpu: " + str(state2copyimage4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2copyimage4hpu > max2copyimagestate4hpu:
        state2copyimage4hpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimage4hpu-set-by-manoeuvre2copyimage4hpu to: " + str(state2copyimage4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4hpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimageinfo4hpu-set-by-manoeuvre2copyimage4hpu to: " + str(ack2copyimageinfo4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4hpu     = '<default4xi>default2copy4hpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimageget4hpu-set-by-manoeuvre2copyimage4hpu to: " + str(message2copyimageget4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4hpu</default4xi>
        message2copyimageput4hpu     = '<default4xi>default2copy4hpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimageput4hpu-set-by-manoeuvre2copyimage4hpu to: " + str(message2copyimageput4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4hpu</default4xi>
        message2copyimagereceive4hpu = '<default4xi>default2copy4hpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagereceive4hpu-set-by-manoeuvre2copyimage4hpu to: " + str(message2copyimagereceive4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4hpu</default4xi>
        message2copyimagesend4hpu    = '<default4xi>default2copy4hpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagesend4hpu-set-by-manoeuvre2copyimage4hpu to: " + str(message2copyimagesend4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4hpu</default4xi>
        max2copyimagestate4hpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagestate4hpu-set-by-manoeuvre2copyimage4hpu to: " + str( max2copyimagestate4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4hpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimage4hpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4hpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4hpu = consumerchannel2copymanoeuvre4hpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagereceive4hpu-received-by-manoeuvre2copyimage4hpu: " + str(message2copyimagereceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2copy4hpu</copy4xi>
                         
            if message2copyimagereceive4hpu == '<copy4xi>copy2copy4hpu</copy4xi>':
                message2copyimagesend4hpu = '<roger4xi>roger2copy4hpu</roger4xi>'
            else:
                message2copyimagesend4hpu = '<negative4xi><negative2copy4hpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4hpu.send(message2copyimagesend4hpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagesend4hpu-sent-by-manoeuvre2copyimage4hpu: " + str(message2copyimagesend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2copy4hpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagereceive4hpu-failed-by-manoeuvre2copyimage4hpu: " + str(message2copyimagereceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4hpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimage4hpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4hpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4hpu = consumerchannel2copymanoeuvre4hpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagereceive4hpu-received-by-manoeuvre2copyimage4hpu: " + str(message2copyimagereceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2copy4hpu</ready4xi>
                         
            if message2copyimagereceive4hpu == '<ready4xi>ready2copy4hpu</ready4xi>':
                message2copyimagesend4hpu = '<ready4xi>ready2copy4hpu</ready4xi>'
            else:
                message2copyimagesend4hpu = '<negative4xi><negative2copy4hpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4hpu.send(message2copyimagesend4hpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagesend4hpu-sent-by-manoeuvre2copyimage4hpu: " + str(message2copyimagesend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagereceive4hpu-failed-by-manoeuvre2copyimage4hpu: " + str(message2copyimagereceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4hpu == 3:    
        info2image4hpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4hpu-try-by-manoeuvre2copyimage4hpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4hpu = queue2copymanoeuvre4hpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4hpu-got-by-manoeuvre2copyimage4hpu: " + str(info2image4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4hpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimageinfo4hpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimageinfo4hpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copymanoeuvre4hpu-failed-by-manoeuvre2copyimage4hpu: " + str(queue2copymanoeuvre4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagesend4hpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimageinfo4hpu is True:
                message2copyimagesend4hpu = '<ack4xi>ack2imageinfo4hpu</ack4xi>'
            else:
                message2copyimagesend4hpu = '<nack4xi>nack2imageinfo4hpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagesend4hpu-set-by-manoeuvre2copyimage4hpu to: " + str(message2copyimagesend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4hpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4hpu-failed-by-manoeuvre2copyimage4hpu: " + str(message2copyimagesend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copymanoeuvre4hpu-calling-by-manoeuvre2copyimage4hpu: " + str(publisherchannel2copymanoeuvre4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagesend4hpu-passing-to-send-by-manoeuvre2copyimage4hpu: " + str(message2copyimagesend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4hpu-failed-by-manoeuvre2copyimage4hpu: " + str(message2copyimagesend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copymanoeuvre4hpu.send(message2copyimagesend4hpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagesend4hpu-sent-by-manoeuvre2copyimage4hpu: " + str(message2copyimagesend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4hpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagesend4hpu-failed-by-manoeuvre2copyimage4hpu: " + str(message2copyimagesend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copy['ACK2INFO4HPU']        = ack2copyimageinfo4hpu
    response2manouvre4copy['CODE2ACK4HPU']        = code2ack
    response2manouvre4copy['CODE2NACK4HPU']       = code2nack
    response2manouvre4copy['MESSAGE2GET4HPU']     = message2copyimageget4hpu
    response2manouvre4copy['MESSAGE2PUT4HPU']     = message2copyimageput4hpu
    response2manouvre4copy['MESSAGE2RECEIVE4HPU'] = message2copyimagereceive4hpu
    response2manouvre4copy['MESSAGE2SEND4HPU']    = message2copyimagesend4hpu
    response2manouvre4copy['STATE2MANOEUVRE4HPU'] = state2copyimage4hpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copy-returning-by-manoeuvre2copyimage4hpu: " + str(response2manouvre4copy) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4HPU': '<roger4xi>roger2copy4hpu</roger4xi>',
    #   'ACK2INFO4HPU': False,
    #   'MESSAGE2GET4HPU': '<default4xi>default2copy4hpu</default4xi>',
    #   'MESSAGE2RECEIVE4HPU': '<copy4xi>copy2copy4hpu</copy4xi>',
    #   'CODE2ACK4HPU': ' 0D-1C-',
    #   'MESSAGE2PUT4HPU': '<default4xi>default2copy4hpu</default4xi>',
    #   'CODE2NACK4HPU': ' ',
    #   'STATE2MANOEUVRE4HPU': 1}; time: 05:25:15 10/03/11 Pacific Daylight Time}

 
    #2 {'MESSAGE2SEND4HPU': '<ready4xi>ready2copy4hpu</ready4xi>',
    #   'ACK2INFO4HPU': False,
    #   'MESSAGE2GET4HPU': '<default4xi>default2copy4hpu</default4xi>',
    #   'MESSAGE2RECEIVE4HPU': '<ready4xi>ready2copy4hpu</ready4xi>',
    #   'CODE2ACK4HPU': ' 0D-2C-',
    #   'MESSAGE2PUT4HPU': '<default4xi>default2copy4hpu</default4xi>',
    #   'CODE2NACK4HPU': ' ',
    #   'STATE2MANOEUVRE4HPU': 2}; time: 05:25:15 10/03/11 Pacific Daylight Time}

   
    #3 {'MESSAGE2SEND4HPU': '<ack4xi>ack2imageinfo4hpu</ack4xi>',
    #   'ACK2INFO4HPU': True,
    #   'MESSAGE2GET4HPU': '<default4xi>default2copy4hpu</default4xi>',
    #   'MESSAGE2RECEIVE4HPU': '<ready4xi>ready2copy4hpu</ready4xi>',
    #   'CODE2ACK4HPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4HPU': '<default4xi>default2copy4hpu</default4xi>',
    #   'CODE2NACK4HPU': ' ', 'STATE2MANOEUVRE4HPU': 3}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    
    #4 {'MESSAGE2SEND4HPU': '<default4xi>default2copy4hpu</default4xi>',
    #   'ACK2INFO4HPU': False,
    #   'MESSAGE2GET4HPU': '<default4xi>default2copy4hpu</default4xi>',
    #   'MESSAGE2RECEIVE4HPU': '<default4xi>default2copy4hpu</default4xi>',
    #   'CODE2ACK4HPU': ' 0D-',
    #   'MESSAGE2PUT4HPU': '<default4xi>default2copy4hpu</default4xi>',
    #   'CODE2NACK4HPU': ' ', 'STATE2MANOEUVRE4HPU': 0}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    return response2manouvre4copy           
          
#</function4xi>


# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimagetobuffer4hpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4HPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4hpu is True:
#       print("(Step 62A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4HPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimagetobuffer4hpu(queue4copy2imagetobuffer4hpu, channel4copy2imagetobufferconsume4hpu, channel4copy2imagetobufferpublish4hpu)
#           print("(Step 62B/700) OF HPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4HPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # { 'MESSAGE2SEND4HPU': '<roger4xi>roger2copyimagetobuffer4hpu</roger4xi>',
#           #   'ACK2INFO4HPU': False,
#           #   'MESSAGE2GET4HPU': '<default4xi>default2copyimagetobuffer4hpu</default4xi>',
#           #   'MESSAGE2RECEIVE4HPU': '<copy4xi>copy2copyimagetobuffer4hpu</copy4xi>',
#           #   'CODE2ACK4HPU': ' 0D-1C-',
#           #   'MESSAGE2PUT4HPU': '<default4xi>default2copyimagetobuffer4hpu</default4xi>',
#           #   'CODE2NACK4HPU': ' ',
#           #   'STATE2MANOEUVRE4HPU': 1}
#        except:
#           print("(Step 62Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4HPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimagetobuffer4hpu(queue2copyimagetobuffermanoeuvre4hpu, consumerchannel2copyimagetobuffermanoeuvre4hpu, publisherchannel2copyimagetobuffermanoeuvre4hpu):
    '''
    manoeuvres copy control and command for copy2imagetobuffer4hpu
    '''
    global state2copyimagetobuffer4hpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagetobufferstate4hpu                                                                                        # MAX FOR STATE
    global ack2copyimagetobufferinfo4hpu                                                                                         # ACK FOR IMAGE INFO
    global message2copyimagetobufferget4hpu                                 
    global message2copyimagetobufferput4hpu
    global message2copyimagetobufferreceive4hpu
    global message2copyimagetobuffersend4hpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimagetobuffer4hpu" 
    method2print = "manoeuvre2copyimagetobuffer4hpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimagetobuffer4hpu-started: " + str(manoeuvre2copyimagetobuffer4hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copyimagetobuffermanoeuvre4hpu-to-manoeuvre2copyimagetobuffer4hpu: " + str(queue2copyimagetobuffermanoeuvre4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copyimagetobuffermanoeuvre4hpu-to-manoeuvre2copyimagetobuffer4hpu: " + str(consumerchannel2copyimagetobuffermanoeuvre4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copyimagetobuffermanoeuvre4hpu-to-manoeuvre2copyimagetobuffer4hpu: " + str(publisherchannel2copyimagetobuffermanoeuvre4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copyimagetobuffer = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimagetobuffer4hpu-try-by-manoeuvre2copyimagetobuffer4hpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimagetobuffer4hpu is None:
            state2copyimagetobuffer4hpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimagetobuffer4hpu-initialized-by-manoeuvre2copyimagetobuffer4hpu to: " + str(state2copyimagetobuffer4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimagetobuffer4hpu-used-by-manoeuvre2copyimagetobuffer4hpu: " + str(state2copyimagetobuffer4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimagetobuffer4hpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimagetobuffer4hpu-initialized-by-manoeuvre2copyimagetobuffer4hpu to: " + str(state2copyimagetobuffer4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimagetobufferinfo4hpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimagetobufferinfo4hpu-initialized-by-manoeuvre2copyimagetobuffer4hpu to: " + str(ack2copyimagetobufferinfo4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimagetobufferget4hpu     = '<default4xi>default2copyimagetobuffer4hpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimagetobufferget4hpu-initialized-by-manoeuvre2copyimagetobuffer4hpu to: " + str(message2copyimagetobufferget4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4hpu     = '<default4xi>default2copyimagetobuffer4hpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimagetobufferput4hpu-initialized-by-manoeuvre2copyimagetobuffer4hpu to: " + str(message2copyimagetobufferput4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4hpu = '<default4xi>default2copyimagetobuffer4hpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagetobufferreceive4hpu-initialized-by-manoeuvre2copyimagetobuffer4hpu to: " + str(message2copyimagetobufferreceive4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4hpu    = '<default4xi>default2copyimagetobuffer4hpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagetobuffersend4hpu-initialized-by-manoeuvre2copyimagetobuffer4hpu to: " + str(message2copyimagetobuffersend4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4hpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagetobufferstate4hpu-initialized-by-manoeuvre2copyimagetobuffer4hpu to: " + str( max2copyimagetobufferstate4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimagetobuffer4hpu = state2copyimagetobuffer4hpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimagetobuffer4hpu-added-by-manoeuvre2copyimagetobuffer4hpu: " + str(state2copyimagetobuffer4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copyimagetobuffer4hpu > max2copyimagetobufferstate4hpu:
        state2copyimagetobuffer4hpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimagetobuffer4hpu-set-by-manoeuvre2copyimagetobuffer4hpu to: " + str(state2copyimagetobuffer4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copyimagetobufferinfo4hpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimagetobufferinfo4hpu-set-by-manoeuvre2copyimagetobuffer4hpu to: " + str(ack2copyimagetobufferinfo4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copyimagetobufferget4hpu     = '<default4xi>default2copyimagetobuffer4hpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimagetobufferget4hpu-set-by-manoeuvre2copyimagetobuffer4hpu to: " + str(message2copyimagetobufferget4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4hpu     = '<default4xi>default2copyimagetobuffer4hpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimagetobufferput4hpu-set-by-manoeuvre2copyimagetobuffer4hpu to: " + str(message2copyimagetobufferput4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4hpu = '<default4xi>default2copyimagetobuffer4hpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagetobufferreceive4hpu-set-by-manoeuvre2copyimagetobuffer4hpu to: " + str(message2copyimagetobufferreceive4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4hpu    = '<default4xi>default2copyimagetobuffer4hpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagetobuffersend4hpu-set-by-manoeuvre2copyimagetobuffer4hpu to: " + str(message2copyimagetobuffersend4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4hpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagetobufferstate4hpu-set-by-manoeuvre2copyimagetobuffer4hpu to: " + str( max2copyimagetobufferstate4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4hpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4hpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4hpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4hpu = consumerchannel2copyimagetobuffermanoeuvre4hpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagetobufferreceive4hpu-received-by-manoeuvre2copyimagetobuffer4hpu: " + str(message2copyimagetobufferreceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4hpu == '<copy4xi>copy2copyimagetobuffer4hpu</copy4xi>': 
                message2copyimagetobuffersend4hpu = '<roger4xi>roger2copyimagetobuffer4hpu</roger4xi>'
            else:
                message2copyimagetobuffersend4hpu = '<negative4xi><negative2copyimagetobuffer4hpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4hpu.send(message2copyimagetobuffersend4hpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagetobuffersend4hpu-sent-by-manoeuvre2copyimagetobuffer4hpu: " + str(message2copyimagetobuffersend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagetobufferreceive4hpu-failed-by-manoeuvre2copyimagetobuffer4hpu: " + str(message2copyimagetobufferreceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4hpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4hpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4hpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4hpu = consumerchannel2copyimagetobuffermanoeuvre4hpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagetobufferreceive4hpu-received-by-manoeuvre2copyimagetobuffer4hpu: " + str(message2copyimagetobufferreceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4hpu == '<ready4xi>ready2copyimagetobuffer4hpu</ready4xi>':
                message2copyimagetobuffersend4hpu = '<ready4xi>ready2copyimagetobuffer4hpu</ready4xi>'
            else:
                message2copyimagetobuffersend4hpu = '<negative4xi><negative2copyimagetobuffer4hpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4hpu.send(message2copyimagetobuffersend4hpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagetobuffersend4hpu-sent-by-manoeuvre2copyimagetobuffer4hpu: " + str(message2copyimagetobuffersend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagetobufferreceive4hpu-failed-by-manoeuvre2copyimagetobuffer4hpu: " + str(message2copyimagetobufferreceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4hpu == 3:    
        info2image4hpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4hpu-try-by-manoeuvre2copyimagetobuffer4hpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4hpu = queue2copyimagetobuffermanoeuvre4hpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4hpu-got-by-manoeuvre2copyimagetobuffer4hpu: " + str(info2image4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4hpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimagetobufferinfo4hpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimagetobufferinfo4hpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copyimagetobuffermanoeuvre4hpu-failed-by-manoeuvre2copyimagetobuffer4hpu: " + str(queue2copyimagetobuffermanoeuvre4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagetobuffersend4hpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimagetobufferinfo4hpu is True:
                message2copyimagetobuffersend4hpu = '<ack4xi>ack2imageinfo4hpu</ack4xi>'
            else:
                message2copyimagetobuffersend4hpu = '<nack4xi>nack2imageinfo4hpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagetobuffersend4hpu-set-by-manoeuvre2copyimagetobuffer4hpu to: " + str(message2copyimagetobuffersend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4hpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4hpu-failed-by-manoeuvre2copyimagetobuffer4hpu: " + str(message2copyimagetobuffersend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copyimagetobuffermanoeuvre4hpu-calling-by-manoeuvre2copyimagetobuffer4hpu: " + str(publisherchannel2copyimagetobuffermanoeuvre4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagetobuffersend4hpu-passing-to-send-by-manoeuvre2copyimagetobuffer4hpu: " + str(message2copyimagetobuffersend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4hpu-failed-by-manoeuvre2copyimagetobuffer4hpu: " + str(message2copyimagetobuffersend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copyimagetobuffermanoeuvre4hpu.send(message2copyimagetobuffersend4hpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagetobuffersend4hpu-sent-by-manoeuvre2copyimagetobuffer4hpu: " + str(message2copyimagetobuffersend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4hpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagetobuffersend4hpu-failed-by-manoeuvre2copyimagetobuffer4hpu: " + str(message2copyimagetobuffersend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copyimagetobuffer['ACK2INFO4HPU']        = ack2copyimagetobufferinfo4hpu
    response2manouvre4copyimagetobuffer['CODE2ACK4HPU']        = code2ack
    response2manouvre4copyimagetobuffer['CODE2NACK4HPU']       = code2nack
    response2manouvre4copyimagetobuffer['MESSAGE2GET4HPU']     = message2copyimagetobufferget4hpu
    response2manouvre4copyimagetobuffer['MESSAGE2PUT4HPU']     = message2copyimagetobufferput4hpu
    response2manouvre4copyimagetobuffer['MESSAGE2RECEIVE4HPU'] = message2copyimagetobufferreceive4hpu
    response2manouvre4copyimagetobuffer['MESSAGE2SEND4HPU']    = message2copyimagetobuffersend4hpu
    response2manouvre4copyimagetobuffer['STATE2MANOEUVRE4HPU'] = state2copyimagetobuffer4hpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copyimagetobuffer-returning-by-manoeuvre2copyimagetobuffer4hpu: " + str(response2manouvre4copyimagetobuffer) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4HPU': '<roger4xi>roger2copyimagetobuffer4hpu</roger4xi>',
    #   'ACK2INFO4HPU': False,
    #   'MESSAGE2GET4HPU': '<default4xi>default2copyimagetobuffer4hpu</default4xi>',
    #   'MESSAGE2RECEIVE4HPU': '<copy4xi>copy2copyimagetobuffer4hpu</copy4xi>',
    #   'CODE2ACK4HPU': ' 0D-1C-',
    #   'MESSAGE2PUT4HPU': '<default4xi>default2copyimagetobuffer4hpu</default4xi>',
    #   'CODE2NACK4HPU': ' ',
    #   'STATE2MANOEUVRE4HPU': 1}; time: 02:00:20 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4HPU': '<ready4xi>ready2copyimagetobuffer4hpu</ready4xi>',
    #   'ACK2INFO4HPU': False,
    #   'MESSAGE2GET4HPU': '<default4xi>default2copyimagetobuffer4hpu</default4xi>',
    #   'MESSAGE2RECEIVE4HPU': '<ready4xi>ready2copyimagetobuffer4hpu</ready4xi>',
    #   'CODE2ACK4HPU': ' 0D-2C-',
    #   'MESSAGE2PUT4HPU': '<default4xi>default2copyimagetobuffer4hpu</default4xi>',
    #   'CODE2NACK4HPU': ' ',
    #   'STATE2MANOEUVRE4HPU': 2}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #3 {'MESSAGE2SEND4HPU': '<ack4xi>ack2imageinfo4hpu</ack4xi>',
    #   'ACK2INFO4HPU': True,
    #   'MESSAGE2GET4HPU': '<default4xi>default2copyimagetobuffer4hpu</default4xi>',
    #   'MESSAGE2RECEIVE4HPU': '<ready4xi>ready2copyimagetobuffer4hpu</ready4xi>',
    #   'CODE2ACK4HPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4HPU': '<default4xi>default2copyimagetobuffer4hpu</default4xi>',
    #   'CODE2NACK4HPU': ' ',
    #   'STATE2MANOEUVRE4HPU': 3}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #4 {'MESSAGE2SEND4HPU': '<default4xi>default2copyimagetobuffer4hpu</default4xi>',
    #   'ACK2INFO4HPU': False,
    #   'MESSAGE2GET4HPU': '<default4xi>default2copyimagetobuffer4hpu</default4xi>',
    #   'MESSAGE2RECEIVE4HPU': '<default4xi>default2copyimagetobuffer4hpu</default4xi>',
    #   'CODE2ACK4HPU': ' 0D-',
    #   'MESSAGE2PUT4HPU': '<default4xi>default2copyimagetobuffer4hpu</default4xi>',
    #   'CODE2NACK4HPU': ' ',
    #   'STATE2MANOEUVRE4HPU': 0}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    return response2manouvre4copyimagetobuffer           
          
#</function4xi>

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copybuffertoimage4hpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4HPU - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4hpu is True:
#       print("(Step 82A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4HPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copybuffertoimage4hpu(queue4copybuffertoimage4hpu, channel4copybuffertoimageconsume4hpu, channel4copybuffertoimagepublish4hpu)
#           print("(Step 82B/700) OF HPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4HPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4HPU': '<roger4xi>roger2copybuffertoimage4hpu</roger4xi>',
#           #   'ACK2INFO4HPU': False,
#           #   'MESSAGE2GET4HPU': '<default4xi>default2copybuffertoimage4hpu</default4xi>',
#           #   'MESSAGE2RECEIVE4HPU': '<copy4xi>copy2copybuffertoimage4hpu</copy4xi>',
#           #   'CODE2ACK4HPU': ' 0D-1C-',
#           #   'MESSAGE2PUT4HPU': '<default4xi>default2copybuffertoimage4hpu</default4xi>',
#           #   'CODE2NACK4HPU': ' ',
#           #   'STATE2MANOEUVRE4HPU': 1}
#       except:
#           print("(Step 82Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4HPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copybuffertoimage4hpu(queue2copybuffertoimagemanoeuvre4hpu, consumerchannel2copybuffertoimagemanoeuvre4hpu, publisherchannel2copybuffertoimagemanoeuvre4hpu):
    '''
    manoeuvres copy control and command for copy2buffertoimage4hpu
    '''
    global state2copybuffertoimage4hpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copybuffertoimagestate4hpu                                                                                        # MAX FOR STATE
    global ack2copybuffertoimageinfo4hpu                                                                                         # ACK FOR IMAGE INFO
    global message2copybuffertoimageget4hpu                                 
    global message2copybuffertoimageput4hpu
    global message2copybuffertoimagereceive4hpu
    global message2copybuffertoimagesend4hpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copybuffertoimage4hpu" 
    method2print = "manoeuvre2copybuffertoimage4hpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copybuffertoimage4hpu-started: " + str(manoeuvre2copybuffertoimage4hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copybuffertoimagemanoeuvre4hpu-to-manoeuvre2copybuffertoimage4hpu: " + str(queue2copybuffertoimagemanoeuvre4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copybuffertoimagemanoeuvre4hpu-to-manoeuvre2copybuffertoimage4hpu: " + str(consumerchannel2copybuffertoimagemanoeuvre4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copybuffertoimagemanoeuvre4hpu-to-manoeuvre2copybuffertoimage4hpu: " + str(publisherchannel2copybuffertoimagemanoeuvre4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copybuffertoimage = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copybuffertoimage4hpu-try-by-manoeuvre2copybuffertoimage4hpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copybuffertoimage4hpu is None:
            state2copybuffertoimage4hpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copybuffertoimage4hpu-initialized-by-manoeuvre2copybuffertoimage4hpu to: " + str(state2copybuffertoimage4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copybuffertoimage4hpu-used-by-manoeuvre2copybuffertoimage4hpu: " + str(state2copybuffertoimage4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copybuffertoimage4hpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copybuffertoimage4hpu-initialized-by-manoeuvre2copybuffertoimage4hpu to: " + str(state2copybuffertoimage4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copybuffertoimageinfo4hpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copybuffertoimageinfo4hpu-initialized-by-manoeuvre2copybuffertoimage4hpu to: " + str(ack2copybuffertoimageinfo4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copybuffertoimageget4hpu     = '<default4xi>default2copybuffertoimage4hpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copybuffertoimageget4hpu-initialized-by-manoeuvre2copybuffertoimage4hpu to: " + str(message2copybuffertoimageget4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4hpu     = '<default4xi>default2copybuffertoimage4hpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copybuffertoimageput4hpu-initialized-by-manoeuvre2copybuffertoimage4hpu to: " + str(message2copybuffertoimageput4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4hpu = '<default4xi>default2copybuffertoimage4hpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copybuffertoimagereceive4hpu-initialized-by-manoeuvre2copybuffertoimage4hpu to: " + str(message2copybuffertoimagereceive4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4hpu    = '<default4xi>default2copybuffertoimage4hpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copybuffertoimagesend4hpu-initialized-by-manoeuvre2copybuffertoimage4hpu to: " + str(message2copybuffertoimagesend4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4hpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copybuffertoimagestate4hpu-initialized-by-manoeuvre2copybuffertoimage4hpu to: " + str( max2copybuffertoimagestate4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copybuffertoimage4hpu = state2copybuffertoimage4hpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copybuffertoimage4hpu-added-by-manoeuvre2copybuffertoimage4hpu: " + str(state2copybuffertoimage4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copybuffertoimage4hpu > max2copybuffertoimagestate4hpu:
        state2copybuffertoimage4hpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copybuffertoimage4hpu-set-by-manoeuvre2copybuffertoimage4hpu to: " + str(state2copybuffertoimage4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copybuffertoimageinfo4hpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copybuffertoimageinfo4hpu-set-by-manoeuvre2copybuffertoimage4hpu to: " + str(ack2copybuffertoimageinfo4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copybuffertoimageget4hpu     = '<default4xi>default2copybuffertoimage4hpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copybuffertoimageget4hpu-set-by-manoeuvre2copybuffertoimage4hpu to: " + str(message2copybuffertoimageget4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4hpu     = '<default4xi>default2copybuffertoimage4hpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copybuffertoimageput4hpu-set-by-manoeuvre2copybuffertoimage4hpu to: " + str(message2copybuffertoimageput4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4hpu = '<default4xi>default2copybuffertoimage4hpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copybuffertoimagereceive4hpu-set-by-manoeuvre2copybuffertoimage4hpu to: " + str(message2copybuffertoimagereceive4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4hpu    = '<default4xi>default2copybuffertoimage4hpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copybuffertoimagesend4hpu-set-by-manoeuvre2copybuffertoimage4hpu to: " + str(message2copybuffertoimagesend4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4hpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copybuffertoimagestate4hpu-set-by-manoeuvre2copybuffertoimage4hpu to: " + str( max2copybuffertoimagestate4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4hpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4hpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4hpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4hpu = consumerchannel2copybuffertoimagemanoeuvre4hpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copybuffertoimagereceive4hpu-received-by-manoeuvre2copybuffertoimage4hpu: " + str(message2copybuffertoimagereceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4hpu == '<copy4xi>copy2copybuffertoimage4hpu</copy4xi>':
                message2copybuffertoimagesend4hpu = '<roger4xi>roger2copybuffertoimage4hpu</roger4xi>'
            else:
                message2copybuffertoimagesend4hpu = '<negative4xi><negative2copybuffertoimage4hpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4hpu.send(message2copybuffertoimagesend4hpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copybuffertoimagesend4hpu-sent-by-manoeuvre2copybuffertoimage4hpu: " + str(message2copybuffertoimagesend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copybuffertoimagereceive4hpu-failed-by-manoeuvre2copybuffertoimage4hpu: " + str(message2copybuffertoimagereceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4hpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4hpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4hpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4hpu = consumerchannel2copybuffertoimagemanoeuvre4hpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copybuffertoimagereceive4hpu-received-by-manoeuvre2copybuffertoimage4hpu: " + str(message2copybuffertoimagereceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4hpu == '<ready4xi>ready2copybuffertoimage4hpu</ready4xi>':
                message2copybuffertoimagesend4hpu = '<ready4xi>ready2copybuffertoimage4hpu</ready4xi>'
            else:
                message2copybuffertoimagesend4hpu = '<negative4xi><negative2copybuffertoimage4hpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4hpu.send(message2copybuffertoimagesend4hpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copybuffertoimagesend4hpu-sent-by-manoeuvre2copybuffertoimage4hpu: " + str(message2copybuffertoimagesend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copybuffertoimagereceive4hpu-failed-by-manoeuvre2copybuffertoimage4hpu: " + str(message2copybuffertoimagereceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4hpu == 3:    
        info2image4hpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4hpu-try-by-manoeuvre2copybuffertoimage4hpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4hpu = queue2copybuffertoimagemanoeuvre4hpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4hpu-got-by-manoeuvre2copybuffertoimage4hpu: " + str(info2image4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4hpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copybuffertoimageinfo4hpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copybuffertoimageinfo4hpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copybuffertoimagemanoeuvre4hpu-failed-by-manoeuvre2copybuffertoimage4hpu: " + str(queue2copybuffertoimagemanoeuvre4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copybuffertoimagesend4hpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copybuffertoimageinfo4hpu is True:
                message2copybuffertoimagesend4hpu = '<ack4xi>ack2imageinfo4hpu</ack4xi>'
            else:
                message2copybuffertoimagesend4hpu = '<nack4xi>nack2imageinfo4hpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copybuffertoimagesend4hpu-set-by-manoeuvre2copybuffertoimage4hpu to: " + str(message2copybuffertoimagesend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4hpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4hpu-failed-by-manoeuvre2copybuffertoimage4hpu: " + str(message2copybuffertoimagesend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copybuffertoimagemanoeuvre4hpu-calling-by-manoeuvre2copybuffertoimage4hpu: " + str(publisherchannel2copybuffertoimagemanoeuvre4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copybuffertoimagesend4hpu-passing-to-send-by-manoeuvre2copybuffertoimage4hpu: " + str(message2copybuffertoimagesend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4hpu-failed-by-manoeuvre2copybuffertoimage4hpu: " + str(message2copybuffertoimagesend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copybuffertoimagemanoeuvre4hpu.send(message2copybuffertoimagesend4hpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copybuffertoimagesend4hpu-sent-by-manoeuvre2copybuffertoimage4hpu: " + str(message2copybuffertoimagesend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4hpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copybuffertoimagesend4hpu-failed-by-manoeuvre2copybuffertoimage4hpu: " + str(message2copybuffertoimagesend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copybuffertoimage['ACK2INFO4HPU']        = ack2copybuffertoimageinfo4hpu
    response2manouvre4copybuffertoimage['CODE2ACK4HPU']        = code2ack
    response2manouvre4copybuffertoimage['CODE2NACK4HPU']       = code2nack
    response2manouvre4copybuffertoimage['MESSAGE2GET4HPU']     = message2copybuffertoimageget4hpu
    response2manouvre4copybuffertoimage['MESSAGE2PUT4HPU']     = message2copybuffertoimageput4hpu
    response2manouvre4copybuffertoimage['MESSAGE2RECEIVE4HPU'] = message2copybuffertoimagereceive4hpu
    response2manouvre4copybuffertoimage['MESSAGE2SEND4HPU']    = message2copybuffertoimagesend4hpu
    response2manouvre4copybuffertoimage['STATE2MANOEUVRE4HPU'] = state2copybuffertoimage4hpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copybuffertoimage-returning-by-manoeuvre2copybuffertoimage4hpu: " + str(response2manouvre4copybuffertoimage) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4HPU': '<roger4xi>roger2copybuffertoimage4hpu</roger4xi>',
    #   'ACK2INFO4HPU': False,
    #   'MESSAGE2GET4HPU': '<default4xi>default2copybuffertoimage4hpu</default4xi>',
    #   'MESSAGE2RECEIVE4HPU': '<copy4xi>copy2copybuffertoimage4hpu</copy4xi>',
    #   'CODE2ACK4HPU': ' 0D-1C-',
    #   'MESSAGE2PUT4HPU': '<default4xi>default2copybuffertoimage4hpu</default4xi>',
    #   'CODE2NACK4HPU': ' ',
    #   'STATE2MANOEUVRE4HPU': 1}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4HPU': '<ready4xi>ready2copybuffertoimage4hpu</ready4xi>',
    #   'ACK2INFO4HPU': False,
    #   'MESSAGE2GET4HPU': '<default4xi>default2copybuffertoimage4hpu</default4xi>',
    #   'MESSAGE2RECEIVE4HPU': '<ready4xi>ready2copybuffertoimage4hpu</ready4xi>',
    #   'CODE2ACK4HPU': ' 0D-2C-',
    #   'MESSAGE2PUT4HPU': '<default4xi>default2copybuffertoimage4hpu</default4xi>',
    #   'CODE2NACK4HPU': ' ',
    #   'STATE2MANOEUVRE4HPU': 2}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    #3 {'MESSAGE2SEND4HPU': '<ack4xi>ack2imageinfo4hpu</ack4xi>',
    #   'ACK2INFO4HPU': True,
    #   'MESSAGE2GET4HPU': '<default4xi>default2copybuffertoimage4hpu</default4xi>',
    #   'MESSAGE2RECEIVE4HPU': '<ready4xi>ready2copybuffertoimage4hpu</ready4xi>',
    #   'CODE2ACK4HPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4HPU': '<default4xi>default2copybuffertoimage4hpu</default4xi>',
    #   'CODE2NACK4HPU': ' ',
    #   'STATE2MANOEUVRE4HPU': 3}; time: 02:18:55 10/04/11 Pacific Daylight Time}
 

    #4 {'MESSAGE2SEND4HPU': '<default4xi>default2copybuffertoimage4hpu</default4xi>',
    #   'ACK2INFO4HPU': False,
    #   'MESSAGE2GET4HPU': '<default4xi>default2copybuffertoimage4hpu</default4xi>',
    #   'MESSAGE2RECEIVE4HPU': '<default4xi>default2copybuffertoimage4hpu</default4xi>',
    #   'CODE2ACK4HPU': ' 0D-',
    #   'MESSAGE2PUT4HPU': '<default4xi>default2copybuffertoimage4hpu</default4xi>',
    #   'CODE2NACK4HPU': ' ',
    #   'STATE2MANOEUVRE4HPU': 0}

    return response2manouvre4copybuffertoimage           
          
#</function4xi>



# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2readimage4hpu" usage = "control function for read2image4hpu function in this module">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2readimage4hpu(queue2readmanoeuvre4hpu, consumerchannel2readmanoeuvre4hpu, publisherchannel2readmanoeuvre4hpu):
    '''
    manoeuvres read control and command
    '''
    global state2read4hpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2readstate4hpu                                                                                        # MAX FOR STATE
    global ack2readinfo4hpu                                                                                         # ACK FOR IMAGE INFO
    global message2readget4hpu                                 
    global message2readput4hpu
    global message2readreceive4hpu
    global message2readsend4hpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2readimage4hpu" 
    method2print = "manoeuvre2readimage4hpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2readimage4hpu-started: " + str(manoeuvre2readimage4hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2readimage4hpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2readmanoeuvre4hpu-to-manoeuvre2readimage4hpu: " + str(queue2readmanoeuvre4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2readmanoeuvre4hpu-to-manoeuvre2readimage4hpu: " + str(consumerchannel2readmanoeuvre4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2readmanoeuvre4hpu-to-manoeuvre2readimage4hpu: " + str(publisherchannel2readmanoeuvre4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4read = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2read4hpu-try-by-manoeuvre2readimage4hpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2read4hpu is None:
            state2read4hpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2read4hpu-initialized-by-manoeuvre2readimage4hpu to: " + str(state2read4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2read4hpu-used-by-manoeuvre2readimage4hpu: " + str(state2read4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2read4hpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2read4hpu-initialized-by-manoeuvre2readimage4hpu to: " + str(state2read4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4hpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2readinfo4hpu-initialized-by-manoeuvre2readimage4hpu to: " + str(ack2readinfo4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4hpu     = '<default4xi>default2read4hpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2readget4hpu-initialized-by-manoeuvre2readimage4hpu to: " + str(message2readget4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4hpu</default4xi>
        message2readput4hpu     = '<default4xi>default2read4hpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2readput4hpu-initialized-by-manoeuvre2readimage4hpu to: " + str(message2readput4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4hpu</default4xi>
        message2readreceive4hpu = '<default4xi>default2read4hpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2readreceive4hpu-initialized-by-manoeuvre2readimage4hpu to: " + str(message2readreceive4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4hpu</default4xi>
        message2readsend4hpu    = '<default4xi>default2read4hpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2readsend4hpu-initialized-by-manoeuvre2readimage4hpu to: " + str(message2readsend4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4hpu</default4xi>
        max2readstate4hpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2readstate4hpu-initialized-by-manoeuvre2readimage4hpu to: " + str( max2readstate4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2read4hpu = state2read4hpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2read4hpu-added-by-manoeuvre2readimage4hpu: " + str(state2read4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2read4hpu > max2readstate4hpu:
        state2read4hpu = 0
        message2print = "{(Step 0I/33) set" + "; state2read4hpu-set-by-manoeuvre2readimage4hpu to: " + str(state2read4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4hpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2readinfo4hpu-set-by-manoeuvre2readimage4hpu to: " + str(ack2readinfo4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4hpu     = '<default4xi>default2read4hpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2readget4hpu-set-by-manoeuvre2readimage4hpu to: " + str(message2readget4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4hpu</default4xi>
        message2readput4hpu     = '<default4xi>default2read4hpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2readput4hpu-set-by-manoeuvre2readimage4hpu to: " + str(message2readput4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4hpu</default4xi>
        message2readreceive4hpu = '<default4xi>default2read4hpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2readreceive4hpu-set-by-manoeuvre2readimage4hpu to: " + str(message2readreceive4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4hpu</default4xi>
        message2readsend4hpu    = '<default4xi>default2read4hpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2readsend4hpu-set-by-manoeuvre2readimage4hpu to: " + str(message2readsend4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4hpu</default4xi>
        max2readstate4hpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2readstate4hpu-set-by-manoeuvre2readimage4hpu to: " + str( max2readstate4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4hpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2readimage4hpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4hpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4hpu = consumerchannel2readmanoeuvre4hpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2readreceive4hpu-received-by-manoeuvre2readimage4hpu: " + str(message2readreceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2read4hpu</copy4xi>
                         
            if message2readreceive4hpu == '<copy4xi>copy2read4hpu</copy4xi>':
                message2readsend4hpu = '<roger4xi>roger2read4hpu</roger4xi>'
            else:
                message2readsend4hpu = '<negative4xi><negative2read4hpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4hpu.send(message2readsend4hpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2readsend4hpu-sent-by-manoeuvre2readimage4hpu: " + str(message2readsend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2read4hpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2readreceive4hpu-failed-by-manoeuvre2readimage4hpu: " + str(message2readreceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4hpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2readimage4hpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4hpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4hpu = consumerchannel2readmanoeuvre4hpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2readreceive4hpu-received-by-manoeuvre2readimage4hpu: " + str(message2readreceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2read4hpu</ready4xi> 
                         
            if message2readreceive4hpu == '<ready4xi>ready2read4hpu</ready4xi>':
                message2readsend4hpu = '<ready4xi>ready2read4hpu</ready4xi>'
            else:
                message2readsend4hpu = '<negative4xi><negative2ready4hpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4hpu.send(message2readsend4hpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2readsend4hpu-sent-by-manoeuvre2readimage4hpu: " + str(message2readsend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2read4hpu</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2readreceive4hpu-failed-by-manoeuvre2readimage4hpu: " + str(message2readreceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4hpu == 3:    
        info2image4hpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4hpu-try-by-manoeuvre2readimage4hpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4hpu = queue2readmanoeuvre4hpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4hpu-got-by-manoeuvre2readimage4hpu: " + str(info2image4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4hpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2readinfo4hpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2readinfo4hpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2readmanoeuvre4hpu-failed-by-manoeuvre2readimage4hpu: " + str(queue2readmanoeuvre4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2readsend4hpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2readinfo4hpu is True:
                message2readsend4hpu = '<ack4xi>ack2imageinfo4hpu</ack4xi>'
            else:
                message2readsend4hpu = '<nack4xi>nack2imageinfo4hpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2readsend4hpu-set-by-manoeuvre2readimage4hpu to: " + str(message2readsend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4hpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4hpu-failed-by-manoeuvre2readimage4hpu: " + str(message2readsend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2readmanoeuvre4hpu-calling-by-manoeuvre2readimage4hpu: " + str(publisherchannel2readmanoeuvre4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2readsend4hpu-passing-to-send-by-manoeuvre2readimage4hpu: " + str(message2readsend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4hpu-failed-by-manoeuvre2readimage4hpu: " + str(message2readsend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2readmanoeuvre4hpu.send(message2readsend4hpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2readsend4hpu-sent-by-manoeuvre2readimage4hpu: " + str(message2readsend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4hpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2readsend4hpu-failed-by-manoeuvre2readimage4hpu: " + str(message2readsend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4read['ACK2INFO4HPU']        = ack2readinfo4hpu
    response2manouvre4read['CODE2ACK4HPU']        = code2ack
    response2manouvre4read['CODE2NACK4HPU']       = code2nack
    response2manouvre4read['MESSAGE2GET4HPU']     = message2readget4hpu
    response2manouvre4read['MESSAGE2PUT4HPU']     = message2readput4hpu
    response2manouvre4read['MESSAGE2RECEIVE4HPU'] = message2readreceive4hpu
    response2manouvre4read['MESSAGE2SEND4HPU']    = message2readsend4hpu
    response2manouvre4read['STATE2MANOEUVRE4HPU'] = state2read4hpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4read-returning-by-manoeuvre2readimage4hpu: " + str(response2manouvre4read) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4HPU': '<roger4xi>roger2read4hpu</roger4xi>',
    #  'ACK2INFO4HPU': False,
    #  'MESSAGE2GET4HPU': '<default4xi>default2read4hpu</default4xi>',
    #  'MESSAGE2RECEIVE4HPU': '<copy4xi>copy2read4hpu</copy4xi>',
    #  'CODE2ACK4HPU': ' 0D-1C-',
    #  'MESSAGE2PUT4HPU': '<default4xi>default2read4hpu</default4xi>',
    #  'CODE2NACK4HPU': ' ', 'STATE2MANOEUVRE4HPU': 1}; time: 01:44:40 09/29/11 Pacific Daylight Time}
 
    #2 {'MESSAGE2SEND4HPU': '<ready4xi>ready2read4hpu</ready4xi>',
    #  'ACK2INFO4HPU': False,
    #  'MESSAGE2GET4HPU': '<default4xi>default2read4hpu</default4xi>',
    #  'MESSAGE2RECEIVE4HPU': '<ready4xi>ready2read4hpu</ready4xi>',
    #  'CODE2ACK4HPU': ' 0D-2C-',
    #  'MESSAGE2PUT4HPU': '<default4xi>default2read4hpu</default4xi>',
    #  'CODE2NACK4HPU': ' ', 'STATE2MANOEUVRE4HPU': 2}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #3 {'MESSAGE2SEND4HPU': '<ack4xi>ack2imageinfo4hpu</ack4xi>',
    #  'ACK2INFO4HPU': True,
    #  'MESSAGE2GET4HPU': '<default4xi>default2read4hpu</default4xi>',
    #  'MESSAGE2RECEIVE4HPU': '<ready4xi>ready2read4hpu</ready4xi>',
    #  'CODE2ACK4HPU': ' 0D-3C-4E-',
    #  'MESSAGE2PUT4HPU': '<default4xi>default2read4hpu</default4xi>',
    #  'CODE2NACK4HPU': ' ',
    #  'STATE2MANOEUVRE4HPU': 3}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4HPU': '<default4xi>default2read4hpu</default4xi>',
    #  'ACK2INFO4HPU': False,
    #  'MESSAGE2GET4HPU': '<default4xi>default2read4hpu</default4xi>',
    #  'MESSAGE2RECEIVE4HPU': '<default4xi>default2read4hpu</default4xi>',
    #  'CODE2ACK4HPU': ' 0D-',
    #  'MESSAGE2PUT4HPU': '<default4xi>default2read4hpu</default4xi>',
    #  'CODE2NACK4HPU': ' ',
    #  'STATE2MANOEUVRE4HPU': 0}; time: 02:11:49 09/29/11 Pacific Daylight Time}


 
    
    
    return response2manouvre4read           
          
#</function4xi>


# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2writeimage4hpu" usage = "control function for write2image4hpu function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2writeimage4hpu(queue2writemanoeuvre4hpu, consumerchannel2writemanoeuvre4hpu, publisherchannel2writemanoeuvre4hpu):
    '''
    manoeuvres control and command
    '''
    global state2write4hpu                                                       # STATE FOR COMMAND AND CONTROL
    global max2writestate4hpu                                                    # MAX FOR STATE
    global ack2writeinfo4hpu                                                     # ACK FOR IMAGE INFO
    global message2writeget4hpu                                 
    global message2writeput4hpu
    global message2writereceive4hpu
    global message2writesend4hpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2writeimage4hpu" 
    method2print = "manoeuvre2writeimage4hpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2writeimage4hpu-started: " + str(manoeuvre2writeimage4hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    # <function manoeuvre2writeimage4hpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2writemanoeuvre4hpu-to-manoeuvre2writeimage4hpu: " + str(queue2writemanoeuvre4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x000000000422D9E8>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2writemanoeuvre4hpu-to-manoeuvre2writeimage4hpu: " + str(consumerchannel2writemanoeuvre4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # read-write PipeConnection, handle 568>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2writemanoeuvre4hpu-to-manoeuvre2writeimage4hpu: " + str(publisherchannel2writemanoeuvre4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 572> 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4write = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2write4hpu-try-by-manoeuvre2writeimage4hpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2write4hpu is None:
            state2write4hpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2write4hpu-initialized-by-manoeuvre2writeimage4hpu to: " + str(state2write4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2write4hpu-used-by-manoeuvre2writeimage4hpu: " + str(state2write4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2write4hpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2write4hpu-initialized-by-manoeuvre2writeimage4hpu to: " + str(state2write4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4hpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2writeinfo4hpu-initialized-by-manoeuvre2writeimage4hpu to: " + str(ack2writeinfo4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4hpu     = '<default4xi>default2write4hpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2writeget4hpu-initialized-by-manoeuvre2writeimage4hpu to: " + str(message2writeget4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4hpu</default4xi>
        message2writeput4hpu     = '<default4xi>default2write4hpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2writeput4hpu-initialized-by-manoeuvre2writeimage4hpu to: " + str(message2writeput4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4hpu</default4xi>
        message2writereceive4hpu = '<default4xi>default2write4hpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2writereceive4hpu-initialized-by-manoeuvre2writeimage4hpu to: " + str(message2writereceive4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4hpu</default4xi>
        message2writesend4hpu    = '<default4xi>default2write4hpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2writesend4hpu-initialized-by-manoeuvre2writeimage4hpu to: " + str(message2writesend4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4hpu</default4xi>
        max2writestate4hpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2writestate4hpu-initialized-by-manoeuvre2writeimage4hpu to: " + str( max2writestate4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2write4hpu = state2write4hpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2write4hpu-added-by-manoeuvre2writeimage4hpu: " + str(state2write4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2write4hpu > max2writestate4hpu:
        state2write4hpu = 0
        message2print = "{(Step 0I/33) set" + "; state2write4hpu-set-by-manoeuvre2writeimage4hpu to: " + str(state2write4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4hpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2writeinfo4hpu-set-by-manoeuvre2writeimage4hpu to: " + str(ack2writeinfo4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4hpu     = '<default4xi>default2write4hpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2writeget4hpu-set-by-manoeuvre2writeimage4hpu to: " + str(message2writeget4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4hpu</default4xi>
        message2writeput4hpu     = '<default4xi>default2write4hpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2writeput4hpu-set-by-manoeuvre2writeimage4hpu to: " + str(message2writeput4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4hpu</default4xi>
        message2writereceive4hpu = '<default4xi>default2write4hpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2writereceive4hpu-set-by-manoeuvre2writeimage4hpu to: " + str(message2writereceive4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4hpu</default4xi>
        message2writesend4hpu    = '<default4xi>default2write4hpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2writesend4hpu-set-by-manoeuvre2writeimage4hpu to: " + str(message2writesend4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4hpu</default4xi>
        max2writestate4hpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2writestate4hpu-set-by-manoeuvre2writeimage4hpu to: " + str( max2writestate4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4hpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2writeimage4hpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4hpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4hpu = consumerchannel2writemanoeuvre4hpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2writereceive4hpu-received-by-manoeuvre2writeimage4hpu: " + str(message2writereceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2write4hpu</copy4xi>
                         
            if message2writereceive4hpu == '<copy4xi>copy2write4hpu</copy4xi>':
                message2writesend4hpu = '<roger4xi>roger2write4hpu</roger4xi>'
            else:
                message2writesend4hpu = '<negative4xi><negative2write4hpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4hpu.send(message2writesend4hpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2writesend4hpu-sent-by-manoeuvre2writeimage4hpu: " + str(message2writesend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2write4hpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2writereceive4hpu-failed-by-manoeuvre2writeimage4hpu: " + str(message2writereceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4hpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2writeimage4hpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4hpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4hpu = consumerchannel2writemanoeuvre4hpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2writereceive4hpu-received-by-manoeuvre2writeimage4hpu: " + str(message2writereceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2write4hpu</ready4xi>
                         
            if message2writereceive4hpu == '<ready4xi>ready2write4hpu</ready4xi>':
                message2writesend4hpu = '<ready4xi>ready2write4hpu</ready4xi>'
            else:
                message2writesend4hpu = '<negative4xi><negative2ready4hpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4hpu.send(message2writesend4hpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2writesend4hpu-sent-by-manoeuvre2writeimage4hpu: " + str(message2writesend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2write4hpu</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2writereceive4hpu-failed-by-manoeuvre2writeimage4hpu: " + str(message2writereceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4hpu == 3:    
        info2image4hpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4hpu-try-by-manoeuvre2writeimage4hpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4hpu = queue2writemanoeuvre4hpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4hpu-got-by-manoeuvre2writeimage4hpu: " + str(info2image4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4hpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2writeinfo4hpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2writeinfo4hpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2writemanoeuvre4hpu-failed-by-manoeuvre2writeimage4hpu: " + str(queue2writemanoeuvre4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2writesend4hpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2writeinfo4hpu is True:
                message2writesend4hpu = '<ack4xi>ack2imageinfo4hpu</ack4xi>'
            else:
                message2writesend4hpu = '<nack4xi>nack2imageinfo4hpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2writesend4hpu-set-by-manoeuvre2writeimage4hpu to: " + str(message2writesend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4hpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4hpu-failed-by-manoeuvre2writeimage4hpu: " + str(message2writesend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2writemanoeuvre4hpu-calling-by-manoeuvre2writeimage4hpu: " + str(publisherchannel2writemanoeuvre4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2writesend4hpu-passing-to-send-by-manoeuvre2writeimage4hpu: " + str(message2writesend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4hpu-failed-by-manoeuvre2writeimage4hpu: " + str(message2writesend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2writemanoeuvre4hpu.send(message2writesend4hpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2writesend4hpu-sent-by-manoeuvre2writeimage4hpu: " + str(message2writesend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4hpu</ack4xi>          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2writesend4hpu-failed-by-manoeuvre2writeimage4hpu: " + str(message2writesend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4write['ACK2INFO4HPU']        = ack2writeinfo4hpu
    response2manouvre4write['CODE2ACK4HPU']        = code2ack
    response2manouvre4write['CODE2NACK4HPU']       = code2nack
    response2manouvre4write['MESSAGE2GET4HPU']     = message2writeget4hpu
    response2manouvre4write['MESSAGE2PUT4HPU']     = message2writeput4hpu
    response2manouvre4write['MESSAGE2RECEIVE4HPU'] = message2writereceive4hpu
    response2manouvre4write['MESSAGE2SEND4HPU']    = message2writesend4hpu
    response2manouvre4write['STATE2MANOEUVRE4HPU'] = state2write4hpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4write-returning-by-manoeuvre2writeimage4hpu: " + str(response2manouvre4write) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1
    # {'MESSAGE2SEND4HPU': '<roger4xi>roger2write4hpu</roger4xi>', 
    #  'ACK2INFO4HPU': False, 'MESSAGE2GET4HPU': '<default4xi>default2write4hpu</default4xi>', 
    #  'MESSAGE2RECEIVE4HPU': '<copy4xi>copy2write4hpu</copy4xi>', 
    #  'CODE2ACK4HPU': ' 0D-1C-', 
    #  'MESSAGE2PUT4HPU': '<default4xi>default2write4hpu</default4xi>', 
    #  'CODE2NACK4HPU': ' ', 
    #  'STATE2MANOEUVRE4HPU': 1}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #2
    # {'MESSAGE2SEND4HPU': '<ready4xi>ready2write4hpu</ready4xi>', 
    #  'ACK2INFO4HPU': False, 'MESSAGE2GET4HPU': '<default4xi>default2write4hpu</default4xi>', 
    #  'MESSAGE2RECEIVE4HPU': '<ready4xi>ready2write4hpu</ready4xi>', 
    #  'CODE2ACK4HPU': ' 0D-2C-', 
    #  'MESSAGE2PUT4HPU': '<default4xi>default2write4hpu</default4xi>', 
    #  'CODE2NACK4HPU': ' ', 
    #  'STATE2MANOEUVRE4HPU': 2}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #3
    # {'MESSAGE2SEND4HPU': '<ack4xi>ack2imageinfo4hpu</ack4xi>', 
    #  'ACK2INFO4HPU': True, 'MESSAGE2GET4HPU': '<default4xi>default2write4hpu</default4xi>', 
    #  'MESSAGE2RECEIVE4HPU': '<ready4xi>ready2write4hpu</ready4xi>', 
    #  'CODE2ACK4HPU': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4HPU': '<default4xi>default2write4hpu</default4xi>', 
    #  'CODE2NACK4HPU': ' ', 
    #  'STATE2MANOEUVRE4HPU': 3}; time: 00:11:53 10/03/11 Pacific Daylight Time}

    return response2manouvre4write           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2mapimage4hpu" usage = "control function for map2image4hpu function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2mapimage4hpu(queue2mapmanoeuvre4hpu, consumerchannel2mapmanoeuvre4hpu, publisherchannel2mapmanoeuvre4hpu):
    '''
    manoeuvres control and command
    '''
    global state2map4hpu                                                       # STATE FOR COMMAND AND CONTROL
    global max2mapstate4hpu                                                    # MAX FOR STATE
    global ack2mapinfo4hpu                                                     # ACK FOR IMAGE INFO
    global message2mapget4hpu                                 
    global message2mapput4hpu
    global message2mapreceive4hpu
    global message2mapsend4hpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2mapimage4hpu" 
    method2print = "manoeuvre2mapimage4hpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2mapimage4hpu-started: " + str(manoeuvre2mapimage4hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    #  
    
    message2print = "{(Step 0B/33) passed" + "; queue2mapmanoeuvre4hpu-to-manoeuvre2mapimage4hpu: " + str(queue2mapmanoeuvre4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2mapmanoeuvre4hpu-to-manoeuvre2mapimage4hpu: " + str(consumerchannel2mapmanoeuvre4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2mapmanoeuvre4hpu-to-manoeuvre2mapimage4hpu: " + str(publisherchannel2mapmanoeuvre4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4map = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2map4hpu-try-by-manoeuvre2mapimage4hpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2map4hpu is None:
            state2map4hpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2map4hpu-initialized-by-manoeuvre2mapimage4hpu to: " + str(state2map4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2map4hpu-used-by-manoeuvre2mapimage4hpu: " + str(state2map4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2map4hpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2map4hpu-initialized-by-manoeuvre2mapimage4hpu to: " + str(state2map4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4hpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2mapinfo4hpu-initialized-by-manoeuvre2mapimage4hpu to: " + str(ack2mapinfo4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4hpu     = '<default4xi>default2map4hpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2mapget4hpu-initialized-by-manoeuvre2mapimage4hpu to: " + str(message2mapget4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4hpu</default4xi>
        message2mapput4hpu     = '<default4xi>default2map4hpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2mapput4hpu-initialized-by-manoeuvre2mapimage4hpu to: " + str(message2mapput4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4hpu</default4xi>
        message2mapreceive4hpu = '<default4xi>default2map4hpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2mapreceive4hpu-initialized-by-manoeuvre2mapimage4hpu to: " + str(message2mapreceive4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4hpu</default4xi>
        message2mapsend4hpu    = '<default4xi>default2map4hpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2mapsend4hpu-initialized-by-manoeuvre2mapimage4hpu to: " + str(message2mapsend4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4hpu</default4xi>
        max2mapstate4hpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2mapstate4hpu-initialized-by-manoeuvre2mapimage4hpu to: " + str( max2mapstate4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2map4hpu = state2map4hpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2map4hpu-added-by-manoeuvre2mapimage4hpu: " + str(state2map4hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2map4hpu > max2mapstate4hpu:
        state2map4hpu = 0
        message2print = "{(Step 0I/33) set" + "; state2map4hpu-set-by-manoeuvre2mapimage4hpu to: " + str(state2map4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4hpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2mapinfo4hpu-set-by-manoeuvre2mapimage4hpu to: " + str(ack2mapinfo4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4hpu     = '<default4xi>default2map4hpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2mapget4hpu-set-by-manoeuvre2mapimage4hpu to: " + str(message2mapget4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4hpu</default4xi>
        message2mapput4hpu     = '<default4xi>default2map4hpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2mapput4hpu-set-by-manoeuvre2mapimage4hpu to: " + str(message2mapput4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4hpu</default4xi>
        message2mapreceive4hpu = '<default4xi>default2map4hpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2mapreceive4hpu-set-by-manoeuvre2mapimage4hpu to: " + str(message2mapreceive4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4hpu</default4xi>
        message2mapsend4hpu    = '<default4xi>default2map4hpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2mapsend4hpu-set-by-manoeuvre2mapimage4hpu to: " + str(message2mapsend4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4hpu</default4xi>
        max2mapstate4hpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2mapstate4hpu-set-by-manoeuvre2mapimage4hpu to: " + str( max2mapstate4hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4hpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2mapimage4hpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4hpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4hpu = consumerchannel2mapmanoeuvre4hpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2mapreceive4hpu-received-by-manoeuvre2mapimage4hpu: " + str(message2mapreceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4hpu == '<copy4xi>copy2mapimage4hpu</copy4xi>':
                message2mapsend4hpu = '<roger4xi>roger2mapimage4hpu</roger4xi>'
            else:
                message2mapsend4hpu = '<negative4xi><negative2mapimage4hpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4hpu.send(message2mapsend4hpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2mapsend4hpu-sent-by-manoeuvre2mapimage4hpu: " + str(message2mapsend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2mapreceive4hpu-failed-by-manoeuvre2mapimage4hpu: " + str(message2mapreceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4hpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2mapimage4hpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4hpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4hpu = consumerchannel2mapmanoeuvre4hpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2mapreceive4hpu-received-by-manoeuvre2mapimage4hpu: " + str(message2mapreceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4hpu == '<ready4xi>ready2mapimage4hpu</ready4xi>':
                message2mapsend4hpu = '<ready4xi>ready2mapimage4hpu</ready4xi>'
            else:
                message2mapsend4hpu = '<negative4xi><negative2ready4hpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4hpu.send(message2mapsend4hpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2mapsend4hpu-sent-by-manoeuvre2mapimage4hpu: " + str(message2mapsend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2mapreceive4hpu-failed-by-manoeuvre2mapimage4hpu: " + str(message2mapreceive4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4hpu == 3:    
        info2image4hpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4hpu-try-by-manoeuvre2mapimage4hpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4hpu = queue2mapmanoeuvre4hpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4hpu-got-by-manoeuvre2mapimage4hpu: " + str(info2image4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info2image4hpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2mapinfo4hpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2mapinfo4hpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2mapmanoeuvre4hpu-failed-by-manoeuvre2mapimage4hpu: " + str(queue2mapmanoeuvre4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2mapsend4hpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2mapinfo4hpu is True:
                message2mapsend4hpu = '<ack4xi>ack2imageinfo4hpu</ack4xi>'
            else:
                message2mapsend4hpu = '<nack4xi>nack2imageinfo4hpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2mapsend4hpu-set-by-manoeuvre2mapimage4hpu to: " + str(message2mapsend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #      
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4hpu-failed-by-manoeuvre2mapimage4hpu: " + str(message2mapsend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2mapmanoeuvre4hpu-calling-by-manoeuvre2mapimage4hpu: " + str(publisherchannel2mapmanoeuvre4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message2mapsend4hpu-passing-to-send-by-manoeuvre2mapimage4hpu: " + str(message2mapsend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4hpu-failed-by-manoeuvre2mapimage4hpu: " + str(message2mapsend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2mapmanoeuvre4hpu.send(message2mapsend4hpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2mapsend4hpu-sent-by-manoeuvre2mapimage4hpu: " + str(message2mapsend4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2mapsend4hpu-failed-by-manoeuvre2mapimage4hpu: " + str(message2mapsend4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4map['ACK2INFO4HPU']        = ack2mapinfo4hpu
    response2manouvre4map['CODE2ACK4HPU']        = code2ack
    response2manouvre4map['CODE2NACK4HPU']       = code2nack
    response2manouvre4map['MESSAGE2GET4HPU']     = message2mapget4hpu
    response2manouvre4map['MESSAGE2PUT4HPU']     = message2mapput4hpu
    response2manouvre4map['MESSAGE2RECEIVE4HPU'] = message2mapreceive4hpu
    response2manouvre4map['MESSAGE2SEND4HPU']    = message2mapsend4hpu
    response2manouvre4map['STATE2MANOEUVRE4HPU'] = state2map4hpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4map-returning-by-manoeuvre2mapimage4hpu: " + str(response2manouvre4map) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4HPU': '<roger4xi>roger2mapimage4hpu</roger4xi>', 
    #  'ACK2INFO4HPU': False, 
    #  'MESSAGE2GET4HPU': '<default4xi>default2map4hpu</default4xi>', 
    #  'MESSAGE2RECEIVE4HPU': '<copy4xi>copy2mapimage4hpu</copy4xi>', 
    #  'CODE2ACK4HPU': ' 0D-1C-', 
    #  'MESSAGE2PUT4HPU': '<default4xi>default2map4hpu</default4xi>', 
    #  'CODE2NACK4HPU': ' ', 
    #  'STATE2MANOEUVRE4HPU': 1}
     
    #2 {'MESSAGE2SEND4HPU': '<ready4xi>ready2mapimage4hpu</ready4xi>', 
    #  'ACK2INFO4HPU': False, 
    #  'MESSAGE2GET4HPU': '<default4xi>default2map4hpu</default4xi>', 
    #  'MESSAGE2RECEIVE4HPU': '<ready4xi>ready2mapimage4hpu</ready4xi>', 
    #  'CODE2ACK4HPU': ' 0D-2C-', 
    #  'MESSAGE2PUT4HPU': '<default4xi>default2map4hpu</default4xi>', 
    #  'CODE2NACK4HPU': ' ', 
    #  'STATE2MANOEUVRE4HPU': 2}; time: 06:49:48 10/04/11 Pacific Daylight Time}
     
    #3 {'MESSAGE2SEND4HPU': '<ack4xi>ack2imageinfo4hpu</ack4xi>', 
    #  'ACK2INFO4HPU': True, 
    #  'MESSAGE2GET4HPU': '<default4xi>default2map4hpu</default4xi>', 
    #  'MESSAGE2RECEIVE4HPU': '<ready4xi>ready2mapimage4hpu</ready4xi>', 
    #  'CODE2ACK4HPU': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4HPU': '<default4xi>default2map4hpu</default4xi>', 
    #  'CODE2NACK4HPU': ' ', 
    #  'STATE2MANOEUVRE4HPU': 3}; time: 06:49:48 10/04/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4HPU': '<default4xi>default2map4hpu</default4xi>', 
    #  'ACK2INFO4HPU': False, 
    #  'MESSAGE2GET4HPU': '<default4xi>default2map4hpu</default4xi>', 
    #  'MESSAGE2RECEIVE4HPU': '<default4xi>default2map4hpu</default4xi>', 
    #  'CODE2ACK4HPU': ' 0D-', 
    #  'MESSAGE2PUT4HPU': '<default4xi>default2map4hpu</default4xi>', 
    #  'CODE2NACK4HPU': ' ', 'STATE2MANOEUVRE4HPU': 0}; time: 06:49:48 10/04/11 Pacific Daylight Time}

    return response2manouvre4map           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "write2image4hpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4HPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF HPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4HPU ', time4xi())    
#   respon4write2image = {} 
#   task4write2image = None
#   try:                               
#       response4write2image = write2image4hpu(task4write2image)                                                                                                          
#       print('(Step 16B/700) OF HPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4HPU : ', response4write2image)
#       queue2write4hpu      = response4write2image['QUEUE']
#       channel2publish4hpu = response4write2image['PUBLISHERCHANNEL']
#       channel2consume4hpu = response4write2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4HPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4HPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF HPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4HPU ', time4xi())     
#   try:                               
#       message2writesend4hpu = "<copy4xi>copy2write4hpu</copy4xi>"
#       channel2publish4hpu = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4hpu.send(message2writesend4hpu)
#       print("(Step 17B/700) OF HPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4HPU: ", channel2publish4hpu, " MESSAGE2SEND4HPU: " , message2writesend4hpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF HPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4HPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4HPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi()) 
#   try:        
#       response2manoeuvre4hpu = manoeuvre2readimage4hpu(queue2write4hpu, channel2consume4hpu, channel2publish4hpu)
#       print("(Step 18D/700) OF HPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", manoeuvre2readimage4hpu)
#   except:
#       print("(Step 18Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4HPU ', time4xi())     
#   try:
#       channel2consume4hpu = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4hpu = channel2consume4hpu.recv()
#       print("(Step 19B/700) OF HPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4HPU: ", channel2consume4hpu, " CONTROL & COMMAND: ", message2writereceive4hpu)                                                                                                  
#       # <roger4xi>roger2write4hpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4HPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4HPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF HPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4HPU ', time4xi())     
#   try:                               
#       message2writesend4hpu = "<write4xi>write2write4hpu</write4xi>"
#       channel2publish4hpu = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4hpu.send(message2writesend4hpu)
#       print("(Step 20B/700) OF HPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4HPU: ", channel2publish4hpu, " MESSAGE2SEND4HPU: " , message2writesend4hpu)                                                                                                
#       # <write4xi>write2write4hpu</write4xi> 
#   except:
#        print('(Step 20Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4HPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4HPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi()) 
#   try:        
#       response2manoeuvre4hpu = manoeuvre2readimage4hpu(queue2write4hpu, channel2consume4hpu, channel2publish4hpu)
#       print("(Step 21B/700) OF HPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", manoeuvre2readimage4hpu)
#   except:
#       print("(Step 21Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4HPU ', time4xi())     
#   try:
#       channel2consume4hpu = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4hpu = channel2consume4hpu.recv()
#       print("(Step 22B/700) OF HPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4HPU: ", channel2consume4hpu, " CONTROL & COMMAND: ", message2writereceive4hpu)                                                                                                  
#       # <write4xi>write2write4hpu</write4xi> 
#   except:
#       print('(Step 22Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4HPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2write4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4HPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi()) 
#   try:        
#       response2manoeuvre4hpu = manoeuvre2readimage4hpu(queue2write4hpu, channel2consume4hpu, channel2publish4hpu)
#       print("(Step 25B/700) OF HPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", manoeuvre2readimage4hpu)
#   except:
#       print("(Step 25Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2writereceive4hpu = channel2consume4hpu.recv()
#       print("(Step 26D/700) OF HPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4HPU: ", channel2consume4hpu, " CONTROL & COMMAND: ", message2writereceive4hpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4hpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4HPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF HPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi()) 
#   try:        
#       response2manoeuvre4hpu = manoeuvre2readimage4hpu(queue2write4hpu, channel2consume4hpu, channel2publish4hpu)
#       print("(Step 27B/700) OF HPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4HPU ", response2manoeuvre4hpu)
#   except:
#       print("(Step 27Z/700) OF HPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4HPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def write2image4hpu(task4imagewrite2hpu=None):

    '''
    writes 2D images for HPU
    '''
    global hpu2image4xi                                                     # 2D-IMAGES FOR HPU  
    global control2writeimage4hpu                                              # COMMAND AND CONTROL FOR HPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS        
    # -----------------------------------------------------------------------------------------          
    class2print = "write2image4hpu"
    method2print = "write2image4hpu"

    message2print = "{(Step 1A/33) started" + "; write2image4hpu-started: " + str(write2image4hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function write2image4hpu at 0x00000000038E8EC8>
    
    message2print = "{(Step 1B/33) passed" + "; task4imagewrite2hpu-to-write2image4hpu: " + str(task4imagewrite2hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE HPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if hpu2image4xi is None:
            hpu2image4xi = create2image4hpu()                                                                              # INITIALIZE IMAGE FOR HPU 
            message2print = "{(Step 1C/70) created" + "; hpu2image4xi-created-by-write2image4hpu to: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4hpu-skipped-by-write2image4hpu: " + str(write2image4hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4hpu-skipped-by-write2image4hpu: <function write2image4hpu at 0x00000000038E8EC8>                                 
    except:
        try:
            hpu2image4xi = create2image4hpu()
            message2print = "{(Step 1E/70) set" + "; hpu2image4xi-set-by-write2image4hpu to: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            hpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; hpu2image4xi-yielded-by-write2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagewrite2hpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagewrite2hpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagewrite2hpu-created-by-write2image4hpu to: " + str(task4imagewrite2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4hpu = trace2image4hpu                                                                 # CALLBACK FUNCTION FOR HPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4hpu-created-by-write2image4hpu to: " + str(function2callback4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4hpu at 0x00000000038E2A48>
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4hpu"                                                            # MODULE OF HPUIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4hpu"                                                            # FUNCTION OF HPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4hpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR HPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-write2image4hpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; write2image4hpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'write2image4hpu', 
            #  'METHOD': 'write2image4hpu'}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2hpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO HPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2hpu['FUNCTION2CALLBACK'] = function2callback4hpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagewrite2hpu-created-by-write2image4hpu to: " + str(task4imagewrite2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4hpu at 0x00000000038E2A48>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; write2image4hpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}',
            #  'CLASS': 'write2image4hpu',
            #  'METHOD': 'write2image4hpu'}}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-write2image4hpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-write2image4hpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 02:32:29 10/05/11 Pacific Daylight Time}

                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-write2image4hpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagewrite2hpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2hpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2hpu-trying-by-write2image4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2hpu = get2context4hpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2hpu-got-by-write2image4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4hpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4hpu', ...
   
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2hpu-failed-by-write2image4hpu: " + str(context2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagewrite2hpu ['CONTEXT'] = context2hpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2hpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2hpu-trying-by-write2image4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2hpu = get2hpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2hpu-got-by-write2image4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ...
               
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2hpu-failed-by-write2image4hpu: " + str(device2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagewrite2hpu ['DEVICE'] = device2hpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2hpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2hpu-trying-by-write2image4hpu: " + str(imageinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2hpu = get2imageinfo4hpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2hpu-got-by-write2image4hpu: " + str(imageinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004494E10>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4hpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4hpu', 
                #  'METHOD': 'create2context4hpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4hpu at 0x000000000387E348>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
                #  'COUNT2REFERENCE': 1, 
                #  'FUNCTION2CALLBACK': <function trace2image4hpu at 0x00000000038E2A48>, 
                #  'COMMANDQUEUE': ...

                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2hpu-failed-by-write2image4hpu: " + str(imageinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagewrite2hpu ['IMAGE'] = imageinfo2hpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY HPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2hpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2hpu-trying-by-write2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2hpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2hpu-got-by-write2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003D4FA90>
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2hpu-failed-by-write2image4hpu: " + str(queue2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagewrite2hpu ['QUEUE'] = queue2hpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY HPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2hpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2hpu-trying-by-write2image4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2hpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2hpu-got-by-write2image4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 568>, <read-write PipeConnection, handle 572>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2hpu-failed-by-write2image4hpu: " + str(pipe2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagewrite2hpu ['PIPE'] = pipe2hpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2hpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2hpu-trying-by-write2image4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2hpu = get2commandqueueinfo4hpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2hpu-got-by-write2image4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4hpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4hpu', 
                #  'METHOD': 'create2commandqueue4hpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4hpu at 0x000000000388DA48> ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2hpu-failed-by-write2image4hpu: " + str(commandqueueinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagewrite2hpu ['COMMANDQUEUE'] = commandqueueinfo2hpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY HPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2hpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2hpu-trying-by-write2image4hpu: " + str(bufferinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2hpu = get2bufferinfo4hpu()                                                               # SYSTEM CREATES BUFFER FOR HPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2hpu-got-by-write2image4hpu: " + str(bufferinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4hpu-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2buffer4hpu', 
                #  'METHOD': 'create2buffer4hpu'}, ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2hpu-failed-by-write2image4hpu: " + str(bufferinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagewrite2hpu ['BUFFER'] = bufferinfo2hpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2hpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2hpu-trying-by-write2image4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2hpu = get2eventinfo4hpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2hpu-got-by-write2image4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  {'CODE2ERROR': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4hpu-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2event4hpu', 
                #  'METHOD': 'create2event4hpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2event4hpu at 0x00000000038A3F48>, ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2hpu-failed-by-write2image4hpu: " + str(eventinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagewrite2hpu ['EVENT'] = eventinfo2hpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2hpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2hpu-trying-by-write2image4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2hpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2hpu-set-by-write2image4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2hpu-failed-by-write2image4hpu: " + str(eventwaitlist2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagewrite2hpu ['EVENTWAITLIST'] = eventwaitlist2hpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2hpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2hpu-trying-by-write2image4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2hpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2hpu = eventwaitlistquantity2hpu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2hpu-set-by-write2image4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2hpu-failed-by-write2image4hpu: " + str(eventwaitlistquantity2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagewrite2hpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2hpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2hpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2hpu-trying-by-write2image4hpu: " + str(blocking2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2hpu = 'CL_FALSE'                                                                           # the write command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2hpu-set-by-write2image4hpu to: " + str(blocking2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE 
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2hpu-failed-by-write2image4hpu: " + str(blocking2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagewrite2hpu ['BLOCKING'] = blocking2hpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2hpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2hpu-trying-by-write2image4hpu: " + str(origin2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2hpu['X'] = 0 
                origin2hpu['Y'] = 0  
                origin2hpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2hpu-set-by-write2image4hpu to: " + str(origin2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2hpu-failed-by-write2image4hpu: " + str(origin2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagewrite2hpu ['ORIGIN'] = origin2hpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2hpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2hpu-trying-by-write2image4hpu: " + str(region2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2hpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2hpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2hpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2hpu-set-by-write2image4hpu to: " + str(region2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2hpu-failed-by-write2image4hpu: " + str(region2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagewrite2hpu ['REGION'] = region2hpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.

            height2hpu = 800
            width2hpu = 800
            rowpitch2hpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2hpu-trying-by-write2image4hpu: " + str(rowpitch2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #  
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2hpu = int(width2hpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2hpu-set-by-write2image4hpu to: " + str(rowpitch2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                            
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2hpu-failed-by-write2image4hpu: " + str(rowpitch2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagewrite2hpu ['ROWPITCH'] = rowpitch2hpu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2hpu = None
            size2hpu = int(width2hpu * height2hpu)                                                                  # CALCULATED MEMORY
            if size2hpu == 0:
                size2hpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2hpu-trying-by-write2image4hpu: " + str(pointer2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2hpu = Arena4Xi(size2hpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2hpu-got-by-write2image4hpu: " + str(arena2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041348D0>
                
                size2arena = arena2hpu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-write2buffer4hpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2hpu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-write2buffer4hpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x000000000421A450> 
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2hpu:
                    size2hpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2hpu-set-by-write2buffer4hpu to: " + str(size2hpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2hpu = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2hpu-set-by-write2buffer4hpu to: " + str(pointer2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x000000000421A450>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2hpu-failed-by-write2image4hpu: " + str(pointer2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagewrite2hpu ['POINTER'] = pointer2hpu                            
            task4imagewrite2hpu ['SIZE']    = size2hpu                                                               # AUTOMATIC BY HPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2hpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2hpu-trying-by-write2image4hpu: " + str(slicepitch2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2hpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2hpu-set-by-write2image4hpu to: " + str(slicepitch2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2hpu-failed-by-write2image4hpu: " + str(slicepitch2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagewrite2hpu ['SLICEPITCH'] = slicepitch2hpu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2hpu.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-write2image4hpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4hpu = trace2image4hpu                                                             # CALLBACK FUNCTION FOR HPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4hpu-created-by-write2image4hpu to: " + str(function2callback4hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4hpu"                                                        # MODULE OF HPUIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4hpu"                                                        # FUNCTION OF HPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4hpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR HPUIZED OPENCL TO RETURN    
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-write2image4hpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2hpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO HPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2hpu['FUNCTION2CALLBACK'] = function2callback4hpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagewrite2hpu-created-by-write2image4hpu to: " + str(task4imagewrite2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-write2image4hpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-write2image4hpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                     
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-write2image4hpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                     
            task4imagewrite2hpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
              
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2hpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2hpu-trying-by-write2image4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2hpu = get2context4hpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2hpu-got-by-write2image4hpu: " + str(context2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2hpu-failed-by-write2image4hpu: " + str(context2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagewrite2hpu ['CONTEXT'] = context2hpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2hpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2hpu-trying-by-write2image4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2hpu = get2hpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2hpu-got-by-write2image4hpu: " + str(device2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2hpu-failed-by-write2image4hpu: " + str(device2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagewrite2hpu ['DEVICE'] = device2hpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2hpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2hpu-trying-by-write2image4hpu: " + str(imageinfo2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2hpu = get2imageinfo4hpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2hpu-got-by-write2image4hpu: " + str(imageinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2hpu-failed-by-write2image4hpu: " + str(imageinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagewrite2hpu ['IMAGE'] = imageinfo2hpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY HPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2hpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2hpu-trying-by-write2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2hpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2hpu-got-by-write2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2hpu-failed-by-write2image4hpu: " + str(queue2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagewrite2hpu ['QUEUE'] = queue2hpu

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY HPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2hpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2hpu-trying-by-write2image4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2hpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2hpu-got-by-write2image4hpu: " + str(pipe2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2hpu-failed-by-write2image4hpu: " + str(pipe2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagewrite2hpu ['PIPE'] = pipe2hpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2hpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2hpu-trying-by-write2image4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2hpu = get2commandqueueinfo4hpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2hpu-got-by-write2image4hpu: " + str(commandqueueinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2hpu-failed-by-write2image4hpu: " + str(commandqueueinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagewrite2hpu ['COMMANDQUEUE'] = commandqueueinfo2hpu
            
            
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY HPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2hpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2hpu-trying-by-write2image4hpu: " + str(bufferinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2hpu = get2bufferinfo4hpu()                                                           # SYSTEM CREATES BUFFER FOR HPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2hpu-got-by-write2image4hpu: " + str(bufferinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2hpu-failed-by-write2image4hpu: " + str(bufferinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagewrite2hpu ['BUFFER'] = bufferinfo2hpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2hpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2hpu-trying-by-write2image4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2hpu = get2eventinfo4hpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2hpu-got-by-write2image4hpu: " + str(eventinfo2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2hpu-failed-by-write2image4hpu: " + str(eventinfo2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagewrite2hpu ['EVENT'] = eventinfo2hpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2hpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2hpu-trying-by-write2image4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2hpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2hpu-set-by-write2image4hpu: " + str(eventwaitlist2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2hpu-failed-by-write2image4hpu: " + str(eventwaitlist2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagewrite2hpu ['EVENTWAITLIST'] = eventwaitlist2hpu                     

                 
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2hpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2hpu-trying-by-write2image4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2hpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2hpu = eventwaitlistquantity2hpu + 1                                                                     
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2hpu-set-by-write2image4hpu: " + str(eventwaitlistquantity2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2hpu-failed-by-write2image4hpu: " + str(eventwaitlistquantity2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagewrite2hpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2hpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2hpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2hpu-trying-by-write2image4hpu: " + str(blocking2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2hpu = 'CL_FALSE'                                                                       # the write command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2hpu-set-by-write2image4hpu to: " + str(blocking2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2hpu-failed-by-write2image4hpu: " + str(blocking2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagewrite2hpu ['BLOCKING'] = blocking2hpu             
                 
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2hpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2hpu-trying-by-write2image4hpu: " + str(origin2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2hpu['X'] = 0 
                origin2hpu['Y'] = 0  
                origin2hpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2hpu-set-by-write2image4hpu to: " + str(origin2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2hpu-failed-by-write2image4hpu: " + str(origin2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagewrite2hpu ['ORIGIN'] = origin2hpu
                
                 
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2hpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2hpu-trying-by-write2image4hpu: " + str(region2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2hpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2hpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2hpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2hpu-set-by-write2image4hpu to: " + str(region2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2hpu-failed-by-write2image4hpu: " + str(region2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagewrite2hpu ['REGION'] = region2hpu
            
                
                 
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2hpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2hpu-trying-by-write2image4hpu: " + str(rowpitch2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2hpu = int(width2hpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2hpu-set-by-write2image4hpu to: " + str(rowpitch2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2hpu-failed-by-write2image4hpu: " + str(rowpitch2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagewrite2hpu ['ROWPITCH'] = rowpitch2hpu     
                 
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2hpu = None
            size2hpu = int(width2hpu * height2hpu)                                                                  # CALCULATED MEMORY
            if size2hpu == 0:
                size2hpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2hpu-trying-by-write2image4hpu: " + str(pointer2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2hpu = Arena4Xi(size2hpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2hpu-got-by-write2image4hpu: " + str(arena2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2hpu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-write2buffer4hpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2hpu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-write2buffer4hpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2hpu:
                    size2hpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2hpu-set-by-write2buffer4hpu to: " + str(size2hpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2hpu = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2hpu-set-by-write2buffer4hpu to: " + str(pointer2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2hpu-failed-by-write2image4hpu: " + str(pointer2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imagewrite2hpu ['POINTER'] = pointer2hpu                            
            task4imagewrite2hpu ['SIZE']    = size2hpu                                                               # AUTOMATIC BY HPUXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2hpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2hpu-trying-by-write2image4hpu: " + str(slicepitch2hpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2hpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2hpu-set-by-write2image4hpu to: " + str(slicepitch2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2hpu-failed-by-write2image4hpu: " + str(slicepitch2hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagewrite2hpu ['SLICEPITCH'] = slicepitch2hpu                  

        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2hpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-write2image4hpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}
             
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagewrite2hpu-yielded-by-write2image4hpu: " + str(task4imagewrite2hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2writeimage4hpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2hpu = False
    channels2hpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2hpu-trying-by-write2image4hpu: " + str(channels2hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2hpu, publisherchannel2hpu = task4imagewrite2hpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2hpu-got-by-write2image4hpu: " + str(consumerchannel2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 568>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2hpu-got-by-write2image4hpu: " + str(publisherchannel2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 572>                     
        ack4channels2hpu = True
        
        control2writeimage4hpu['CONSUMERCHANNEL']  = consumerchannel2hpu
        
        
        control2writeimage4hpu['PUBLISHERCHANNEL'] = publisherchannel2hpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2hpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2hpu = False
    queue2hpu = None
    message2print = "{(Step 41A/70) trying" + "queue2hpu-trying-by-write2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2hpu = task4imagewrite2hpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2hpu-got-by-write2image4hpu: " + str(queue2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003C83A90> 
              
        ack4queue2hpu = True
        control2writeimage4hpu['QUEUE']  = queue2hpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2hpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4hpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4hpu-trying-by-write2image4hpu: " + str(dummy2image4hpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4hpu = DummyProcess4Xi(target=manoeuvre2readimage4hpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2hpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2hpu, publisherchannel2hpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4hpu-created-by-write2image4hpu: " + str(dummy2image4hpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-write2image4hpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-write2image4hpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2writeimage4hpu-returning-by-write2image4hpu: " + str(control2writeimage4hpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003C83A90>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 572>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 568>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

    return control2writeimage4hpu
                    
#</function4xi> 


 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "release2image4hpu" usage = "
#
#   task4release2image  = {} 
#   task4release2image  ['COUNT2REFERENCE'] = 1 
#   response4release2image = {}
#    
#   print('(Step 12A/33) OF HPU2IMAGE4XI5TEST - TRYING TO RELEASE IMAGE OF HPU DEVICE ', time4xi())   
#   try:                               
#       response4release2image = release2image4hpu(task4release2image)                                                                                             
#       print('(Step 12B/33) OF HPU2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4release2image)                           
#   except:
#       print('(Step 12Z1/33) OF HPU2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF HPU DEVICE ', time4xi())  
#   
#                 
#">

# -------------------------------------------------------------------------------------------------   
def release2image4hpu(task4release2hpu=None):

    '''
    releases hpu2image4xi
    '''
    global hpu2image4xi  
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "release2image4hpu"
    method2print = "release2image4hpu" 

    message2print = "{(Step 1A/33) started" + "; release2image4hpu-started: " + str(release2image4hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function release2image4hpu at 0x0000000003A9E8C8>
    
    message2print = "{(Step 1B/33) passed" + "; task4release2hpu-to-release2image4hpu: " + str(task4release2hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1} 

             
    code2ack  = ' '
    code2nack = ' '

    try:
        if hpu2image4xi is None:
            hpu2image4xi = create2image4hpu()                                                                       # INITIALIZE IMAGE                 
    except:
        try:
            hpu2image4xi = create2image4hpu()
        except:
            hpu2image4xi = {}
            message2print = "{(Step 2Z0/70) yielded" + "; hpu2image4xi-yielded-by-release2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '2Z0-' 
                
    message2print = "{(Step 1A/70) try" + "; release-try-by-release2image4hpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    if task4release2hpu is None:
        task4release2hpu = {}
        task4release2hpu['COUNT2REFERENCE'] = 1
           
    try:
        # -----------------------------------------------------------------------------------------
        # RELEASE
        # -----------------------------------------------------------------------------------------               
        hpu2image4xi['COUNT2REFERENCE'] = hpu2image4xi['COUNT2REFERENCE'] - task4release2hpu['COUNT2REFERENCE']
        
        message2print = "{(Step 1A/70) calculated" + "; count2reference-calculated-by-release2image4hpu: " + str(hpu2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
         
        if hpu2image4xi['COUNT2REFERENCE'] < 0:
            hpu2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 1B/70) warning" + "; count2reference-warning-by-release2image4hpu: " + " ** " + str(hpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            # { release2image4hpu : release2image4hpu ; (Step 8Z1/70) warning; count2reference-warning-by-release2image4hpu:  ** 0 ** ; time: 23:41:37 09/06/11 Pacific Daylight Time}    
            code2nack  = code2nack + '1B-'
        else:
            code2ack  = code2ack + '1B-'
    except:
        try:
            hpu2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 8Z1/70) warning" + "; count2reference-warning-by-release2image4hpu: " + " ** " + str(hpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            code2nack  = code2nack + '8Z1-'
            
        except:
            message2print = "{(Step 8Z2/70) yielded" + "; hpu2image4xi-yielded-by-release2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '8Z2-'
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    hpu2image4xi ['ACK2RETURN']  = code2ack
    hpu2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; hpu2image4xi-returning-by-release2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0,
    #  'ACK2RETURN': ' ',
    #  'ROWPITCH': 800,
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4hpu-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'create2image4hpu',
    #  'METHOD': 'create2image4hpu'},
    #  'WIDTH': 800,
    #  'BUFFER': {'CODE2ERROR': 0, 
    #  'COUNT2REFERENCE': 0,
    
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4hpu-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}', 'CLASS': 'create2buffer4hpu', 'METHOD': 'create2buffer4hpu'},
    #  'FUNCTION2CALLBACK': <function trace2buffer4hpu at 0x0000000003A97348>,
    #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CDE298>,
    #  'FLAGS': 0, 'CONTEXT': {'COUNT2REFERENCE': 0, ...
    
    
    return hpu2image4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "retain2image4hpu" usage = "
#   task4retain2image  = {}
#   task4retain2image  ['DIMENSION'] = 2 
#   task4retain2image  ['COUNT2REFERENCE'] = 1 
#   response4retain2Dimage = {}
#   print('(Step 13A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF HPU DEVICE ', time4xi())  
#   try:                               
#       response4retain2Dimage = retain2image4hpu(task4retain2image)                                                                                                 
#       print('(Step 13B/700) OF HPU2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', response4retain2Dimage)                          
#       # 
#   except:
#       print('(Step 13Z1/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF HPU DEVICE ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def retain2image4hpu(task4retain2hpu=None):

    '''
    retains either hpu2image4xi
    '''
    global hpu2image4xi 
    
     
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "retain2image4hpu"
    method2print = "retain2image4hpu"

    message2print = "{(Step 1A/33) started" + "; retain2image4hpu-started: " + str(retain2image4hpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function retain2image4hpu at 0x0000000003A9E948>   
    
    message2print = "{(Step 1B/33) passed" + "; task4retain2hpu-to-retain2image4hpu: " + str(task4retain2hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 2}
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 3}
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # (Step 2/70) - PROVIDE TASKS, IF NECESSARY AND CHECK GIVEN TASKS 
    # -----------------------------------------------------------------------------------------    
    if task4retain2hpu is None:
        task4retain2hpu = {}
        task4retain2hpu['DIMENSION'] = 2
        task4retain2hpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 2A/33) set" + "; task4retain2hpu-set-by-retain2image4hpu to: " + str(task4retain2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    else:
        pass
        
    if 'DIMENSION' in task4retain2hpu.keys():
        if task4retain2hpu['DIMENSION'] == 2:
            pass
        else:
            task4retain2hpu['DIMENSION'] = 2 
            message2print = "{(Step 2B/33) set" + "; dimension-set-by-retain2image4hpu to: " + str(task4retain2hpu['DIMENSION'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                       
    else:
        task4retain2hpu['DIMENSION'] = 2
    
    # -----------------------------------------------------------------------------------------
    # (Step 3/70) - CHECK COUNT2REFERENCE
    # -----------------------------------------------------------------------------------------         
    if 'COUNT2REFERENCE' in task4retain2hpu.keys():
        pass
    else:
        task4retain2hpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3A/33) set" + "; count2reference-set-by-retain2image4hpu to: " + str(task4retain2hpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                                   
    if task4retain2hpu['COUNT2REFERENCE'] < 1:
        task4retain2hpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3B/33) set" + "; count2reference-set-by-retain2image4hpu to: " + str(task4retain2hpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
        
    # -----------------------------------------------------------------------------------------
    # (Step 4/70) - INITIALIZE 2D IMAGE IF NECESSARY
    # -----------------------------------------------------------------------------------------
    try:
        if hpu2image4xi is None:
            hpu2image4xi = create2image4hpu()
            message2print = "{(Step 4A/70) created" + "; hpu2image4xi-created-by-retain2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                                                                          
    except:
        try:
            hpu2image4xi = create2image4hpu()
            message2print = "{(Step 4B/70) created" + "; hpu2image4xi-created-by-retain2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
        except:
            hpu2image4xi = {}
            message2print = "{(Step 4Z/70) yielded" + "; hpu2image4xi-yielded-by-retain2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '4Z-' 


    # -----------------------------------------------------------------------------------------
    # (Step 5/70) - RETAIN
    # -----------------------------------------------------------------------------------------             
    try:
        hpu2image4xi['COUNT2REFERENCE'] = hpu2image4xi['COUNT2REFERENCE'] + task4retain2hpu['COUNT2REFERENCE']
        message2print = "{(Step 6A/70) calculated" + "; count2reference-calculated-by-retain2image4hpu: " + str(hpu2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #              
        if hpu2image4xi['COUNT2REFERENCE'] <= 0:
            hpu2image4xi['COUNT2REFERENCE'] = 1
            message2print = "{(Step 6Z1/70) warning" + "; count2reference-warning-by-retain2image4hpu: " + " ** " + str(hpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            code2nack  = code2nack + '6Z1-'
        else:
            code2ack  = code2ack + '6C-'

    except:
        try:
            hpu2image4xi['COUNT2REFERENCE'] = 1
        
            message2print = "{(Step 7Z1/70) warning" + "; count2reference-warning-by-retain2image4hpu: " + " ** " + str(hpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            code2nack  = code2nack + '7Z1-'
        
        except:
            message2print = "{(Step 7Z2/70) yielded" + "; hpu2image4xi-yielded-by-retain2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '7Z2-'
        
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70) - RETURN           
    # -----------------------------------------------------------------------------------------
    
    hpu2image4xi ['ACK2RETURN']  = code2ack
    hpu2image4xi ['NACK2RETURN'] = code2nack     
    message2print = "{(Step 70A/70) returning" + "; hpu2image4xi-returning-by-retain2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 6C-', 'ROWPITCH': 800, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4hpu-failed ; time: 23:25:02 10/05/11 Pacific Daylight Time}', 'CLASS': 'create2image4hpu', 'METHOD': 'create2image4hpu'}, 'WIDTH': 800, 'BUFFER': {'CODE2ERROR': 0, 
    #                   'COUNT2REFERENCE': 0, ...
    # 'COUNT2REFERENCE': 1, 
    # 'FLAGS': 0, 'CONTEXT': ...
    
    return hpu2image4xi

#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2imageinfo4hpu" usage = "
#   #1
#   print('(Step 3A/33) OF HPU2IMAGE4XI5TEST - TRYING TO GET 2D IMAGE INFO FOR HPU', time4xi())  
#   try:
#       imageinfo2hpu = get2imageinfo4hpu()                                                                       
#       print('(Step 3B/33) OF HPU2IMAGE4XI5TEST - GOT 2D IMAGE INFO FOR HPU: ', imageinfo2hpu) 
#   except:
#       print('(Step 3Z/33) OF HPU2IMAGE4XI5TEST *** FAILED TO GET 2D IMAGE INFO FOR HPU', time4xi())
#   #2
#   task4getinfo2image = {}
#   task4getinfo2image ['DIMENSION'] = 2
#   task4getinfo2image ['WIDTH'] = 0
#   task4getinfo2image ['HEIGHT'] = 0
#   response4getinfo2image = {}
#   print('(Step 12C/33) OF HPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 3D IMAGE INFO OF HPU DEVICE ', time4xi())    
# 
#   try:                               
#       response4getinfo2image = get2imageinfo4hpu (task4getinfo2image)                                                                                             
#       print('(Step 12D/33) OF HPU2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4getinfo2image)                           
#       #
#   except:
#       print('(Step 12Z2/33) OF HPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 3D IMAGE INFO OF HPU DEVICE ', time4xi()) 
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2imageinfo4hpu(task4getinfo2image=None):

    '''
    gets hpu2image4xi
    '''
    
    global hpu2image4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2imageinfo4hpu"
    method2print = "get2imageinfo4hpu"

    message2print = "{(Step 0A/33) started" + "; get2imageinfo4hpu-started: " + str(get2imageinfo4hpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # <function get2imageinfo4hpu at 0x0000000003A9E9C8>                                                              
    
    message2print = "{(Step 0A/33) passed" + "; task4getinfo2image-passed-to-get2imageinfo4hpu: " + str(task4getinfo2image) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # {'WIDTH': 0, 'DIMENSION': 2, 'HEIGHT': 0}
    # {'WIDTH': 0, 'DEPTH': 0, 'DIMENSION': 3, 'HEIGHT': 0}
    # {'DOMAIN': 'ALL', 'DIMENSION': 2}
    
    image2hpu = {}
    try: 
        if hpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE 2D IMAGE FOR HPU           
            # -------------------------------------------------------------------------------------           
            image2hpu = create2image4hpu()
            message2print = "{(Step 1B2/33) initialized" + "; image2hpu-initialized-by-get2imageinfo4hpu to: " + str(image2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        else:
            message2print = "{(Step 1B2/33) using" + "; hpu2image4xi-using-by-get2imageinfo4hpu: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-', 
            #  'ROWPITCH': 800, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4hpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2image4hpu', 
            #  'METHOD': 'create2image4hpu'}, 
            #  'WIDTH': 800, 
            #  'BUFFER': ...
            
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF THE IMAGE INFO OF HPU          
        # -------------------------------------------------------------------------------------     
        dimension2xi = 0
        
        if task4getinfo2image is None:
            task4getinfo2image = {}
            task4getinfo2image ['DIMENSION'] = 2 
            task4getinfo2image ['DOMAIN'] = 'ALL'
            image2hpu = hpu2image4xi
            message2print = "{(Step 1C/33) using" + "; image2hpu-got-by-get2imageinfo4hpu: " + str(image2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0, 
            #  'DOMAIN': 'ALL', 
            #  'CONTEXT': ...
            
        else:
            pass    

        keys2task4imageinfo = task4getinfo2image.keys()
        if 'DIMENSION' in keys2task4imageinfo:
            dimension2xi = task4getinfo2image['DIMENSION']
            if dimension2xi == 2:
                pass
            else:
                dimension2xi = 2    
        else:
            pass
            
        if 'DOMAIN' in keys2task4imageinfo:
            domain2xi = task4getinfo2image['DOMAIN']
            if domain2xi == 'ALL':
                pass
            else:
                domain2xi = 'ALL'

            keys2domain = hpu2image4xi.keys()
            for key2domain in keys2domain:
                task4getinfo2image[key2domain] = hpu2image4xi[key2domain]
                    
            message2print = "{(Step 1C/33) using" + "; task4getinfo2image-using-by-get2imageinfo4hpu: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                          
        else:
            pass            
            
                    

        keys2task = task4getinfo2image.keys()
        for key2task in keys2task:
            try:
                message2print = "{(Step 2D2/33) using" + "; key2task-using-by-get2image4hpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # WIDTH; DIMENSION; HEIGHT  
                image2hpu [key2task] = hpu2image4xi[key2task]
                
                message2print = "{(Step 2D2/33) set" + "; info-of-image2hpu-set-by-get2imageinfo4hpu to: " + str(image2hpu [key2task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 800; 800  
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key2task-yielded-by-get2image4hpu: " + str(key2task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # DIMENSION;
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo2image-using-by-get2image4hpu: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; hpu2image4xi-using-by-get2image4hpu: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                           
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2hpu = create2image4hpu()
            message2print = "{(Step 1F/33) initialized" + "; image2hpu-initialized-by-get2imageinfo4hpu to: " + str(image2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
            if task4getinfo2image is None:
                image2hpu = hpu2image4xi
                message2print = "{(Step 1G/33) got" + "; image2hpu-got-by-get2imageinfo4hpu: " + str(image2hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            else:
                keys2task = task4getinfo2image.keys()
                for key2task in keys2task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key2task-using-by-get2image4hpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        image2hpu [key2task] = hpu2image4xi[key2task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-image2hpu-set-by-get2imageinfo4hpu to: " + str(image2hpu [key2task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key2task-yielded-by-get2image4hpu: " + str(key2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                            
        except:
            image2hpu = {}
            message2print = "{(Step 1Z3/70) yielded" + "; image2hpu-yielded-by-get2image4hpu: " + str(image2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; image2hpu-returning-by-get2imageinfo4hpu: " + str(image2hpu) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)
    # 2D: {'WIDTH': 800, 'HEIGHT': 800}
    # 3D: {'WIDTH': 800, 'DEPTH': 800, 'HEIGHT': 800}
    # 2D ALL: {'CODE2ERROR': 0, 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4hpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create2image4hpu', 
    #  'METHOD': 'create2image4hpu'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ...
    
    # 3D ALL: {'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create3image4hpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create3image4hpu', 
    #  'METHOD': 'create3image4hpu'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ....
    #  'HEIGHT': 800, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004504400>, 
    #  'WIDTH': 800, 
    #  'DEPTH': 800, 
    #  'FLAGS': 0, 
    #  'CONTEXT': ...
    
    return image2hpu
            
#</function4xi> 
 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2supportedimageformats4hpu" usage = "
#
#   print('(Step 3A/33) OF HPU2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR HPU', time4xi())  
#   try:
#       imageformats = get2supportedimageformats4hpu()                                                                       
#       print('(Step 3B/33) OF HPU2IMAGE4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR HPU: ', imageformats) 
#   except:
#       print('(Step 3Z/33) OF HPU2IMAGE4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR HPU', time4xi())
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2supportedimageformats4hpu(task4supportedimageformats2hpu=None):  

    '''
    gets supported image formats for hpu
    '''
    
    global hpu2image4xi

    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2supportedimageformats4hpu"
    method2print = "get2supportedimageformats4hpu"

    message2print = "{(Step 0A/33) started" + "; get2supportedimageformats4hpu-started: " + str(get2supportedimageformats4hpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    # <function get2supportedimageformats4hpu at 0x0000000003BBE8C8>                                                          
    
    message2print = "{(Step 0B/33) passed" + "; task4supportedimageformats2hpu-passed-to-get2supportedimageformats4hpu: " + str(task4supportedimageformats2hpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # None; {'DIMENSION': 3} 
    
    
    if task4supportedimageformats2hpu is None:
        task4supportedimageformats2hpu = {} 
        task4supportedimageformats2hpu['DIMENSION'] = 2                                                             # 2D image is the default
    
    dimension2xi = 2
    try:
        keys2supportedimageformats = task4supportedimageformats2hpu.keys()    
        if 'DIMENSION' in keys2supportedimageformats:
            dimension2xi = task4supportedimageformats2hpu['DIMENSION']
            if ((dimension2xi <=0) or (dimension2xi > 2)): 
                dimension2xi = 2
        else:
            dimension2xi = 2
    except:
        pass        
    
    message2print = "{(Step 1A/33) using" + "; dimension2xi-using-by-dimension2xi: " + str(dimension2xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # 2; 3
                        
    supportedimageformats2hpu = {} 
    try: 
        if hpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2hpu = create2image4hpu()                                                                          # create hpu2image4xi
            message2print = "{(Step 1B/33) initialized" + "; image2hpu-initialized-by-get2supportedimageformats4hpu to: " + str(image2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                
        else:
            pass

                        
        # -------------------------------------------------------------------------------------
        # SUPPORTED IMAGE FORMATS FOR HPU       
        # -------------------------------------------------------------------------------------     
        supportedimageformats2hpu = hpu2image4xi['FORMAT']
        message2print = "{(Step 1E/33) got" + "; supportedimageformats2hpu-got-by-get2supportedimageformats4hpu: " + str(supportedimageformats2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
            
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2hpu = create2image4hpu()
            message2print = "{(Step 1F1/33) initialized" + "; image2hpu-initialized-by-get2supportedimageformats4hpu to: " + str(image2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
     
            
            # -------------------------------------------------------------------------------------
            # SUPPORTED IMAGE FORMATS FOR HPU       
            # -------------------------------------------------------------------------------------     
            supportedimageformats2hpu = hpu2image4xi['FORMAT']
            message2print = "{(Step 1H/33) got" + "; supportedimageformats2hpu-got-by-get2supportedimageformats4hpu: " + str(supportedimageformats2hpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
            #  
                                              
        except:
            message2print = "{(Step 1Z/70) yielded" + "; supportedimageformats2hpu-yielded-by-get2image4hpu: " + str(supportedimageformats2hpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; supportedimageformats2hpu-returning-by-get2supportedimageformats4hpu: " + str(supportedimageformats2hpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # 2: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    # 3: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    
    return supportedimageformats2hpu
            
#</function4xi> 

 



# -------------------------------------------------------------------------------------------------
#<function4xi function = "wait2image4hpu" usage = "
#   print('=' *33)          
#   print('(Step 16A/700) OF HPU2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE FOR HPU ', time4xi())   
#   print('=' *33)      
#   try:
#       response4wait2hpu = wait2image4hpu()
#       keys2response = response4wait2hpu.keys()
#       print('(Step 16B/700) OF HPU2IMAGE4XI5TEST - WAITED 2D IMAGE FOR HPU ', time4xi()) 
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4wait2hpu[key2response])
#           # ACK2RETURN  :   6C-
#           # BUFFER  :  {'CODE2ERROR': 0 ...
#           # CALLBACK  :  <function trace2image4hpu at 0x0000000004E93B48>
#           # CODE2ERROR  :  0
#           # COMMANDQUEUE  :  {'CODE2ERROR': 0, ...
#           # CONTEXT  :  {'COUNT2REFERENCE': 0 ...
#           # COUNT2REFERENCE  :  1
#           # DATA2CALLBACK  :  {'MESSAGE': '{(Step 1Z1/33) failed; create2image4hpu-failed ; time: 02:00:43 10/06/11 Pacific Daylight Time}', 'CLASS': 'create2image4hpu', 'METHOD': 'create2image4hpu'}
#           # DEPTH  :  0
#           # DEVICE  :  {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK',...
#           # EMPTY2QUEUE  :  True
#           # EVENT  :  {'CODE2ERROR': 0, 'DATA2CALLBACK':...
#           # FLAGS  :  0
#           # FORMAT  :  {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
#           # FULL2QUEUE  :  False
#           # FUNCTION2CALLBACK  :  <function trace2image4hpu at 0x0000000004E93B48>
#           # HEIGHT  :  800
#           # NACK2RETURN  :   
#           # PIPE  :  (<read-write PipeConnection, handle 556>, <read-write PipeConnection, handle 560>)
#           # PLATFORM  :  {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
#           # POINTER  :  <mmap.mmap object at 0x0000000004351500>
#           # QUEUE  :  <queue4xi.Queue4Xi object at 0x00000000044E95C0>
#           # ROWPITCH  :  800
#           # SIZE2QUEUE  :  2147483646 
#   except:
#       print('(Step 16Z1/700) OF HPU2IMAGE4XI5TEST *** FAILED TO WAIT 2D IMAGE FOR HPU: ', wait2hpu)            
#">

# -------------------------------------------------------------------------------------------------   
def wait2image4hpu(task4wait2image=None):

    '''
    waits hpu2image4hpu
    '''
    
    global hpu2image4xi
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "wait2image4hpu"
    method2print = "wait2image4hpu"

    message2print = "{(Step 1A1/33) started" + "; wait2image4hpu-started: " + str(wait2image4hpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    #  
    
    message2print = "{(Step 1A2/33) passed" + "; task4wait2image-passed-by-wait2image4hpu to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    
    try: 
        if hpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE - KEEP SYSTEM RUNNING           
            # -------------------------------------------------------------------------------------           
            hpu2image4xi = create2image4hpu()
            
            message2print = "{(Step 1B/33) created" + "; hpu2image4xi-created-by-wait2image4hpu: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                                                
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4hpu = None
            try:
                queue2image4hpu = hpu2image4xi['QUEUE']
                message2print = "{(Step 1C/33) got" + "; queue2image4hpu-got-by-wait2image4hpu to: " + str(queue2image4hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z1/70) yielded" + "; hpu2xi-yielded-by-wait2image4hpu: " + str(hpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            size2image4hpu = 0
            try:                    
                size2image4hpu = queue2image4hpu.qsize()
                message2print = "{(Step 1D/33) got" + "; size2image4hpu-got-by-wait2image4hpu to: " + str(size2image4hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
                               
                
            except:     
                message2print = "{(Step 1Z2/70) yielded" + "; hpu2xi-yielded-by-wait2image4hpu: " + str(hpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
             
            empty2image4hpu = True       
            try:                    
                empty2image4hpu= queue2image4hpu.empty()
                message2print = "{(Step 1E/33) got" + "; empty2image4hpu-got-by-wait2image4hpu to: " + str(empty2image4hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z3/70) yielded" + "; empty2image4hpu-yielded-by-wait2image4hpu: " + str(empty2image4hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
             
            full2image4hpu = False       
            try:                                
                full2image4hpu = queue2image4hpu.full()
                message2print = "{(Step 1F/33) got" + "; full2image4hpu-got-by-wait2image4hpu to: " + str(full2image4hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z4/70) yielded" + "; full2image4hpu-yielded-by-wait2image4hpu: " + str(full2image4hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                            
            message2print = "{(Step 1G/33) finished" + "; hpu2image4xi-finished-by-wait2image4hpu to: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                            
        else:
            # -------------------------------------------------------------------------------------
            # USE THE EXISTING IMAGE FOR HPU          
            # -------------------------------------------------------------------------------------     
            
            message2print = "{(Step 1H/33) used" + "; hpu2image4xi-used-by-wait2image4hpu: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4hpu = None
            try:
                queue2image4hpu = hpu2image4xi['QUEUE']
                message2print = "{(Step 1I/33) got" + "; queue2image4hpu-got-by-wait2image4hpu to: " + str(queue2image4hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            except:     
                message2print = "{(Step 1Z5/70) yielded" + "; hpu2xi-yielded-by-wait2image4hpu: " + str(hpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                message2print = "{(Step 1J1/33) passed" + "; task4wait2image-passed-by-wait2image4hpu to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                queue2task = {}    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1J2/33) set" + "; queue2task-set-by-wait2image4hpu to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                except:     
                    message2print = "{(Step 1Z61/70) yielded" + "; queue2task-yielded-by-wait2image4hpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                
                try:                        
                    if queue2task == queue2image4hpu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1K/33) accepted" + "; queue2task-accepted-by-wait2image4hpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        try:
                            message2print = "{(Step 1L/33) warned" + "; queue2task-warned-by-wait2image4hpu: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print) 
                            # 
                        
                            message2print = "{(Step 1M/33) warned" + "; queue2image4hpu-warned-by-wait2image4hpu: " + " ** " + str(queue2image4hpu)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                            # 
                        except:
                            pass
                                                     
                except:     
                    message2print = "{(Step 1Z62/70) yielded" + "; queue2task-yielded-by-wait2image4hpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                   
            
            size2queue4hpu = 0        
            try:                    
                size2queue4hpu = queue2image4hpu.qsize()
                message2print = "{(Step 1N1/33) got" + "; size2queue4hpu-got-by-wait2image4hpu to: " + str(size2queue4hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                hpu2image4xi ['SIZE2QUEUE'] = size2queue4hpu
                message2print = "{(Step 1N2/33) set" + "; size2queue4image-got-by-wait2image4hpu to: " + str(hpu2image4xi ['SIZE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                # 
            except:     
                message2print = "{(Step 1Z7/70) yielded" + "; size2queue4hpu-yielded-by-wait2image4hpu: " + str(size2queue4hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            empty2queue4hpu = False
            try:                    
                empty2queue4hpu = queue2image4hpu.empty()
                message2print = "{(Step 1O1/33) got" + "; empty2queue4hpu-got-by-wait2image4hpu to: " + str(empty2queue4hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                hpu2image4xi ['EMPTY2QUEUE'] = empty2queue4hpu
                message2print = "{(Step 1O2/33) set" + "; empty2queue4image-got-by-wait2image4hpu to: " + str(hpu2image4xi ['EMPTY2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #               
            except:     
                message2print = "{(Step 1Z8/70) yielded" + "; empty2queue4hpu-yielded-by-wait2image4hpu: " + str(empty2queue4hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            full2queue4hpu = False
            try:                                
                full2queue4hpu = queue2image4hpu.full()
                message2print = "{(Step 1P1/33) got" + "; full2queue4hpu-got-by-wait2image4hpu to: " + str(full2queue4hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                hpu2image4xi ['FULL2QUEUE'] = full2queue4hpu
                message2print = "{(Step 1P2/33) set" + "; full2queue4image-got-by-wait2image4hpu to: " + str(hpu2image4xi ['FULL2QUEUE'])  + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)
                #                          
            except:     
                message2print = "{(Step 1Z9/70) yielded" + "; full2queue4hpu-yielded-by-wait2image4hpu: " + str(full2queue4hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            state2queue4hpu = None
            try:                                
                state2queue4hpu = queue2image4hpu.__getstate__()
                message2print = "{(Step 1P3/33) got" + "; state2queue4hpu-got-by-wait2image4hpu to: " + str(state2queue4hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                hpu2image4xi ['STATE2QUEUE'] = state2queue4hpu
                message2print = "{(Step 1P4/33) set" + "; state2queue4image-set-by-wait2image4hpu to: " + str(hpu2image4xi ['STATE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #           
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; state2queue4hpu-yielded-by-wait2image4hpu: " + str(state2queue4hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)               
                
                                                            
            message2print = "{(Step 1Q/33) finished" + "; hpu2image4xi-finished-by-wait2image4hpu: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            hpu2image4hpu = create2image4hpu()
            message2print = "{(Step 1R/33) got" + "; hpu2image4xi-got-by-wait2image4hpu to: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4hpu = None
            try:
                queue2image4hpu = hpu2image4xi['QUEUE']
                message2print = "{(Step 1S/33) got" + "; queue2image4hpu-got-by-wait2image4hpu to: " + str(queue2image4hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; queue2image4hpu-yielded-by-wait2image4hpu: " + str(queue2image4hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE QUEUE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                queue2task = None    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1T/33) passed" + "; queue2task-passed-by-wait2image4hpu to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #
                    
                    if queue2task == queue2image4hpu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1U/33) accepted" + "; queue2task-accepted-by-wait2image4hpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        message2print = "{(Step 1X/33) warned" + "; queue2task-warned-by-wait2image4hpu: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                        
                        message2print = "{(Step 1Y1/33) warned" + "; queue2image4hpu-warned-by-wait2image4hpu: " + " ** " + str(queue2image4hpu)  + " ** " + "; time: " + time4xi() + "}" 
                        print4xi(class2print, method2print, message2print)    
                        #                      
                except:     
                    message2print = "{(Step 1Z11/70) yielded" + "; queue2task-yielded-by-wait2image4hpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
            
            size2image4hpu = 0                             
            try:                    
                size2image4hpu = queue2image4hpu.qsize()
                message2print = "{(Step 1Y2/33) got" + "; size2image4hpu-got-by-wait2image4hpu to: " + str(size2image4hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z12/70) yielded" + "; size2image4hpu-yielded-by-wait2image4hpu: " + str(size2image4hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            empty2image4hpu = True          
            try:                    
                empty2image4hpu = queue2image4hpu.empty()
                message2print = "{(Step 1Y3/33) got" + "; empty2image4hpu-got-by-wait2image4hpu to: " + str(empty2image4hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z13/70) yielded" + "; hpu2xi-yielded-by-wait2image4hpu: " + str(hpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            full2image4hpu = False          
            try:                                
                full2image4hpu= queue2image4hpu.full()
                message2print = "{(Step 1Y4/33) got" + "; full2image4hpu-got-by-wait2image4hpu to: " + str(full2image4hpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z14/70) yielded" + "; full2image4hpu-yielded-by-wait2image4hpu: " + str(full2image4hpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                                        
            message2print = "{(Step 1Y5/33) finished" + "; hpu2image4xi-finished-by-wait2image4hpu: " + str(hpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                    
        except:
            hpu2image4hpu= {}
            message2print = "{(Step 1Z15/70) yielded" + "; hpu2image4xi-yielded-by-wait2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; hpu2image4xi-returning-by-wait2image4hpu: " + str(hpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # {'SIZE2QUEUE': 2147483646, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004420E10>, 
    #  'WIDTH': 800, 
    #  'CONTEXT': ...
    
    return hpu2image4xi
    
   
#</function4xi> 
                                               
# =================================================================================================
#<class4xi class = "HPU2Image4Xi" base = "object" child = "None"
#   synopsis = "offers methods for HPU 2D images" usage = "2D Image4Xi of HPU"
#>

# =================================================================================================
class HPU2Image4Xi(object): 
    ''' 
    HPU2Image4Xi offers methods for HPU 2D images
    '''            

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    classvariable2hpu = cpu_corecount4xi()
    #"
    #/>
        
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = (
            'create2image',                    # ACK 5.31   p76    
            'copy2image',                      # ACK 5.31   p88
            'copy2imagetobuffer',              # ACK 5.3.4  p90
            'copy2buffertoimage',              # ACK 5.3.4  p92
            'read2image',                      # ACK 5.3.3  p84
            'write2image',                     # ACK 5.3.3  p84            
            'map2image',                       # ACK 5.3.5  p94            
            'get2imageinfo',                   # ACK 5.3.6  p97
            'get2supportedimageformats',       # ACK 5.3.2  p82 nb: a separate method to get format whereas using get2imageinfo you can access any element by key such as 'FORMAT'   
            'retain2image',                    # ACK 5.4.1  p99 nb: for compability to OpenCL - you can get any element of dictionary by get2imageinfo
            'release2image',                   # ACK 5.4.1  p99
            'trace2image',                     # ACK modernized 
            'set2imagecallback',               # ACK modernized
            'set2image',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    
            'wait2image',                      # ACK modernized nb: checks Queue object and thereby generates delay   
            'sense',                           # ACK modernized  
            'reason',                          # ACK modernized  
            'ace')                             # ACK modernized           
    #"
    #/>
 
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-HPU2Image4Xi" usage = "
    #   print('(Step 5A/33) OF HPU2IMAGE4XI5TEST - CREATING IMAGE4HPU OBJECT ', time4xi())     
    #   try:
    #       imageobject2hpu = HPU2Image4Xi()
    #       print('(Step 5B/33) OF HPU2IMAGE4XI5TEST - CREATED IMAGE4HPU OBJECT: ', imageobject2hpu)                                  
    #   except:
    #       print('(Step 5Z/33) OF HPU2IMAGE4XI5TEST *** FAILED TO CREATE IMAGE4HPU OBJECT')
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def __init__(self, task4imagecreate2hpu = None):
        '''
        _init__ method initializes HPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------
                    
        class2print = "HPU2Image4Xi"
        method2print = "__init__"
                     
        message2print = "{(Step 1A/33) started" + "; init-of-HPU2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 23:41:37 09/06/11 Pacific Daylight Time 

        message2print = "{(Step 1B/33) borning" + "; hpu2image2xi-of-HPU2Image4Xi-borning: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
                
        message2print = "{(Step 1C/33) inheriting" + "; object-inheriting-by-hpu2image2xi-of-HPU2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'>
        
        message2print = "{(Step 1D/33) sharing" + "; classvariable2hpu-sharing-by-hpu2image2xi-of-HPU2Image4Xi: " + str(HPU2Image4Xi.classvariable2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4             
                                
        message2print = "{(Step 1E/33) passed" + "; task4imagecreate2hpu-passed-to-init-of-HPU2Image4Xi: " + str(task4imagecreate2hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None                            
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)             
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) passing" + "; task4imagecreate2hpu-passing-to-create2image4hpu-by-init-of-HPU2Image4Xi: " + str(task4imagecreate2hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None  
        
        hpu2image2xi = create2image4hpu(task4imagecreate2hpu)
        
        message2print = "{(Step 2B/33) performed" + "; create2image4hpu-performed-by-init-of-HPU2Image4Xi: " + str(create2image4hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <function create2image4hpu at 0x0000000003A9E7C8>
                        
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)             
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3A/33) activated" + "; hpu2image2xi-of-HPU2Image4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # { HPU2Image4Xi : __init__ ; (Step 3A/33) activated; hpu2image2xi-of-HPU2Image4Xi-activated: ; time: 00:14:05 09/07/11 Pacific Daylight Time} 
                
        message2print = "{(Step 3B/33) inherited" + "; object-inherited-by-hpu2image2xi-of-HPU2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'> 
        
        message2print = "{(Step 3C/33) shared" + "; classvariable2hpu-shared-by-hpu2image2xi-of-HPU2Image4Xi: " + str(HPU2Image4Xi.classvariable2hpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4           
       
        message2print = "{(Step 3D/33) returning" + "; init-of-HPU2Image4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 00:14:05 09/07/11 Pacific Daylight Time
        

    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create2image-of-HPU2Image4Xi" usage = "
    #   print('(Step 23A/33) OF HPU2IMAGE4XI5TEST - TRYING TO CREATE HPU IMAGE', time4xi()) 
    #   try:
    #       xi2hpu = HPU2Image4Xi()                            
    #       image2hpu = xi2hpu.create2image()                                  
    #       print('(Step 23B/33) OF HPU2IMAGE4XI5TEST - CREATED HPU IMAGE: ', image2hpu)
    #   except:
    #       print('(Step 23Z/33) OF HPU2IMAGE4XI5TEST *** FAILED TO CREATE HPU IMAGE ***', time4xi())                  
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def create2image(self, task4image2create = None):
        '''
        create2image method creates 2D image objects of HPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "HPU2Image4Xi"
        method2print = "create2image"
                     
        message2print = "{(Step 1A/33) started" + "; create2image-of-HPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4image2create-passed-to-create-of-HPU2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4image2create-passing-to-create2image4hpu-by-create2image-of-HPU2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None      
  
        hpu2image2xi = create2image4hpu(task4image2create)

        message2print = "{(Step 2B/33) performed" + "; create2image4hpu-performed-by-create2image-of-HPU2Image4Xi: " + str(create2image4hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function create2image4hpu at 0x00000000039DF7C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; hpu2image2xi-returning-by-create2image-of-HPU2Image4Xi: " + str(hpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4hpu-failed ; time: 00:53:55 10/05/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4hpu',
        #  'METHOD': 'create2image4hpu'},
        #  'WIDTH': 800 ...

        return hpu2image2xi
                                
    #                        
    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2image-of-HPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF HPU2IMAGE4XI - TRYING TO SET HPU IMAGE')
    #   try:
    #       cc2xi = HPU2Image4Xi()                            
    #       image2hpu = cc2xi.set2image()                                  
    #       print('(Step 1B/33) OF HPU4XI5TEST - SET HPU IMAGE: ', image2hpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF HPU4XI5TEST *** FAILED TO SET HPU IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2image(self, task4set2image = None):
        '''
        sets 2D image HPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "HPU2Image4Xi"
        method2print = "set2image"
                     
        message2print = "{(Step 1A/33) started" + "; set2image-of-HPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4set2image-passed-to-set2image-of-HPU2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE2ERROR': 0,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2image-passing-to-set2image4hpu-by-set2image-of-HPU2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'CODE2ERROR': 0, 
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>, 
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...

        hpu2image2xi = set2image4hpu(task4set2image)

        message2print = "{(Step 2B/33) performed" + "; set2image4hpu-performed-by-set2image-of-HPU2Image4Xi: " + str(set2image4hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function set2image4hpu at 0x0000000003AB36C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; hpu2image2xi-returning-by-set2image-of-HPU2Image4Xi: " + str(hpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4hpu-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4hpu', 'METHOD': 'create2image4hpu'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4hpu-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4hpu',
        #  'METHOD': 'create2buffer4hpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4hpu at 0x0000000003AA81C8>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CC92F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                    
                
        return hpu2image2xi
                                
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2imageinfo-of-HPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF HPU2IMAGE4XI - TRYING TO GET HPU 2D IMAGE')
    #   try:
    #       cc2xi = HPU2Image4Xi()                            
    #       image2hpu = cc2xi.get2imageinfo()                                  
    #       print('(Step 1B/33) OF HPU4XI5TEST - GOT HPU IMAGE: ', image2hpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF HPU4XI5TEST *** FAILED TO GET HPU 2D IMAGE ***')   
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2imageinfo(self, task4getinfo2image=None):
        '''
        get2imageinfo method gets 2D objects of HPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "HPU2Image4Xi"
        method2print = "get2imageinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2imageinfo-of-HPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        hpu2image2xi = get2imageinfo4hpu(task4getinfo2image)
        
        message2print = "{(Step 2A/33) returning" + "; hpu2image2xi-returning-by-get2imageinfo-of-HPU2Image4Xi: " + str(hpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4hpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4hpu', 'METHOD': 'create2image4hpu'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4hpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4hpu',
        #  'METHOD': 'create2buffer4hpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4hpu at 0x00000000039D8348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000037D32F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        # or when only two elemets required
        # {'WIDTH': 800, 'HEIGHT': 800}
        
        return hpu2image2xi
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2supportedimageformats-of-HPU2Image4Xi" usage = "
    #  print('(Step 6B1/33) OF HPU2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR HPU')
    #  try:
    #      cc2xi = HPU2Image4Xi()                            
    #      supportedimageformats2hpu = cc2xi.get2supportedimageformats()                                  
    #      print('(Step 6B2/33) OF HPU4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR HPU: ', supportedimageformats2hpu)
    #                                                   
    #  except:
    #      print('(Step 2Z/33) OF HPU4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR HPU***')     
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2supportedimageformats(self):
        '''
        get2supportedimageformats method gets supported imageformats by objects of HPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "HPU2Image4Xi"
        method2print = "get2supportedimageformats"
                     
        message2print = "{(Step 1A/33) started" + "; get2supportedimageformats-of-HPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:43:00 09/07/11 Pacific Daylight Time

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        supportedimageformats2hpu = get2supportedimageformats4hpu()
        
        message2print = "{(Step 2A/33) returning" + "; supportedimageformats2hpu-returning-by-get2imageinfo-of-HPU2Image4Xi: " + str(supportedimageformats2hpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        
        return supportedimageformats2hpu
                            
    #</methodize4xi>    
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "read2image-of-HPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF HPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi()) 
    #   response4read2hpu = {}
    #   try:
    #       xi2hpu = HPU2Image4Xi()    
    #       response4read2hpu = xi2hpu.read2image()
    #       print('(Step 127B/700) OF HPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH READ IMAGE OBJECT, RESPONSE4READ2HPU: ', response4read2hpu) 
    #   except:
    #       print('(Step 127Z1/700) OF HPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi())
    #   ... continue as with read2image4hpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def read2image(self, task4read2image = None):
        '''
        reads HPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "HPU2Image4Xi"
        method2print = "read2image"
                     
        message2print = "{(Step 1A/33) started" + "; read2image-of-HPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:20:02 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4read2image-passed-to-read2image-of-HPU2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4read2image-passing-to-read2image4hpu-by-read2image-of-HPU2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4readimage2hpu = read2image4hpu(task4read2image)

        message2print = "{(Step 2B/33) performed" + "; read2image4hpu-performed-by-read2image-of-HPU2Image4Xi: " + str(read2image4hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function read2image4hpu at 0x000000000385DA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4readimage2hpu-returning-by-read2image-of-HPU2Image4Xi: " + str(response4readimage2hpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000417DD68>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 936>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 932>}; time: 00:20:02 10/05/11 Pacific Daylight Time}
  
        return response4readimage2hpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "map2image-of-HPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF HPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi()) 
    #   xi2hpu = HPU2Image4Xi() 
    #   ...
    #   response4map2hpu = {}
    #   try:
    #       response4map2hpu = xi2hpu.map2image()
    #       print('(Step 127B/700) OF HPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH MAP IMAGE OBJECT, RESPONSE4READ2HPU: ', response4map2hpu) 
    #   except:
    #       print('(Step 127Z1/700) OF HPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi())
    #   ... continue as with map2image4hpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def map2image(self, task4map2image = None):
        '''
        maps HPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "HPU2Image4Xi"
        method2print = "map2image"
                     
        message2print = "{(Step 1A/33) started" + "; map2image-of-HPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4map2image-passed-to-map2image-of-HPU2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4map2image-passing-to-map2image4hpu-by-map2image-of-HPU2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4mapimage2hpu = map2image4hpu(task4map2image)

        message2print = "{(Step 2B/33) performed" + "; map2image4hpu-performed-by-map2image-of-HPU2Image4Xi: " + str(map2image4hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function map2image4hpu at 0x000000000390BAC8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4mapimage2hpu-returning-by-map2image-of-HPU2Image4Xi: " + str(response4mapimage2hpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x00000000050E9C18>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1100>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1096>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
        return response4mapimage2hpu
                                                           
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "release2image-of-HPU2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RELEASE 2D IMAGE BY DECREMENTING REFERENCE COUNT WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4release2image  = {} 
    #   task4release2image  ['COUNT2REFERENCE'] = 1 
    #   response4release2image = {}
    #   print('(Step 133A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RELEASE 2D IMAGE OF HPU DEVICE ', time4xi())    
    # 
    #   try:                               
    #       response4release2Dimage = xi2hpu.release2image(task4release2image)                                                                                             
    #       print('(Step 133B/700) OF HPU2IMAGE4XI5TEST - RELEASED 2D IMAGE OF HPU DEVICE ', time4xi())                           
    #       keys2response = response4release2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4release2Dimage[key2response])
    #           #
    #   except:
    #       print('(Step 133Z1/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF HPU DEVICE ', time4xi())  
    #   
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 133C/700) OF HPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF HPU DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2hpu.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 133D/700) OF HPU2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF HPU DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 133Z2/700) OF HPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF HPU DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def release2image(self, task4release2image = None):
        '''
        releases HPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "HPU2Image4Xi"
        method2print = "release2image"
                     
        message2print = "{(Step 1A/33) started" + "; release2image-of-HPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4release2image-passed-to-release2image-of-HPU2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4release2image-passing-to-release2image4hpu-by-release2image-of-HPU2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        hpu2image2xi = release2image4hpu(task4release2image)

        message2print = "{(Step 2B/33) performed" + "; release2image4hpu-performed-by-release2image-of-HPU2Image4Xi: " + str(release2image4hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; hpu2image2xi-returning-by-release2image-of-HPU2Image4Xi: " + str(hpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #                          
                
        return hpu2image2xi
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "retain2image-of-HPU2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RETAIN 2D IMAGE BY DECREASING REFERENCE WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4retain2image  = {}
    #   task4retain2image  ['COUNT2REFERENCE'] = 1 
    #   response4retain2Dimage = {}
    #   print('(Step 134A/700) OF HPU2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF HPU DEVICE ', time4xi())   
    #   try:                               
    #       response4retain2Dimage = xi2hpu.retain2image(task4retain2image)                                                                                                 
    #       print('(Step 134B/700) OF HPU2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', time4xi())                         
    #       keys2response = response4retain2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4retain2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z1/700) OF HPU2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF HPU DEVICE ', time4xi())
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 134C/700) OF HPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF HPU DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2hpu.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 134D/700) OF HPU2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF HPU DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z2/700) OF HPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF HPU DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def retain2image(self, task4retain2image = None):
        '''
        retains HPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "HPU2Image4Xi"
        method2print = "retain2image"
                     
        message2print = "{(Step 1A/33) started" + "; retain2image-of-HPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4retain-passed-to-retain2image-of-HPU2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'COUNT2REFERENCE': 1}
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4retain-passing-to-retain2image4hpu-by-retain-of-HPU2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'COUNT2REFERENCE': 1}
  
        hpu2image2xi = retain2image4hpu(task4retain2image)

        message2print = "{(Step 2B/33) performed" + "; retain2image4hpu-performed-by-retain2image-of-HPU2Image4Xi: " + str(retain2image4hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function retain2image4hpu at 0x00000000039DF948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; hpu2image2xi-returning-by-retain2image-of-HPU2Image4Xi: " + str(hpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 
        #  'ACK2RETURN': ' 1D-', 
        #  'ROWPITCH': 800, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4hpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2image4hpu', 
        #  'METHOD': 'create2image4hpu'}, 
        #  'WIDTH': 800, 
        #  'BUFFER': {'CODE2ERROR': 0, 
        #  ...
        #  'COUNT2REFERENCE': 1,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                         
                
        return hpu2image2xi
                                                           
    #</methodize4xi>   
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "wait2image-of-HPU2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 126A/700) OF HPU2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE OPERATION FOR HPU ', time4xi()) 
    #   print('=' *33)      
    #   wait2hpu = {}
    #   try:
    #       response4wait2Dimage = xi2hpu.wait2image()
    #       print('(Step 122B/700) OF HPU2IMAGE4XI5TEST - WAITED 2D IMAGE OPERATION FOR HPU ', time4xi()) 
    #       keys2response = response4wait2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4wait2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 126Z1/700) OF HPU2IMAGE4XI5TEST *** FAILED TO 2D IMAGE OPERATION FOR HPU ', time4xi())    
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def wait2image(self, task4wait2image = None):
        '''
        waits HPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "HPU2Image4Xi"
        method2print = "wait2image"
                     
        message2print = "{(Step 1A/33) started" + "; wait2image-of-HPU2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4wait2image-passed-to-wait2image-of-HPU2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4wait-passing-to-wait2image4hpu-by-wait2image-of-HPU2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        hpu2image2xi = wait2image4hpu(task4wait2image)

        message2print = "{(Step 2B/33) performed" + "; wait2image4hpu-performed-by-wait2image-of-HPU2Image4Xi: " + str(wait2image4hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # semaphore and lock analysis by multiprocessing4xi
        # <function wait2image4hpu at 0x00000000039DFA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; hpu2image2xi-returning-by-wait2image-of-HPU2Image4Xi: " + str(hpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'SIZE2QUEUE': 2147483646,
        #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'},
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000434DDA0>,
        #  'WIDTH': 800,
        #  'CONTEXT': ...
                
        return hpu2image2xi
                                                           
    #</methodize4xi> 
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "write2image-of-HPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF HPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4write2hpu = {}
    #   try:
    #       response4write2hpu = xi2hpu.write2image()
    #       print('(Step 127B/700) OF HPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2HPU: ', response4write2hpu) 
    #   except:
    #       print('(Step 127Z1/700) OF HPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with write2image4hpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def write2image(self, task4write2image = None):
        '''
        writes HPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "HPU2Image4Xi"
        method2print = "write2image"
                     
        message2print = "{(Step 1A/33) started" + "; write2image-of-HPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 02:45:31 10/05/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4write2image-passed-to-write2image-of-HPU2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4write2image-passing-to-write2image4hpu-by-write2image-of-HPU2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4writeimage2hpu = write2image4hpu(task4write2image)

        message2print = "{(Step 2B/33) performed" + "; write2image4hpu-performed-by-write2image-of-HPU2Image4Xi: " + str(write2image4hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function write2image4hpu at 0x000000000380AEC8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4writeimage2hpu-returning-by-write2image-of-HPU2Image4Xi: " + str(response4writeimage2hpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004D10E10>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 968>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 964>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

                     
                
        return response4writeimage2hpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2image-of-HPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF HPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4copy2hpu = {}
    #   try:
    #       response4copy2hpu = xi2hpu.copy2image()
    #       print('(Step 127B/700) OF HPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2HPU: ', response4copy2hpu) 
    #   except:
    #       print('(Step 127Z1/700) OF HPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with copy2image4hpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2image(self, task4copy2image = None):
        '''
        copies HPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "HPU2Image4Xi"
        method2print = "copy2image"
                     
        message2print = "{(Step 1A/33) started" + "; copy2image-of-HPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2image-passed-to-copy2image-of-HPU2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2image-passing-to-copy2image4hpu-by-copy2image-of-HPU2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None 
  
        response4copyimage2hpu = copy2image4hpu(task4copy2image)

        message2print = "{(Step 2B/33) performed" + "; copy2image4hpu-performed-by-copy2image-of-HPU2Image4Xi: " + str(copy2image4hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2image4hpu at 0x000000000390B8C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2hpu-returning-by-copy2image-of-HPU2Image4Xi: " + str(response4copyimage2hpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004EC4128>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1000>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 996>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
        
        return response4copyimage2hpu
                                                           
    #</methodize4xi>
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2imagetobuffer-of-HPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF HPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER OBJECT', time4xi()) 
    #   response4copyimagetobuffer = {}
    #   try:
    #       response4copyimagetobuffer = xi2hpu.copy2imagetobuffer()
    #       print('(Step 127B/700) OF HPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER  OBJECT, RESPONSE4WRITE2HPU: ', response4copyimagetobuffer) 
    #   except:
    #       print('(Step 127Z1/700) OF HPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER ', time4xi()) 
    #
    #   ... continue as with copy2imagetobuffer4hpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2imagetobuffer(self, task4copy2imagetobuffer = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "HPU2Image4Xi"
        method2print = "copy2imagetobuffer"
                     
        message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer-of-HPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2imagetobuffer-passed-to-copy2imagetobuffer-of-HPU2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2imagetobuffer-passing-to-copy2imagetobuffer4hpu-by-copy2imagetobuffer-of-HPU2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2hpu = copy2imagetobuffer4hpu(task4copy2imagetobuffer)

        message2print = "{(Step 2B/33) performed" + "; copy2imagetobuffer4hpu-performed-by-copy2imagetobuffer-of-HPU2Image4Xi: " + str(copy2imagetobuffer4hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2imagetobuffer4hpu at 0x000000000390B948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2hpu-returning-by-copy2imagetobuffer-of-HPU2Image4Xi: " + str(response4copyimage2hpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004FE5FD0>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1036>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1032>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
          
        return response4copyimage2hpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2buffertoimage-of-HPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF HPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE OBJECT', time4xi()) 
    #   response4copybuffertoimage = {}
    #   try:
    #       response4copybuffertoimage = xi2hpu.copy2buffertoimage()
    #       print('(Step 127B/700) OF HPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE  OBJECT, RESPONSE4WRITE2HPU: ', response4copybuffertoimage) 
    #   except:
    #       print('(Step 127Z1/700) OF HPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE ', time4xi()) 
    #
    #   ... continue as with copy2buffertoimage4hpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2buffertoimage(self, task4copy2buffertoimage = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "HPU2Image4Xi"
        method2print = "copy2buffertoimage"
                     
        message2print = "{(Step 1A/33) started" + "; copy2buffertoimage-of-HPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2buffertoimage-passed-to-copy2buffertoimage-of-HPU2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2buffertoimage-passing-to-copy2buffertoimage4hpu-by-copy2buffertoimage-of-HPU2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2hpu = copy2buffertoimage4hpu(task4copy2buffertoimage)

        message2print = "{(Step 2B/33) performed" + "; copy2buffertoimage4hpu-performed-by-copy2buffertoimage-of-HPU2Image4Xi: " + str(copy2buffertoimage4hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2buffertoimage4hpu at 0x000000000390B9C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2hpu-returning-by-copy2buffertoimage-of-HPU2Image4Xi: " + str(response4copyimage2hpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000505A6D8>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1068>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1064>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

                
        return response4copyimage2hpu
                                                           
    #</methodize4xi>    
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "trace2image-of-HPU2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 15A/25) OF HPU2IMAGE4XI5TEST - TRYING TO TRACE HPU: ', time4xi())      
    #   print('=' *33)      
    #   trace2hpu = None
    #   try:
    #       trace2hpu = trace2image4hpu() 
    #       print('(Step 15B/25) OF HPU2IMAGE4XI5TEST - TRACE2HPU: ', trace2hpu)  
    #       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4hpu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4hpu', 'METHOD': 'trace2image4hpu'}
    #    except:
    #       print('(Step 15Z1/25) OF HPU2IMAGE4XI5TEST *** FAILED TO TRACE HPU: ', trace2hpu)     
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def trace2image(self, task4trace2image = None):
        '''
        traces HPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "HPU2Image4Xi"
        method2print = "trace2image"
                     
        message2print = "{(Step 1A/33) started" + "; trace2image-of-HPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4trace2image-passed-to-trace-of-HPU2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None   
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4trace2image-passing-to-trace2image4hpu-by-trace2image-of-HPU2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None  
  
        trace2xi = trace2image4hpu(task4trace2image)

        message2print = "{(Step 2B/33) performed" + "; trace2image4hpu-performed-by-trace2image-of-HPU2Image4Xi: " + str(trace2image4hpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function trace2image4hpu at 0x00000000039DF348>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace2xi-returning-by-trace2image-of-HPU2Image4Xi: " + str(trace2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4hpu-failed {}; time: 01:05:20 09/08/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4hpu',
        #  'METHOD': 'trace2image4hpu'}; time: 01:05:20 09/08/11 Pacific Daylight Time}
                 
                
        return trace2xi
                                                           
    #</methodize4xi>       


    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2imagecallback-of-HPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF HPU2IMAGE4XI - TRYING TO SET HPU IMAGE')
    #   try:
    #       cc2xi = HPU2Image4Xi()                            
    #       image2hpu = cc2xi.set2imagecallback()                                  
    #       print('(Step 1B/33) OF HPU4XI5TEST - SET HPU IMAGE: ', image2hpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF HPU4XI5TEST *** FAILED TO SET HPU IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2imagecallback(self, task4set2imagecallback = None): 
        '''
        set2imagecallback of HPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "HPU2Image4Xi"
        method2print = "set2imagecallback"
                     
        message2print = "{(Step 1A/33) started" + "; set2imagecallback-of-HPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4set2imagecallback-passed-to-set2imagecallback-of-HPU2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None    
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2imagecallback-passing-to-set2imagecallback -by-set2imagecallback-of-HPU2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
        
        callback2hpu4image = {} 
        try:
            callback2hpu4image = set2imagecallback4hpu (task4set2imagecallback)

            message2print = "{(Step 2B/33) callback2hpu4image" + "; callback2hpu4image-set-by-set2imagecallback-of-HPU2Image4Xi to: " + str(callback2hpu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # <function trace2image4hpu at 0x00000000039DF348>
        except:
            message2print = "{(Step 4A/33) failed" + "; callback2hpu4image-failed-by-set2imagecallback-of-HPU2Image4Xi: " + str(callback2hpu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                            
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; callback2hpu4image-returning-by-set2imagecallback-of-HPU2Image4Xi: " + str(callback2hpu4image) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4hpu at 0x0000000003AB31C8> 
                                             
        return callback2hpu4image
                                                           
    #</methodize4xi>     
    
                                                              
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-HPU2Image4Xi" usage = "
    #   try:
    #       print(xi2hpu.sense())
    #       print('(Step 24B/33) OF HPU2IMAGE4XI5TEST - SENSED ', time4xi()) 
    #   except:
    #       print('(Step 24Z/33) OF HPU2IMAGE4XI5TEST *** FAILED TO SENSE ', time4xi())  
    # ">
    
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses HPU2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'HPU2Image4Xi'
        method2print = 'sense'
        
                     
        message2print = "{(Step 1A/33) started" + "; sense-of-HPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        
        message2print = "{(Step 2A/33) sensing" + "; HPU2Image4Xi-sensing-by-sense-of-HPU2Image4Xi: " + str(HPU2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'hpu2image4xi.HPU2Image4Xi'>         
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3A/33) got" + "; kb2xi-got-by-sense-of-HPU2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object': <hpu2image4xi.HPU2Image4Xi object at 0x0000000004188400>}   
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['classvariable2hpu-of-HPU2Image4Xi'] = HPU2Image4Xi.classvariable2hpu
        except:
            message2print = "{(Step 4A/33) yielded" + "; kb2xi-yielded-by-sense-of-HPU2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
      
        message2print = "{(Step 5A/33) sensed" + "; HPU2Image4Xi-sensed-by-sense-of-HPU2Image4Xi: " + str(HPU2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # <class 'hpu2image4xi.HPU2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        
        message2print = "{(Step 6A/33) returning" + "; kb2xi-returning-by-sense-of-HPU2Image4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object': <hpu2image4xi.HPU2Image4Xi object at 0x0000000004188400>, 'classvariable2hpu-of-HPU2Image4Xi': 4}             
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-HPU2Image4Xi" usage = "
    #   print('(Step 25A/33) OF HPU2IMAGE4XI5TEST - REASONING ', time4xi())
    #   try:
    #       print(xi2hpu.reason())
    #       print('(Step 25B/33) OF HPU2IMAGE4XI5TEST - REASONED ', time4xi())
    #   except:
    #       print('(Step 25Z/33) OF HPU2IMAGE4XI5TEST *** FAILED TO REASON ', time4xi())   
    #">
    
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons HPU2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'HPU2Image4Xi'
        method2print = 'reason'

                     
        message2print = "{(Step 1A/33) started" + "; reason-of-HPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) reasoning" + "; HPU2Image4Xi-reasoning-by-reason-of-HPU2Image4Xi: " + str(HPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <class 'hpu2image4xi.HPU2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{(Step 3A/33) got" + "; kb4xi-got-by-reason-of-HPU2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object-reason-object4xi': <hpu2image4xi.HPU2Image4Xi object at 0x0000000004188400>}      
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        try:
            kb4xi['time-reason-of-HPU2Image4Xi'] = time4xi() #adding time
        except:
            message2print = "{(Step 4Z/33) yielded" + "; kb4xi-yielded-by-reason-of-HPU2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
          
        
        message2print = "{(Step 5A/33) reasoned" + "; HPU2Image4Xi-reasoned-by-reason-of-HPU2Image4Xi: " + str(HPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # <class 'hpu2image4xi.HPU2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        
        message2print = "{(Step 6A/33) returning" + "; kb4xi-returning-by-reason-of-HPU2Image4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object-reason-object4xi': <hpu2image4xi.HPU2Image4Xi object at 0x0000000004188400>, 'time-reason-of-HPU2Image4Xi': '01:02:22 09/07/11 Pacific Daylight Time'}                
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-HPU2Image4Xi" usage = "
    #   print('(Step 26A/33) OF HPU2IMAGE4XI5TEST - ACING ', time4xi())   
    #   try:
    #       print(xi2hpu.ace())
    #       print('(Step 26B/33) OF HPU2IMAGE4XI5TEST - ACED ', time4xi())                                                 
    #   except:
    #       print('(Step 26Z/33) OF HPU2IMAGE4XI5TEST *** FAILED TO ACE ', time4xi())                                       
    #    
    #">
    
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces HPU2Image4Xi class objects 
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------       
        
        
        class2print = 'HPU2Image4Xi'
        method2print = 'ace'
        
                     
        message2print = "{(Step 1A/33) started" + "; ace-of-HPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) acing" + "; HPU2Image4Xi-acing-by-ace-of-HPU2Image4Xi: " + str(HPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'hpu2image4xi.HPU2Image4Xi'>           
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3A/33) got" + "; kb8xi-got-by-ace-of-HPU2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object8xi': <hpu2image4xi.HPU2Image4Xi object at 0x0000000004188400>}   


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        try:
             kb8xi['cores-of-HPU2Image4Xi'] = HPU2Image4Xi.classvariable2hpu
        except:
             message2print = "{(Step 4A/33) yielded" + "; kb8xi-yielded-by-ace-of-HPU2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
             print4xi(class2print, method2print, message2print) 
  
        message2print = "{(Step 5A/33) aced" + "; HPU2Image4Xi-aced-by-ace-of-HPU2Image4Xi: " + str(HPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'hpu2image4xi.HPU2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 6A/33) returning" + "; kb8xi-returning-by-ace-of-HPU2Image4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'cores-of-HPU2Image4Xi': 4, 'object8xi': <hpu2image4xi.HPU2Image4Xi object at 0x0000000004188400>}
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   hpu2image4xi5test.txt
#   sense-hpu2image4xi5test.xml
#   reason-hpu2image4xi5test.xml
#   ace-hpu2image4xi5test.xml
#"
#>

#<acknowledge4xi acknowledgement = "33" tester = "Small & Smart"/>

# -------------------------------------------------------------------------------------------------
# to-be-done                              
# -------------------------------------------------------------------------------------------------    
#<do4xi do = "
# nothing
#"
#/>

#<note4xi note = "
# parse4xi.py - to parse this module
# "/>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of hpu2image4xi.py                              
# ------------------------------------------------------------------------------------------------- 
#"
#/>
